PROMPT -- CREO TABLA D7_TAREAS --;
CREATE TABLE DUMBO.D7_TAREAS
(
  TAR_SIS_COD        VARCHAR2(3 BYTE)           NOT NULL,
  TAR_COD            NUMBER(8)                  NOT NULL,
  TAR_TIP_COD        VARCHAR2(1 BYTE)           NOT NULL,
  TAR_ASUNTO         VARCHAR2(70 BYTE)          NOT NULL,
  TAR_USU_COD        VARCHAR2(8 BYTE)           NOT NULL,
  TAR_CAT_COD        NUMBER(3),
  TAR_PRI_COD        VARCHAR2(1 BYTE),
  TAR_APL_COD        VARCHAR2(10 BYTE),
  TAR_INF_COD        VARCHAR2(8 BYTE),
  TAR_FECCREA        DATE,
  TAR_ESTADO         VARCHAR2(1 BYTE),
  TAR_DESCRIP        VARCHAR2(4000 BYTE),
  TAR_URGENCIA       VARCHAR2(1 BYTE),
  TAR_FECMAX_RESOL   DATE,
  TAR_UBICACION      VARCHAR2(500 BYTE),
  TAR_EMAIL_COPIA    VARCHAR2(240 BYTE),
  TAR_FECFIN         DATE,
  TAR_PLANIF         DATE,
  TAR_DESCRIP_SEGUI  VARCHAR2(4000 BYTE),
  TAR_PADRE_TAR_COD  NUMBER(8),
  TAR_HORAS_RES      NUMBER(7,2),
  TAR_COSTE_ECO      NUMBER(8,2),
  TAR_COSTE_MAT      NUMBER(8,2),
  TAR_TELEFONO       VARCHAR2(20),
  TAR_EDUBICACION    VARCHAR2(4 BYTE),
  CONSTRAINT AVCON_822099_TAR_E_000 CHECK (TAR_ESTADO IN ('P', 'A', 'W', 'N', 'R', 'X','F')),
  CONSTRAINT AVCON_822099_TAR_U_000 CHECK (TAR_URGENCIA IN ('N', 'U')),
  CONSTRAINT PK_D7_TAREAS PRIMARY KEY (TAR_SIS_COD, TAR_COD)
    USING INDEX 
    TABLESPACE DUMBO
    PCTFREE    10
    INITRANS   2
    MAXTRANS   255
    STORAGE    (
                INITIAL          40K
                NEXT             40K
                MINEXTENTS       1
                MAXEXTENTS       505
                PCTINCREASE      50
                FREELISTS        1
                FREELIST GROUPS  1
               ),
  CONSTRAINT D7_TAREAS_D7_CATEGORIAS_FK FOREIGN KEY (TAR_SIS_COD, TAR_CAT_COD) 
    REFERENCES DUMBO.D7_CATEGORIAS (CAT_SIS_COD,CAT_COD),
  CONSTRAINT FK_TAREAS_APLICACIONES FOREIGN KEY (TAR_SIS_COD, TAR_APL_COD) 
    REFERENCES DUMBO.D7_APLICACIONES (APL_SIS_COD,APL_COD),
  CONSTRAINT FK_TAREAS_INFORMATICOS FOREIGN KEY (TAR_SIS_COD, TAR_INF_COD) 
    REFERENCES DUMBO.D7_INFORMATICOS (INF_SIS_COD,INF_COD),
  CONSTRAINT FK_TAREAS_PRIORIDADES FOREIGN KEY (TAR_SIS_COD, TAR_PRI_COD) 
    REFERENCES DUMBO.D7_PRIORIDADES (PRI_SIS_COD,PRI_COD),
  CONSTRAINT FK_TAREAS_SISTEMAS FOREIGN KEY (TAR_SIS_COD) 
    REFERENCES DUMBO.D7_SISTEMAS (SIS_COD),
  CONSTRAINT FK_TAREAS_TAREAS FOREIGN KEY (TAR_SIS_COD, TAR_PADRE_TAR_COD) 
    REFERENCES DUMBO.D7_TAREAS (TAR_SIS_COD,TAR_COD),
  CONSTRAINT FK_TAREAS_TIPOS_TAREAS FOREIGN KEY (TAR_SIS_COD, TAR_TIP_COD) 
    REFERENCES DUMBO.D7_TIPOS_TAREAS (TIP_SIS_COD,TIP_COD)
)
TABLESPACE DUMBO
PCTUSED    40
PCTFREE    10
INITRANS   1
MAXTRANS   255
STORAGE    (
            INITIAL          40K
            NEXT             40K
            MINEXTENTS       1
            MAXEXTENTS       505
            PCTINCREASE      50
            FREELISTS        1
            FREELIST GROUPS  1
            BUFFER_POOL      DEFAULT
           )
LOGGING 
NOCACHE
NOPARALLEL;

COMMENT ON TABLE DUMBO.D7_TAREAS IS 'Tabla que recoge las tareas solicitadas por DUMBO';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_COD IS 'Cdigo identificativo de la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_TIP_COD IS 'Cdigo identificativo del tipo de tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_ASUNTO IS 'Asunto vinculado a la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_USU_COD IS 'Cdigo identificativo del usuario que solicita la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_APL_COD IS 'Cdigo identificativo de la aplicacin a la que esta vinculada la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_INF_COD IS 'Cdigo identificativo del informtico al que se le asigna la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_FECCREA IS 'Fecha en la que se ha registrado en el sistema la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_ESTADO IS 'Estado en el que se encuentra la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_DESCRIP IS 'Texto tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_URGENCIA IS 'Ugencia del usuario que solicita la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_FECMAX_RESOL IS 'fecha mxima resolucin tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_UBICACION IS 'ubicacin en la que personarse para resolver la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_EMAIL_COPIA IS 'e-mail a los que se ha enviado copia de notificacin de la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_FECFIN IS 'Fecha en la que se ha dado por finalizada la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_PLANIF IS 'Fecha mxima de resolucin en tarea con prioridad planificada';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_DESCRIP_SEGUI IS 'Texto actual seguimiento tarea para informacin usuario';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_PADRE_TAR_COD IS 'Cdigo identificativo tarea padre';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_HORAS_RES IS 'Tiempo real  de resolucin de tarea en horas';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_COSTE_ECO IS 'coste econmico de resolucin de tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_COSTE_MAT IS 'coste material de resolucin de tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_TELEFONO IS 'Telfono de contacto';

COMMENT ON COLUMN DUMBO.D7_TAREAS.TAR_EDUBICACION IS 'Ubicacin por edificios';

GRANT DELETE, INSERT, SELECT, UPDATE ON  DUMBO.D7_TAREAS TO JV_DUMBO;
