PROMPT -- CREO TABLA D7_TAREAS_EXTERNAS --;
CREATE TABLE DUMBO.D7_TAREAS_EXTERNAS
(
  TAREXT_ID           VARCHAR2(100 BYTE)        NOT NULL,
  TAREXT_TITULO       VARCHAR2(200 BYTE)        NOT NULL,
  TAREXT_ACTIVO       VARCHAR2(1 BYTE)          NOT NULL,
  TAREXT_SIS_COD      VARCHAR2(3 BYTE)          NOT NULL,
  TAREXT_TIP_COD      VARCHAR2(1 BYTE),
  TAREXT_ASUNTO       VARCHAR2(70 BYTE),
  TAREXT_USU_COD      VARCHAR2(60 BYTE),
  TAREXT_APL_COD      VARCHAR2(10 BYTE)         NOT NULL,
  TAREXT_INF_COD      VARCHAR2(8 BYTE),
  TAREXT_DESCRIP      VARCHAR2(4000 BYTE),
  TAREXT_URGENCIA     VARCHAR2(1 BYTE),
  TAREXT_UBICACION    VARCHAR2(500 BYTE),
  TAREXT_EMAIL_COPIA  VARCHAR2(240 BYTE),
  TAREXT_TELEFONO     VARCHAR2(20 BYTE),
  TAREXT_ANEXOS       VARCHAR2(1 BYTE),
  TAREXT_SUBTITULO    VARCHAR2(4000 BYTE),
  CONSTRAINT AVCON_TAREXT_ACT CHECK (TAREXT_ACTIVO IN ('S', 'N')),
  CONSTRAINT AVCON_TAREXT_ANX CHECK (TAREXT_ANEXOS IN ('S', 'N')),
  CONSTRAINT AVCON_TAREXT_URG CHECK (TAREXT_URGENCIA IN ('N', 'U')),
  CONSTRAINT PK_D7_TAREAS_EXT PRIMARY KEY (TAREXT_ID)
    USING INDEX 
    TABLESPACE DUMBO
    PCTFREE    10
    INITRANS   2
    MAXTRANS   255
    STORAGE    (
                INITIAL          40K
                NEXT             40K
                MINEXTENTS       1
                MAXEXTENTS       505
                PCTINCREASE      50
                FREELISTS        1
                FREELIST GROUPS  1
               ),
  CONSTRAINT FK_TAREASEXT_APLICACIONES FOREIGN KEY (TAREXT_SIS_COD, TAREXT_APL_COD) 
    REFERENCES DUMBO.D7_APLICACIONES (APL_SIS_COD,APL_COD),
  CONSTRAINT FK_TAREASEXT_INFORMATICOS FOREIGN KEY (TAREXT_SIS_COD, TAREXT_INF_COD) 
    REFERENCES DUMBO.D7_INFORMATICOS (INF_SIS_COD,INF_COD),
  CONSTRAINT FK_TAREASEXT_SISTEMAS FOREIGN KEY (TAREXT_SIS_COD) 
    REFERENCES DUMBO.D7_SISTEMAS (SIS_COD),
  CONSTRAINT FK_TAREASEXT_TIPOS_TAREAS FOREIGN KEY (TAREXT_SIS_COD, TAREXT_TIP_COD) 
    REFERENCES DUMBO.D7_TIPOS_TAREAS (TIP_SIS_COD,TIP_COD)
)
TABLESPACE DUMBO
PCTUSED    40
PCTFREE    10
INITRANS   1
MAXTRANS   255
STORAGE    (
            INITIAL          40K
            NEXT             64K
            MINEXTENTS       1
            MAXEXTENTS       505
            PCTINCREASE      50
            FREELISTS        1
            FREELIST GROUPS  1
            BUFFER_POOL      DEFAULT
           )
LOGGING 
NOCACHE
NOPARALLEL;

COMMENT ON TABLE DUMBO.D7_TAREAS_EXTERNAS IS 'Tabla para generar formularios de peticin de tareas externos a Dumbo.';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_ID IS 'Identificador del formulario';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_TITULO IS 'Titulo del formulario';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_SUBTITULO IS 'Subtitulo del formulario';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_ACTIVO IS 'Indica si el formulario est activo';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_SIS_COD IS 'Sistema al que pertenece el formulario';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_TIP_COD IS 'Tipo de tarea por defecto que genera el formulario';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_ASUNTO IS 'Asunto por defecto de la tarea';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_USU_COD IS 'Solicitante si se quiere fijar uno';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_APL_COD IS 'Aplicacin en la que se generan las tareas';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_INF_COD IS 'Informtico si se quiere fijar uno';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_DESCRIP IS 'Descripcin de la tarea por defecto';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_URGENCIA IS 'Urgencia de la tarea si se quiere fijar';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_UBICACION IS 'X para evitar que se solicite la ubicacin';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_EMAIL_COPIA IS 'X para evitar que se solicite email de copia';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_TELEFONO IS 'X para evitar que se solicite un telefono';

COMMENT ON COLUMN DUMBO.D7_TAREAS_EXTERNAS.TAREXT_ANEXOS IS 'S/N si se quiere o no que se permitan anexos';

GRANT DELETE, INSERT, SELECT, UPDATE ON  DUMBO.D7_TAREAS_EXTERNAS TO JV_DUMBO;
