PROMPT -- CREAR TABLA D7_HORARIOS --;
CREATE TABLE DUMBO.D7_HORARIOS
(
  HOR_SIS_COD   VARCHAR2(3 BYTE)                NOT NULL,
  HOR_TUR_COD   NUMBER(3)                       NOT NULL,
  HOR_INF_COD   VARCHAR2(8 BYTE)                NOT NULL,
  HOR_FECHA     DATE                            NOT NULL,
  HOR_TIPO      VARCHAR2(1 BYTE)                NOT NULL,
  HOR_FECMODIF  DATE                            NOT NULL,
  HOR_USU_COD   VARCHAR2(8 BYTE)                NOT NULL,
  HOR_AUSENCIA  VARCHAR2(1 BYTE),
  HOR_MOTIVO    VARCHAR2(240 BYTE),
  CONSTRAINT CHK_HOR_AUSENCIA CHECK (HOR_AUSENCIA IN ('J','N')),
  CONSTRAINT CHK_HOR_TIPO CHECK (HOR_TIPO IN ('O','S','A')),
  CONSTRAINT PK_HORARIOS PRIMARY KEY (HOR_SIS_COD, HOR_TUR_COD, HOR_INF_COD, HOR_FECHA)
    USING INDEX 
    TABLESPACE DUMBO
    PCTFREE    10
    INITRANS   2
    MAXTRANS   255
    STORAGE    (
                INITIAL          40K
                NEXT             40K
                MINEXTENTS       1
                MAXEXTENTS       505
                PCTINCREASE      50
                FREELISTS        1
                FREELIST GROUPS  1
               ),
  CONSTRAINT FK_HORARIOS_INFORMATICOS FOREIGN KEY (HOR_SIS_COD, HOR_INF_COD) 
    REFERENCES DUMBO.D7_INFORMATICOS (INF_SIS_COD,INF_COD),
  CONSTRAINT FK_HORARIOS_SISTEMAS FOREIGN KEY (HOR_SIS_COD) 
    REFERENCES DUMBO.D7_SISTEMAS (SIS_COD),
  CONSTRAINT FK_HORARIOS_TURNOS FOREIGN KEY (HOR_SIS_COD, HOR_TUR_COD) 
    REFERENCES DUMBO.D7_TURNOS (TUR_SIS_COD,TUR_COD)
)
TABLESPACE DUMBO
PCTUSED    40
PCTFREE    10
INITRANS   1
MAXTRANS   255
STORAGE    (
            INITIAL          40K
            NEXT             40K
            MINEXTENTS       1
            MAXEXTENTS       505
            PCTINCREASE      50
            FREELISTS        1
            FREELIST GROUPS  1
            BUFFER_POOL      DEFAULT
           )
LOGGING 
NOCACHE
NOPARALLEL;

COMMENT ON TABLE DUMBO.D7_HORARIOS IS 'tabla que recoge los horarios de los OREJAS';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_SIS_COD IS 'Cdigo identificativo del sistema';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_TUR_COD IS 'cdigo identificativo del turno';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_INF_COD IS 'identificador vlido del informtico';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_FECHA IS 'fecha a la que se refieren los datos';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_TIPO IS 'tipo asignacin: O-original,S-sustitucin, A-Ausencia';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_FECMODIF IS 'fecha de la ltma modificacin del registro';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_USU_COD IS 'usuario que modific por ltima vez el registro';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_AUSENCIA IS 'Indicador que marca si la ausencia es justificada o no (J-justificada)';

COMMENT ON COLUMN DUMBO.D7_HORARIOS.HOR_MOTIVO IS 'Motivo de la ausencia';

GRANT DELETE, INSERT, SELECT, UPDATE ON  DUMBO.D7_HORARIOS TO JV_DUMBO;
