/*
 * 
 *
 */
package dumbo7;

import org.apache.log4j.*;
import org.hibernate.criterion.Expression;

import dumbo7.GestorGenerico;
import dumbo7.solicitudes.GestorSolicitudes;
import dumbo7.utils.*;
import atica.frameweb.utils.*;
import javax.mail.*;
import java.util.*;
import java.io.*;

/**
 * @author Universidad de Murcia - ATICA
 *
 */

public class GestorCorreo extends GeneralManager implements Runnable {
	
	static Logger logger = Logger.getLogger(GestorCorreo.class);
	static String VIGENTE = "V";
	static String TOKEN = ";";
	static String MAIL_REFRESH = "MAIL_REFRESH";
	static String MAIL = "MAIL";
	static String USER = "_USER";
	static String APL = "_APL";
	static String PWD = "_PWD";
	static String NO_SUBJECT_LIKE = "_NO_SUBJECT_LIKE";
	static String NO_TEXT_LIKE = "_NO_TEXT_LIKE";
	static String MAIL_HOST = "MAIL_HOST";
	static String MAIL_PORT = "MAIL_PORT";
	static String MAIL_DEBUG = "MAIL_DEBUG";
	static String DEFAULT_TEXT = "Vea el contenido de los anexos para conocer el contenido de la tarea.";
	
	public String rutaAnexos = "", debug = "N";
	public Dumbo7 srvMail = null;
	public Thread miThread = null;
	
	public GestorCorreo(Dumbo7 servlet) {
		srvMail = servlet;
	}

	public void start() {
		miThread = new Thread(this);
		miThread.start();
	}
	
	public void run() {
		while (true) {
			long refresh = 600000; // 10 minutos por defecto
			org.hibernate.Session conn = null;
			try {
				conn = srvMail.getSession();
				// Cargo el fichero de propiedades
				log("Cargando fichero de propiedades...");
				Properties prop = new Properties();
				prop.load(getClass().getResource("mail.properties").openStream());
				// Recoje si hay Debug
				debug = prop.getProperty(MAIL_DEBUG);
				if (debug==null) debug = "N";
				// Recoge el tiempo de espera
				try { refresh = Long.parseLong(prop.getProperty(MAIL_REFRESH)); } catch (Exception exi) {}
				// Recoge los sistemas en los que lee correo
				StringTokenizer systems = new StringTokenizer(prop.getProperty(MAIL));
				while (systems.hasMoreTokens()) {
					String sys = systems.nextToken();
					log("Leyendo Correos Sistema: "+sys);
					// Recoje el usuario por defecto
					String defaultUser = prop.getProperty(sys+USER);
					log("Usuario Defecto["+sys+"]: "+defaultUser);
					// Creo el usuario
					initUserCtx(conn,sys,defaultUser);
					// Recoge las cuentas de las que lee en ese sistema
					StringTokenizer mails = new StringTokenizer(prop.getProperty(sys));
					while (mails.hasMoreTokens()) {
						String mail = mails.nextToken();
						String apl = prop.getProperty(mail+APL);
						String pwd = prop.getProperty(mail+PWD);
						log("Leyendo Correo: "+mail);
						try {
							Vector msg = readMail(prop,mail,pwd);
							log("Total Mensajes: "+msg.size());
							log("Migrando Mensajes a Dumbo ["+apl+"]...");
							mailsToDumbo(conn,msg,sys,apl);
							log("Mensajes Migrados a Dumbo.");
						} catch (Exception exx) {
							System.out.println("GestorCorreo[read]: "+exx);
						}
					}
				}
			} catch (Exception ex) {
				System.out.println("GestorCorreo[run]: "+ex);
			} finally {
				srvMail.freeSession(conn);
			}
			try {
				log("Proxima lectura en "+refresh+" ms.");
				Thread.sleep(refresh); // Lee los correos cada X minutos
			} catch (Exception ex) {}
		}
	}
	
	public void initUserCtx(org.hibernate.Session conn,String system,String defusr) throws Exception {
		try {
			setUser(new dumbo7.user.DumboUser(conn,defusr,"",system));
			user.getDatosMenu().idSelected = system;
			rutaAnexos = GestorGenerico.createContextPath(user,srvMail);
		} catch (Exception ex) {
			throw new Exception("GestorCorreo[initUserCtx]: "+ex);
		}
	}
	
	private boolean mailPermitido(Properties prop,String mail,Message m) throws Exception {
		String to = "";
		if (m.getRecipients(Message.RecipientType.TO)!=null && m.getRecipients(Message.RecipientType.TO).length>0)
			to = mailFromAddress(m.getRecipients(Message.RecipientType.TO)[0].toString());
		if (to==null || !to.equals(mail)) return false;
		String asunto = m.getSubject().toLowerCase();
		StringTokenizer subject = new StringTokenizer(prop.getProperty(mail+NO_SUBJECT_LIKE),TOKEN);
		while (subject.hasMoreTokens()) {
			if (asunto.indexOf(subject.nextToken(TOKEN).toLowerCase())>=0) return false;
		}
		StringTokenizer text = new StringTokenizer(prop.getProperty(mail+NO_TEXT_LIKE),TOKEN);
		Object obj = m.getContent();
        if (obj instanceof Multipart) {
          permitidoMultipart(text,(Multipart)obj);
        } else {
          permitidoPart(text,m);
        }
		
		return true;
	}
	
	public boolean permitidoMultipart(StringTokenizer st,Multipart multipart) 
    throws MessagingException, IOException {
		for (int i=0, n=multipart.getCount(); i<n; i++) {
			if (!permitidoPart(st,multipart.getBodyPart(i))) return false;
		}
		return true;
	}
	
	public boolean permitidoPart(StringTokenizer st,Part part) 
    throws MessagingException, IOException {
		String disposition = part.getDisposition();
		String contentType = part.getContentType();
		if (disposition == null) { // When just body
			// Check if plain
			String partTxt = ((String)part.getContent()).toLowerCase();
			while (st.hasMoreTokens()) {
				if (partTxt.indexOf(st.nextToken(TOKEN).toLowerCase())>=0) return false;
			}
		}
		return true;
	}
	
	
	public Vector readMail(Properties prop,String mail,String clave) throws Exception {
		Properties props = System.getProperties();
		props.setProperty("mail.pop3.host",prop.getProperty(MAIL_HOST));
		props.setProperty("mail.pop3.user","false");
		props.setProperty("mail.pop3.port",prop.getProperty(MAIL_PORT));

		Vector mensajes = new Vector();
		Session session = Session.getDefaultInstance(props, null);
		URLName urln = new URLName("pop3",prop.getProperty(MAIL_HOST),Integer.parseInt(prop.getProperty(MAIL_PORT)),null,mail,clave);

		try {
			Store store = session.getStore(urln);
			Folder inbox = null;
			store.connect(); // me conecto al servidor
			inbox = store.getFolder("INBOX"); // le digo que saque la carpeta INBOX
			inbox.open(Folder.READ_WRITE);
			FetchProfile profile = new FetchProfile(); // creo un perfil nuevo
			profile.add(FetchProfile.Item.ENVELOPE);
			Message[] messages = inbox.getMessages(); // obtengo las trazas de los mensajes
			inbox.fetch(messages, profile); // finalmente le digo que descargue todos los mensajes de la mejor forma que sepa hacerlo
			
			for (int k=0; k<messages.length; k++) {
				MensajeDumbo md = new MensajeDumbo();
				if (messages[k].getRecipients(Message.RecipientType.TO)!=null && messages[k].getRecipients(Message.RecipientType.TO).length>0)
					md.setTo(messages[k].getRecipients(Message.RecipientType.TO)[0].toString());
				if (mailPermitido(prop,mail,messages[k])) {
					if (messages[k].getFrom()!=null && messages[k].getFrom().length>0)
						md.setFrom(messages[k].getFrom()[0].toString());
					if (messages[k].getReplyTo()!=null && messages[k].getReplyTo().length>0)
						md.setReply(messages[k].getReplyTo()[0].toString());
					md.setAsunto("["+mail+"] "+messages[k].getSubject());
					
					Object obj = messages[k].getContent();
			        if (obj instanceof Multipart) {
			        	md.setTexto(handleMultipart(md,(Multipart)obj));
			        } else {
			        	md.setTexto(handlePart(md,messages[k]));
			        }
			        if (md.getTexto()==null || md.getTexto().equals("")) md.setTexto(DEFAULT_TEXT);
					mensajes.addElement(md);
					log("Mensaje ["+md.getFrom()+"]["+md.getReply()+"]["+md.getAnexos().size()+"]: "+md.getAsunto());
				}
				messages[k].setFlag(Flags.Flag.DELETED,true); // Borro el mensaje
			}
			
			inbox.close(true); // cierro todo
			store.close();
			
		} catch (Exception ex) {
			throw new Exception("GestorCorreo[readMail]: "+ex);
		}
		return mensajes;
	}
	
	public void mailsToDumbo(org.hibernate.Session conn,Vector mensajes,String system,String aplCod) throws Exception {
		try {
			if (mensajes.size()>0) {
				org.hibernate.Transaction tx = conn.beginTransaction();
				Enumeration en = mensajes.elements();
				while (en.hasMoreElements()) {
					MensajeDumbo md = (MensajeDumbo)en.nextElement();
					D7Tareas tar = new D7Tareas();
					tar.setTarSisCod(system);
					tar.setTarTipCod(D7TiposTareas.INCIDENCIA);
					// El from establece el solicitante
					tar.setTarUsuCod(mailFromAddress(md.getFrom()));
					// El usuario debe ser un correo UM vlido o externo
					if (GestorGenerico.isDominioInterno(tar.getTarUsuCod())) {
						String dni = D7Conversor.convierteMail(conn,tar.getTarUsuCod());
						if (dni==null || dni.equals("")) continue;
					}
					tar.setTarAplCod(aplCod);
					String reply = mailFromAddress(md.getReply());
					// Con el reply se puede establecer informtico si es diferente al from
					if (reply!=null && !reply.equals("") && !reply.equals(tar.getTarUsuCod())) {
						String info = mailFromAddress(md.getReply());
						// El informatico debe ser un correo vlido UM
						// Si no es correcto no se establece informtico
						if (GestorGenerico.isDominioInterno(info)) {
							String dni = D7Conversor.convierteMail(conn,info);
							if (dni!=null && !dni.equals("")) tar.setTarInfCod(dni);
							else tar.setTarInfCod("");
						} else tar.setTarInfCod("");
					}
					else tar.setTarInfCod("");
					// El informatico debe ser un informatico vigente del sistema
					if (!tar.getTarInfCod().equals("")) {
						org.hibernate.Criteria ct = conn.createCriteria(D7Informaticos.class);
						ct = ct.add(Expression.eq("InfSisCod",tar.getTarSisCod()));
						ct = ct.add(Expression.eq("InfCod",tar.getTarInfCod()));
						ct = ct.add(Expression.eq("InfEstado",VIGENTE));
						List l = ct.list();
						if (l==null || l.size()!=1) tar.setTarInfCod("");
					}
					tar.setTarAsunto(md.getAsunto());
					tar.setTarDescrip(md.getTexto());
					// Crea el Dumbo
					GestorSolicitudes gs = new GestorSolicitudes();
					gs.setUser(user);
					gs.rutaAnexos = rutaAnexos;
					Enumeration en2 = md.getAnexos().keys();
					while (en2.hasMoreElements()) {
						Hashtable anx = (Hashtable)md.getAnexos().get((String)en2.nextElement());
						gs.readAnexo(anx);
					}
					gs.saveSolicitud(conn,tar);
					
				}
				tx.commit();
			}			
		} catch (Exception ex) {
			throw new Exception("GestorCorreo[mailsToDumbo]: "+ex);
		}
	}
	
	private String mailFromAddress(String txt) {
		if (txt!=null && txt.indexOf("<")>0) {
			// Es del estilo "Nombre <mail@bla>"
			return txt.substring(txt.indexOf("<")+1,txt.indexOf(">"));
		} else return txt;
	}

	public String handleMultipart(MensajeDumbo md,Multipart multipart) 
      throws MessagingException, IOException {
		String txt = "";
		for (int i=0, n=multipart.getCount(); i<n; i++) {
			txt += handlePart(md,multipart.getBodyPart(i));
		}
		return txt;
	}
	
	public String handlePart(MensajeDumbo md,Part part) 
      throws MessagingException, IOException {
		String disposition = part.getDisposition();
		String contentType = part.getContentType();
		if (disposition == null) { // When just body
			// Check if plain
			if ((contentType.length() >= 10) &&	(contentType.toLowerCase().substring(0, 10).equals("text/plain"))) {
				saveFile(md,"Contenido.txt",part);
				return (String)part.getContent();
			} else {
				saveFile(md,"Contenido.html",part);
			}
		} else if (disposition.equalsIgnoreCase(Part.ATTACHMENT)) {
			saveFile(md,part.getFileName(), part.getInputStream());
		} else if (disposition.equalsIgnoreCase(Part.INLINE)) {
			saveFile(md,part.getFileName(), part.getInputStream());
		}
		return "";
	}
	
	public void saveFile(MensajeDumbo md,String filename,InputStream input) throws IOException {
		if (filename == null) filename = "Contenido";
		// Do no overwrite existing file
		String realName = filename;
		File file = new File(rutaAnexos+realName);
		int i=0;
		while (file.exists()) {
			i++; realName = i+"_"+filename;
			file = new File(rutaAnexos+realName);
		}
		FileOutputStream fos = new FileOutputStream(file);
		BufferedOutputStream bos = new BufferedOutputStream(fos);

		BufferedInputStream bis = new BufferedInputStream(input);
		int aByte;
		while ((aByte = bis.read()) != -1) {
			bos.write(aByte);
		}
		bos.flush();
		bos.close();
		bis.close();
		
		md.addAnexo(rutaAnexos,realName);
		
	}

	public void saveFile(MensajeDumbo md,String filename,Part input) throws IOException,MessagingException {
		saveFile(md,filename,input.getInputStream());
	}

	private void log(String st) {
		if (debug!=null && debug.toUpperCase().equals("Y")) System.out.println(st);
	}
	
}
