package dumbo7;

import java.io.*;
import java.util.Calendar;
import java.util.jar.*;
import java.util.zip.*;

/**
 * @author Universidad de Murcia
 *
 * Crea el War de Dumbo automatico.
 * 
 */
public class MakeWar {

	public static String [] copies = new String[]{"lib",
												  "dumbo7\\DUMBO.hibernate.cfg.xml",
												  "dumbo7\\config.properties",
												  "dumbo7\\mail.properties",
												  "dumbo7\\utils\\ldap\\ldap.properties",
												  "dumbo7\\utils\\xml\\utils.xml",
												  "dumbo7\\inicio\\auth.properties",
												  "dumbo7\\user\\access.xml",
												  "dumbo7\\user\\menu.xml",
												  "dumbo7\\remote\\user.properties",
												  "web_dumbo\\WEB-INF\\web.xml",
												  "web_dumbo\\index.html"};
	public static String FILES_PATH = "/../iFiles/";
	
	public static void main(String[] args) {
		try {
			if (args.length!=1) {
				System.out.println("Use: dumbo7.MakeWar <Project Rute>");
			} else {
				MakeWar m = new MakeWar();
				String [] wfile = m.creaWarFile(args[0]);
				m.creaZipFile(wfile);
			}
		} catch (Exception ex) {
			System.out.println("ERROR: "+ex.getMessage());
		}
	}
	
	private String rutaBase = "";
	
	public String [] creaWarFile(String ruta) throws Exception {
		
		String [] jarNames = new String[2];
		
		System.out.println("Creando War File iDumbo ...");
		
		// Ruta de lectura
		if (ruta.endsWith(File.separator)) ruta = ruta.substring(0,ruta.length()-1);
		// Nombre del paquete
		String paqueteBase = ruta.substring(ruta.lastIndexOf(File.separator)+1).toLowerCase();
		String jarName = ruta.substring(0,ruta.lastIndexOf(File.separator)+1)+"dumbo.war";
		// Abro el directorio a compactar
		File base = new File(ruta);
		rutaBase = ruta;
		JarOutputStream out = new JarOutputStream(new BufferedOutputStream(new FileOutputStream(jarName)));
		comprime(null,base,out);
		out.close();
		
		jarNames[0] = jarName;

		System.out.println("Creando War File Forja ...");
		
		jarName = ruta.substring(0,ruta.lastIndexOf(File.separator)+1)+"forja.war";
		// Abro el directorio a compactar
		base = new File(ruta+File.separator+".."+File.separator+".."+File.separator+"forja"+File.separator+"WebContent");
		rutaBase = ruta;
		out = new JarOutputStream(new BufferedOutputStream(new FileOutputStream(jarName)));
		comprime(null,base,out);
		out.close();
		
		jarNames[1] = jarName;
		
		return jarNames;
		
	}

	private String toCadena(int n) {
		if (n<10) return "0"+n;
		else return ""+n;
	}
	
	public void creaZipFile(String [] rutas) throws Exception {
		
		System.out.println("Creando Zip File...");
		Calendar hoy = Calendar.getInstance();
		String zipName = rutas[0].replaceFirst(".war","_"+toCadena(hoy.get(Calendar.DATE))+"-"+toCadena(hoy.get(Calendar.MONTH)+1)+"-"+hoy.get(Calendar.YEAR)+".zip");
		JarOutputStream out = new JarOutputStream(new BufferedOutputStream(new FileOutputStream(zipName)));
		for (int k=0; k<rutas.length; k++) {
			// Abro el war a compactar
			File warFile = new File(rutas[k]);
			//File readmeFile = new File(rutas[k].substring(0,rutas[k].lastIndexOf(File.separator)+1)+File.separator+"iFiles"+FILES_PATH+"Leeme.txt");
			comprime("",warFile,out);
			//comprime("",readmeFile,out);
			warFile.delete();
		}
		out.close();
	}
	
	private boolean excludeDirectory(String nombre) {
		if (nombre.startsWith(".")) return true;
		//if (nombre.endsWith("src")) return true;
		if (nombre.endsWith("estadisticas")) return true;
		if (nombre.endsWith("doc")) return true;
		if (nombre.endsWith("jsp")) return true;
		if (nombre.endsWith("work")) return true;
		if (nombre.endsWith("mano")) return true;
		if (nombre.endsWith("migracion")) return true;
		if (nombre.endsWith("reciclatica")) return true;
		if (nombre.endsWith("dumbo21")) return true;
		if (nombre.endsWith("scriptsUMU")) return true;
		if (nombre.endsWith("rosmiman")) return true;
		if (nombre.endsWith("encuestas")) return true;
		if (nombre.endsWith("contenidos")) return true;
		//if (nombre.endsWith("empresas")) return true;
		//if (nombre.endsWith("geshorario")) return true;
		return false;
	}
	
	private boolean excludeFile(String nombre) {
		if (nombre.startsWith(".")) return true;
		//if (nombre.endsWith(".java")) return true;
		if (nombre.endsWith("MakeWar")) return true;
		if (nombre.endsWith("cewolf.tld")) return true;
		return false;
	}
	
	private File realFile(File org) {
		for (int k=0; k<copies.length; k++) {
			if (org.getAbsolutePath().endsWith(copies[k])) return new File(rutaBase+FILES_PATH+copies[k]);
		}
		return org;
	}
	
	public void comprime(String ruta,File org,JarOutputStream out) throws IOException {
		if (org.isDirectory()) {
			if (excludeDirectory(org.getName())) return;
			if (ruta==null) System.out.println("Packing Directory "+org.getName()+"...");
			else System.out.println("Packing Directory "+ruta+org.getName()+"...");
			File endf = realFile(org);
			File [] lista = endf.listFiles();
			for (int k=0; k<lista.length; k++) {
				if (ruta==null) comprime("",lista[k],out);
				else comprime(ruta+endf.getName()+"/",lista[k],out);
			}
		} else {
			if (excludeFile(org.getName())) return;
			File endf = realFile(org);
			if (ruta==null) System.out.println("Packing File "+endf.getName()+"...");
			else System.out.println("Packing File "+ruta+endf.getName()+"...");
			JarEntry jarAdd = new JarEntry(ruta+endf.getName());
			jarAdd.setTime(endf.lastModified());
			out.putNextEntry(jarAdd);
			// Write file to archive
			FileInputStream in = new FileInputStream(endf);
			byte [] bites = new byte[1024];
			int n = in.read(bites,0,1024);
			while (n==1024) {
				out.write(bites,0,n);
				n = in.read(bites,0,1024);
			}
			if (n>0) out.write(bites,0,n);
			in.close();
			out.closeEntry();
		}
		
	}
	
}
