package dumbo7.administracion.categorias;

import dumbo7.utils.*;
import dumbo7.administracion.*;
import org.hibernate.*;

import java.sql.*;
import java.util.*;
import java.math.BigDecimal;

/**
 * @author prc
 *
 * Gestor de Administracin de Categorias
 * 
 */


public class GestorCategorias extends GestorAdministracion {
	
	public GestorCategorias() {}
	
	//Parametros para el filtrado de categorias segn seccin y tipo de tarea
	private String seccionCode = null;
	private String tipoTareaCode = null;
	private boolean flagFiltrado = false;
	
	protected List categorias=null;
	protected List aplicaciones=null;
	protected List secciones=null;
	protected List tiposTareas=null;
	//Vector donde se guardan las subCategorias de una categoria
	protected Vector subCategorias=null;
	
	private D7Categorias categoria = null;
	private Integer CodigoCategoria = null;
	private String codigoNuevaSeccion = null;//Se usa tanto para crear nuevas categorias como para hacer update
	
	//Mtodos set para el filtrado en index
	public void setFlagFiltrado(String flagFiltrado)
	{
		if(flagFiltrado!=null && flagFiltrado.equals("0")) this.flagFiltrado = false;
		else if(flagFiltrado!=null && flagFiltrado.equals("1")) this.flagFiltrado = true;
		else this.flagFiltrado = false;
	}
	public void setSeccionCode(String seccionCode)
	{
		this.seccionCode = seccionCode;
	}
	public void setTipoTareaCode(String tipoTareaCode)
	{
		this.tipoTareaCode = tipoTareaCode;
	}
	//Lista todas las categoras	
	public void listaCategoriasFiltrado(Session conn)
	{
		Query q = null;
		try
		{
			if(seccionCode==null && tipoTareaCode==null || !flagFiltrado)//Si no se introducen parametros o no hay que filtrar
			{
				q = conn.createQuery(getQuery("AllCategorias"));
			}
			else if(seccionCode!=null && tipoTareaCode==null && flagFiltrado)
			{
				q = conn.createQuery(getQuery("CategoriasSeccion"));
				q = q.setString("sec",seccionCode);
			}
			else if(seccionCode==null && tipoTareaCode!=null && flagFiltrado)
			{
				q = conn.createQuery(getQuery("CategoriasTipoTarea"));
				q = q.setString("tip",tipoTareaCode);
			}
			else
			{
				q = conn.createQuery(getQuery("CategoriasSeccionTipoTarea"));
				q = q.setString("sec",seccionCode);
				q = q.setString("tip",tipoTareaCode);
			}
			q = q.setString("sist",user.getDatosMenu().idSelected());
			categorias  = (List)q.list();
		}
		catch (Exception ex)
		{
			setErrorMessage("GestorCategorias[Lectura de BBDD]: "+((ex.getCause()!=null)?ex.getCause():ex));
		}
	}
	//Lista todas las categoras	
	public void listaCategorias(Session conn) {
		try{
			Query q = conn.createQuery(getQuery("AllCategorias"));
			q = q.setString("sist",user.getDatosMenu().idSelected());
			categorias  = (List)q.list();
		
		} catch (Exception ex) {
			setErrorMessage("GestorCategorias[Lectura de BBDD]: "+((ex.getCause()!=null)?ex.getCause():ex));
		}
	}
	//Lista una categora a partir de su cdigo
	public void listaCategoria(Session conn, D7Categorias cat) {
		try{
			Query q = conn.createQuery(getQuery("UnaCategoria"));
			q = q.setString("sist",user.getDatosMenu().idSelected());
			q = q.setString("cod", "" + cat.getCatCod());
			categorias  = (List)q.list();
		
		} catch (Exception ex) {
			setErrorMessage("GestorCategorias[Lectura de BBDD]: "+((ex.getCause()!=null)?ex.getCause():ex));
		}
	}
	
	//Lista todas las subsecciones de una seccion
	public void listaSubcategorias(Session conn,D7Categorias cat)
	{
		PreparedStatement pstmt;
		subCategorias = new Vector();

		try
		{
			String q = getQuery("subCategorias");
			
			q = q.replaceAll(":sist",user.getDatosMenu().idSelected());
			q = q.replaceAll(":cat", ""+cat.getCatCod());
			pstmt = conn.connection().prepareStatement(q);
			ResultSet rs = pstmt.executeQuery();
			while (rs.next())
			{
				D7Categorias categ = new D7Categorias();
	 	    	//cat_sis_cod, cat_cod, cat_descrip, cat_estado, cat_tip_cod, cat_sec_cod, cat_cat_cod, cat_tiempo_est
	 	    	categ.setCatSisCod(rs.getString(1));
	 	    	categ.setCatCod(new Integer(rs.getString(2)));
	 	    	categ.setCatDescrip(rs.getString(3));
	 	    	categ.setCatEstado(rs.getString(4));
	 	    	categ.setCatTipCod(rs.getString(5));
	 	    	categ.setCatSecCod(rs.getString(6));
	 	    	categ.setCatCatCod(new Integer(rs.getString(7)));
	 	    	if(rs.getString(8)!=null)
	 	    		categ.setCatTiempoEst(new BigDecimal(rs.getString(8)));
	 	    	subCategorias.add(categ);
	 	    }
	 	    rs.close();
	 	    pstmt.close();
		}
		catch (Exception ex)
		{
			System.out.println("Error al listar subcategorias");
			System.out.println(ex.toString());
		}
	}
	//Lista todas las categorias, menos las que se encuentran en el vector subCategorias. Tambin tienen que tener igual tipo tarea y estar vinculadas a la misma seccin
	//Una categoria padre no puede pasar a depender de ninguna de las categorias que dependen de ella directa o indirectamente
	public void listaCategoriasRestriccion(Session conn,D7Categorias cat)
	{
		String cadena = new String();
		//Recogemos los cdigos de las secciones
		for(int i=0;i<subCategorias.size();i++)
		{
			cadena = cadena + "'" + ((D7Categorias)subCategorias.get(i)).getCatCod() + "'" + ",";
		}
		if(cadena.length()!=0)
		{
			cadena = cadena.substring(0,cadena.length()-1);//Eliminamos la ltima coma
			try
			{
				Query q = conn.createQuery(getQuery("CategoriasRestriccion").replaceAll(":categorias","("+cadena+")"));
				q = q.setString("sist",user.getDatosMenu().idSelected());
				q = q.setString("sec",cat.getCatSecCod());
				q = q.setString("tip",cat.getCatTipCod());
				categorias  = (List)q.list();
			}
			catch (Exception ex)
			{
				logger.error(ex.getMessage());user.getDatosMenu().idSelected();
			}
		}
		else
		{
			try
			{
				//Si no tiene subsecciones puede depender de cualquier seccion que tenga igual tipo tarea y seccion
				Query q = conn.createQuery(getQuery("CategoriasSeccionTipoTarea"));
				q = q.setString("sist",user.getDatosMenu().idSelected());
				q = q.setString("sec",cat.getCatSecCod());
				q = q.setString("tip",cat.getCatTipCod());
				categorias  = (List)q.list();
			}
			catch (Exception ex)
			{
				logger.error(ex.getMessage());user.getDatosMenu().idSelected();
			}
		}
	}
	
	//Lista todos los tipos tareas
	public void listaTipoTareas(Session conn){
		try{
			Query q = conn.createQuery(getQuery("TiposTareas"));
			q = q.setString("sist",user.getDatosMenu().idSelected());
			tiposTareas  = (List)q.list();
		
		} catch (Exception ex) {
			setErrorMessage("GestorCategorias[Lectura de BBDD]: "+((ex.getCause()!=null)?ex.getCause():ex));
		}
	}
	
	//Listado de todas las D7Aplicaciones
	public void listaAplicaciones(Session conn)
	{
		try
		{
			Query q = conn.createQuery(getQuery("Aplicaciones"));
			q = q.setString("sist",user.getDatosMenu().idSelected());
			aplicaciones  = (List)q.list();
		}
		catch (Exception ex)
		{
			logger.error(ex.getMessage());
		}
	}
	
	public void listaSecciones(Session conn)
	{
		try
		{
			Query q = conn.createQuery(getQuery("listaSecciones"));
			q = q.setString("sist",user.getDatosMenu().idSelected());
			secciones  = (List)q.list();
			
		}
		catch (Exception ex)
		{
			logger.error(ex.getMessage());
		}
	}	
	
	//Mtodos get para mapeos
	public List getCategorias() { return categorias; }
	public List getAplicaciones(){return aplicaciones;}
	public List getSecciones() { return secciones; }
	public List getTipoTareas() {return tiposTareas; }
	
	
	
	/**CREAS LAS SECUENCIAS DE LOS ARRAYS Y DA DE ALTA LA APLICACION*/
	public void saveCategoria(Session conn,D7Categorias cat) {
		int catCod = 0;
			
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try{
			try {
				String qry = getQuery("NextCodigoCategoria");
				qry = qry.replaceFirst("sistema",user.getDatosMenu().idSelected());
				pstmt = conn.connection().prepareStatement(qry);
				rs = pstmt.executeQuery();
				if (rs.next()) {
					catCod = rs.getInt(1);
				}
				
			} catch (Exception ex) {
				setErrorMessage("GestorCategorias: "+ex);
				//setErrorMessage("Error al crear nueva Categora");
				logger.fatal("Error BD: "+ex);
			
			} finally {
				if (pstmt!=null) pstmt.close();
				if (rs!=null) rs.close();
			}
		}catch(Exception ex) {
			setErrorMessage("GestorCategorias: "+ex);
			logger.fatal("Error BD: "+ex);
			//setErrorMessage("Error al crear nueva Categora");
		}
		
		cat.setCatSisCod(user.getDatosMenu().idSelected());
		cat.setCatCod(new Integer(catCod));
		//Si el codigo es -1, va a ser categoria raiz
		if(cat.getCatCatCod().toString().equals("-1"))
			cat.setCatCatCod(new Integer(catCod));
		CodigoCategoria= new Integer(catCod);
		categoria = cat;
		
		
		try {
			conn.save(categoria);
			conn.flush();
			conn.connection().commit();
			categoria = null;
			setErrorMessage("La categora ha sido creada correctamente");
			
		} catch (Exception ex) {
			setErrorMessage("GestorCategorias: "+ex);
			//setErrorMessage("Error al crear nueva Categora");
		}
	}
	
	
	
	public void updateCategoria(Session conn, D7Categorias cat) {
		List padre;
		D7Categorias padreCat;
	
		CodigoCategoria = cat.getCatCod();
		
		cat.setCatSisCod(user.getDatosMenu().idSelected());
		//Si la categoria no se esta referenciando a si misma, se hace el update simplemente
		//sin hacer la query puesto que esto da problemas por querer almacenar
		//un objeto en base de datos y haberlo sacado de ella anteriormente mediante la query
		try
		{
			if(!(cat.getCatCod().equals(cat.getCatCatCod())))
			{
		
				Query q = conn.createQuery(getQuery("UnaCategoria"));
				q = q.setString("sist",user.getDatosMenu().idSelected());
				q = q.setString("cod",cat.getCatCatCod().toString());
				padre = (List)q.list();
				padreCat = (D7Categorias) padre.get(0);
			}
			
			conn.update(cat);
			conn.flush();
			conn.connection().commit();
			setErrorMessage("La categora ha sido modificada correctamente");
		}
		catch (Exception ex)
		{
			setErrorMessage("GestorCategorias: "+ex);
			setErrorMessage("Error al modificar la Categora");
		}
	}
	
	//-------------------------------------------------------
	//Metodo que borra una categoria en BD
	//-------------------------------------------------------	
	
	public void removeCategoria(Session conn,D7Categorias cat) {
		try
		{
			Query q = conn.createQuery(getQuery("UnaCategoria"));
			q = q.setString("sist",user.getDatosMenu().idSelected());
			q = q.setString("cod",cat.getCatCod().toString());
			categorias = (List)q.list();
		
			categoria = (D7Categorias) categorias.get(0);
			
			conn.delete(categoria);
			conn.flush();
			conn.connection().commit();
			categorias.remove(categoria);
			setErrorMessage("La categora ha sido eliminada correctamente");
		}
		catch (Exception ex)
		{
			categorias.remove(categoria);
			setErrorMessage("No se pudo eliminar la categora. Existen tareas dependientes");
		}
	}
	
	//Invierte el estado de un informatico. De Vigente a no Vigente (Valido a no valido) y de No Vigente a Vigente (no valido a valido)
	public void changeState(Session conn, D7Categorias cat) {
		
		cat.setCatSisCod(user.getDatosMenu().idSelected());
		categoria=cat;

		try
		{
			Query q = conn.createQuery(getQuery("UnaCategoria"));
			q = q.setString("sist",user.getDatosMenu().idSelected());
			q = q.setString("cod",cat.getCatCod().toString());
			categorias  = (List)q.list();
		
		}
		catch (Exception ex)
		{
			setErrorMessage("Error al cambiar estado");
		}
		categoria = (D7Categorias) categorias.get(0);
		
		try {
			if (categoria.getCatEstado().equals("V"))
				categoria.setCatEstado("N");
			else
				categoria.setCatEstado("V");
				conn.update(categoria);
				conn.flush();
				conn.connection().commit();
				//Elimino la seccin para que no se mape aun despues de haberle cambiado el estado,
				//Esto error solo ocurre cuando una seccin es su padre debido a las condiciones de la primera llamada recursiva
				categorias.remove(categoria);
				setErrorMessage("Se ha realizado el cambio de estado correctamente");
			} catch (Exception ex) {
			setErrorMessage("GestorCategorias: "+ex);
		}
	}

	
}
