/*
 * Created on 25-abr-2005
 *
 */
package dumbo7.user;

import atica.frameweb.manager.ManagerException;
import atica.frameweb.user.*;
import atica.frameweb.user.menu.DatosMenu;
import dumbo7.utils.*;
import dumbo7.utils.ldap.LDAPUser;
import dumbo7.utils.ldap.LDAPUtil;
import java.sql.*;
import java.io.IOException;
import java.util.*;
import org.apache.log4j.Logger;
import org.hibernate.*;

/**
 * @author Universidad de Murcia - ATICA
 *
 */
public class DumboUser extends User {

	public static final String PREFIJO_EXTERNO = D7Sistemas.EXTERNO+"-"; 
	public static final String CODIGO_EXTERNO = "EXTERNO";
	public static final String CODIGO_USUARIO = "00";
	public static final String CODIGO_DEFAULT = "DEF";
	public static final String CODIGO_SUPLANTADO = "99";
	public static final String CODIGO_ADMIN = "88";
	public static final String CODIGO_COMITE = "77";
	public static final String CODIGO_JEFE_SECCION = "66";
	public static final String CODIGO_TECNICO_COLA = "55";
	public static final String CODIGO_PAS = "PAS";
	public static final String CODIGO_PDI = "PDI";
	public static final String CODIGO_EMPLEADO = "EMP";
	public static final String SEPARADOR = "@";
	
	static Logger logger = Logger.getLogger(DumboUser.class);
	public D7DatosTrabajador dp = null;
	public Hashtable filtros = new Hashtable();
	public DumboUser usuarioOriginal = null;
	public java.util.Date fechaEntrada = new java.util.Date();
	public String servletUrl = "";
	public Hashtable secciones;
	public Hashtable correos;
	public String telefono = "";
	public String ubicacion = "";
	public String edUbicacion = "";
	
	public DumboUser() { 
		super();
	}

	public DumboUser(String l, String p)
	{ 
		super(l,p);
	}

	public DumboUser(Session c,String log,String pwd) throws Exception 
	{ 
		super(c,log,pwd);
		setPersonalData(c);
	}

	public DumboUser(Session c,String log,String pwd,String sist) throws Exception 
	{ 
		super(c,sist+SEPARADOR+log,pwd);
		setPersonalData(c);
		setLogin(log);
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		super.overwriteObject(out);
		out.writeObject(dp);
		out.writeObject(filtros);
		out.writeObject(usuarioOriginal);
		out.writeObject(fechaEntrada);
		out.writeObject(servletUrl);
		out.writeObject(telefono);
		out.writeObject(ubicacion);
		out.writeObject(edUbicacion);
	}
	
	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
		super.overreadObject(in);
		dp = (D7DatosTrabajador) in.readObject();
		filtros = (Hashtable) in.readObject();
		usuarioOriginal = (DumboUser) in.readObject();
		fechaEntrada = (java.util.Date) in.readObject();
		servletUrl = (String) in.readObject();
		telefono = (String) in.readObject();
		ubicacion = (String) in.readObject();
		edUbicacion = (String) in.readObject();
	}
	
	public String getTelefono() { return telefono; }
	public void setTelefono(String tlf) { telefono = tlf; }
	
	public String getUbicacion() { return ubicacion; }
	public void setUbicacion(String ub) { ubicacion = ub; }

	public String getEdUbicacion() { return edUbicacion; }
	public void setEdUbicacion(String ub) { edUbicacion = ub; }

	public void setServletUrl(String u) { servletUrl = u; }
	public String getServletUrl() {
		if (getUsuarioOriginal()!=null) return getUsuarioOriginal().getServletUrl();
		return servletUrl; 
	}
	
	public DumboUser getUsuarioOriginal() { return usuarioOriginal; }
	public void setUsuarioOriginal(DumboUser u) { usuarioOriginal = u; }
	
	public String getFechaHoy() {
		java.util.GregorianCalendar ahora = new java.util.GregorianCalendar();
		return ahora.get(GregorianCalendar.DAY_OF_MONTH)+"/"
			   +(ahora.get(GregorianCalendar.MONTH)+1)+"/"
			   +(ahora.get(GregorianCalendar.YEAR));
	}
	public long getTiempoConectado() {
		java.util.Date ahora = new java.util.Date();
		return ahora.getTime()-fechaEntrada.getTime();
	}
	
	public DumboUser getRealUser() {
		if (usuarioOriginal==null) return this;
		else return usuarioOriginal.getRealUser(); 
	}
	
	public DumboUser getBaseUser() {
		if (usuarioOriginal==null) return null;
		else return getRealUser();
	}
	
	public FiltroUsuario getFiltroUsuario(String cod) {
		String sis = getDatosMenu().idSelected();
		return (FiltroUsuario)filtros.get(sis+cod);
	}
	
	public void setFiltroUsuario(String cod,FiltroUsuario f) {
		String sis = getDatosMenu().idSelected();
		filtros.put(sis+cod,f);
	}
	
	public String getName() { return dp.getDatNombre()+" "+dp.getDatApellidos(); }
	public String getNombre() { return dp.getDatNombre(); }
	public String getApellidos() { return dp.getDatApellidos(); }
	public String getIdentificador() { return dp.getDatIdentificador(); }
	public String getEmail() { return dp.getDatEmail(); }
	public String getUserDomain() { return LDAPUtil.getDominio(); }
	public Hashtable getSecciones() { return secciones; }
	public Hashtable getCorreos() { return correos; }
	public String getUserSection(String s) {
		if (secciones!=null) return (String)secciones.get(s);
		return null;
	}
	public void addUserSection(String s,String c) {
		if (secciones==null) secciones = new Hashtable(); 
		secciones.put(s,c);
	}
	
	public void setDatosPersonales(D7DatosTrabajador d) {
		d.getDatApellidos();d.getDatNombre();
		d.getDatEmail();d.getDatIdentificador();
		d.getDatDepartamento();d.getDatCentro();
		d.getDatTelefono();d.getDatTipo();
		dp = d; 
	}

	public String getSystemCode() {
		if (isUserSuplantado()) {
			// Es un suplantado !!
			// El login es de la forma sistema@correo@dominio.es
			String sis = login.substring(0,login.indexOf(SEPARADOR));
			//setLogin(login.substring(login.indexOf(SEPARADOR)+1));
			return sis;
		} else return ""; 
	}

	public String getLoginCode() {
		if (isUserSuplantado()) {
			// Es un suplantado !!
			// El login es de la forma sistema@correo@dominio.es
			return login.substring(login.indexOf(SEPARADOR)+1);
		} else return login; 
	}
	
	public boolean isUserSuplantado() { return password.equals(""); }
	public boolean isAdminRol(Role role) { return role.getCode().equals(CODIGO_ADMIN); }
	public boolean isRolSuplantado(Role role) { return role.getCode().equals(CODIGO_SUPLANTADO); }
	public boolean isDefaultSist(D7Sistemas sis) { return sis.getSisCod().equals(CODIGO_DEFAULT); }
	
	public String getDataIdentifier(Session c) {
		// Parche para Entrar como cualquiera !!!
		try {
			String milog = getLoginCode();
			if (milog.indexOf(SEPARADOR)>0)	return D7Conversor.convierteMail(c,getLoginCode());
			else return milog;
		} catch (Exception ex) {}
		return null;
		/*
		String milog = getLoginCode();
		if (milog.indexOf(SEPARADOR)>0) {
			LDAPUser usr = LDAPUtil.findUserByMail(getLoginCode());
			if (usr!=null) return usr.getDni();
			return null;
		} else return milog;
		*/
	}
	
	public void setPersonalData(Session c) throws Exception {
		// Establezco sus Datos
		PreparedStatement pstmt = null;
		CallableStatement cstmt = null;
		ResultSet rs = null;
		try {
			String ident = getDataIdentifier(c);
			if (ident==null) ident = getLoginCode();
			D7DatosTrabajador dpers = null;
			if (!getLoginCode().equals(CODIGO_EXTERNO)) {
				dpers = (D7DatosTrabajador) c.load(D7DatosTrabajador.class,ident);
				setDatosPersonales(dpers);
			} else {
				dpers = new D7DatosTrabajador();
				dpers.setDatIdentificador(CODIGO_EXTERNO);
				dpers.setDatApellidos(CODIGO_EXTERNO);
				dpers.setDatNombre(CODIGO_EXTERNO);
				dpers.setDatEmail(CODIGO_EXTERNO);
				setDatosPersonales(dpers);
			}
			// Recoge los Filtros Definidos !!
			Query q = c.createQuery(getQuery("FiltrosUsuario"));
			q = q.setString("usu",ident);
			List l = q.list();
			for (int k=0; k<l.size(); k++) {
				D7FiltrosUsuario fil = (D7FiltrosUsuario)l.get(k);
				String key = fil.getFilSisCod()+FiltroUsuario.filterName(fil.getFilCod().intValue());
				FiltroUsuario f = FiltroUsuario.importFiltro(java.net.URLDecoder.decode(fil.getFilFiltro()));
				if (f!=null) filtros.put(key,f);
			}
			
			// Recoge el correo de DUMBO
			Query q2 = c.createQuery(getQuery("CorreoDumbo"));
			List l2 = q2.list();
			if (l2.size()>0) correos = new Hashtable(); 
			for (int k=0; k<l2.size(); k++) {
				D7Parametros p = (D7Parametros)l2.get(k);
				correos.put(p.getParSisCod(),p.getParValor());
			}
			
			// Recoge la Ubicacin y el telfono del usuario
			if (dpers.getDatEmail().endsWith(LDAPUtil.getDominio())) {
				cstmt = c.connection().prepareCall(getQuery("UbicacionTlfnoProc"));
				cstmt.setString(1,dp.getDatIdentificador());
				cstmt.registerOutParameter(2,Types.VARCHAR);
				cstmt.registerOutParameter(3,Types.VARCHAR);
				cstmt.registerOutParameter(4,Types.VARCHAR);
				cstmt.execute();
				this.setUbicacion(cstmt.getString(2));
				this.setTelefono(cstmt.getString(3));
				this.setEdUbicacion(cstmt.getString(4));
			}
			
		} catch (Exception ex) {
			throw new Exception("El Usuario no est registrado en el Sistema.");
		} finally {
			try {
				if (rs!=null) rs.close();
				if (pstmt!=null) pstmt.close();
				if (cstmt!=null) cstmt.close();
			} catch (Exception ex) {}
		}
	}
	
	public void crearUsuario(Session c) {
		String sistema = getSystemCode(); 
		String id = getDataIdentifier(c);
		try {
			// Saco los Perfiles como Informatico !!
			Query q = c.createQuery(getQuery("PerfilesInformatico"));
			q = q.setString("identif",id);
			List perfiles = q.list();
			Iterator it = perfiles.iterator(); 
			while (it.hasNext()) {
				D7InformaticosPerfiles perfil = (D7InformaticosPerfiles)it.next();
				String sist = perfil.getIpeSisCod();
				String sec = getUserSection(sist);
				if (sec==null) addUserSection(sist,perfil.getInformatico().getInfSecCod());
				if (!password.equals("") || perfil.getIpeSisCod().equals(sistema))
					// En el caso de Suplantados Slo aado los roles del Usuario en el Sistema en que suplanto
					// En el resto de sistemas ser slo usuario !!
					addStaticRole(perfil.getIpePerCod());
			}
			if (isUserSuplantado()) {
				// Se trata de un usuario suplantado !!
				addStaticRole(CODIGO_SUPLANTADO);
			}
			// Sea Cual sea su perfil TODOS SON USUARIOS !!
			addStaticRole(CODIGO_USUARIO);
			
			// Si es responsable de seccin de primer nivel pertenece al comit de dumbo !!
			Query q2 = c.createQuery(getQuery("ResponsableNivel"));
			q2 = q2.setString("usu",id);
			List secs = q2.list();
			if (secs!=null && secs.size()>0) addStaticRole(CODIGO_COMITE);
			
			// Si es responsable de una seccin es Jefe de Seccin !!
			Query q3 = c.createQuery(getQuery("ResponsableSeccion"));
			q3 = q3.setString("usu",id);
			secs = q3.list();
			if (secs!=null && secs.size()>0) addStaticRole(CODIGO_JEFE_SECCION);

			// Si pertenece a una aplicacion gestionada por cola !!
			Query q4 = c.createQuery(getQuery("InformaticoCola"));
			q4 = q4.setString("usu",id);
			secs = q4.list();
			
			if (secs!=null && secs.size()>0) addStaticRole(CODIGO_TECNICO_COLA);

			// Miro las relaciones con la UMU
			PreparedStatement pstmt = null;
			ResultSet rs = null;
			try {
				// Si tiene relacin vigente de PDI !!
				pstmt = c.connection().prepareStatement(getQuery("PDIVigente"));
				pstmt.setString(1,id);
				rs = pstmt.executeQuery();
				if (rs.next()&& rs.getString(1).equals("S")) addStaticRole(CODIGO_PDI);
				rs.close(); pstmt.close();
				
				// Si tiene relacin vigente de PAS !!
				pstmt = c.connection().prepareStatement(getQuery("PASVigente"));
				pstmt.setString(1,id);
				rs = pstmt.executeQuery();
				if (rs.next()&& rs.getString(1).equals("S")) addStaticRole(CODIGO_PAS);
				rs.close(); pstmt.close();
				
			} catch (Exception ex) {
				System.out.println("Error Obteniendo relaciones UMU: "+ex);
			} finally {
				if (rs!=null) rs.close();
				if (pstmt!=null) pstmt.close();
			}
			
			// Si Tiene Acceso como Usuario Interno Tiene Rol de Empleado en todos los sistemas
			if (this.permitido("dumbo7/user","Usuario Interno")) addStaticRole(CODIGO_EMPLEADO);
			
		} catch (Exception ex) {
			System.out.println("CrearUsuario: "+ex);
		}
	}

	public void cargarPerfilesRol(Role role,Session c) {
		String id = getDataIdentifier(c);
		int cont = 0;
		try {
			// Para cada Rol Ajusto dnde corresponde
			Query q = c.createQuery(getQuery("AjustaRol"));
			q = q.setString("identif",id);
			q = q.setString("code",role.getCode());
			List perfiles = q.list();
			Iterator it = perfiles.iterator();
			while (it.hasNext()) {
				D7InformaticosPerfiles perfil = (D7InformaticosPerfiles)it.next();
				D7Sistemas sistema = perfil.getSistema();
				addProfileToRol(role,sistema.getSisCod(),sistema.getSisDescrip());
				cont++;
			}
			if (cont==0) {
				// El rol se ha asignado de forma automtica
				if (role.getCode().equals(CODIGO_COMITE)) {
					// Estos roles se asignan por las secciones de las que son responsables
					Query q2 = c.createQuery(getQuery("ResponsableNivel"));
					q2 = q2.setString("usu",id);
					List secs = q2.list();
					if (secs!=null && secs.size()>0) {
						it = secs.iterator();
						Vector sis = new Vector();
						while (it.hasNext()) {
							D7Secciones s = (D7Secciones)it.next();
							String sist = s.getSecSisCod();
							if (!sis.contains(sist)) {
								addProfileToRol(role,sist,sist);
								sis.add(sist);
							}
						}
					}
				} else if (role.getCode().equals(CODIGO_JEFE_SECCION)) {
					// Estos roles se asignan por las secciones de las que son responsables
					Query q3 = c.createQuery(getQuery("ResponsableSeccion"));
					q3 = q3.setString("usu",id);
					List secs = q3.list();
					if (secs!=null && secs.size()>0) {
						it = secs.iterator();
						Vector sis = new Vector();
						while (it.hasNext()) {
							D7Secciones s = (D7Secciones)it.next();
							String sist = s.getSecSisCod();
							if (!sis.contains(sist)) {
								addProfileToRol(role,sist,sist);
								sis.add(sist);
							}
						}
					}
				} else if (role.getCode().equals(CODIGO_TECNICO_COLA)) {
					// Estos roles se asignan por las secciones de las que son responsables
					Query q3 = c.createQuery(getQuery("InformaticoCola"));
					q3 = q3.setString("usu",id);
					List secs = q3.list();
					if (secs!=null && secs.size()>0) {
						it = secs.iterator();
						Vector sis = new Vector();
						while (it.hasNext()) {
							D7AplicacionesInformaticos s = (D7AplicacionesInformaticos)it.next();
							String sist = s.getAinSisCod();
							if (!sis.contains(sist)) {
								addProfileToRol(role,sist,sist);
								sis.add(sist);
							}
						}
					}
				} else {
					String mySist = getSystemCode();
					// El Rol Usuario, PAS, PDI y EMPLEADO se Asigna a todos los Sistemas Menos al Default
					// SOLO SI NO ES SUPLANTADO, PARA SUPLANTADOS SE ASIGNA ROL USUARIO SOLO EN EL SISTEMA SUPLANTADO !!
					List sistemas = c.createQuery(getQuery("ListaSistemas")).list();
					Iterator itt = sistemas.iterator();
					while (itt.hasNext()) {
						D7Sistemas sistema = (D7Sistemas)itt.next();
						if (isDefaultSist(sistema)) continue;
						if (isUserSuplantado() && !mySist.equals("") && !mySist.equals(sistema.getSisCod())) continue;
						addProfileToRol(role,(sistema.getSisIndInt().equals(D7Sistemas.EXTERNO)?PREFIJO_EXTERNO:"")+sistema.getSisCod(),sistema.getSisDescrip());
					}
				}
			}
		} catch (Exception ex) {
			System.out.println("CargarPerfilesRol: "+ex);
		}
		
	}
	
	public void crearDatosMenu(Session conn) {
		try {
			// Obtengo los Sistemas Existentes de BBDD
			List sistemas = conn.createQuery(getQuery("ListaSistemas")).list();
			// Leo el Men Genrico Para un Sistema
			readDataMenu();
			// Clono y Aado tantos menus como Sistemas Con Iguales Opciones
			if (menuData!=null) {
				DatosMenu sistema = (DatosMenu)menuData.getItems().elementAt(0);
				int contador=0;
				int tama = sistemas.size();
				Iterator it = sistemas.iterator();
				while (it.hasNext()) {
					D7Sistemas sis = (D7Sistemas)it.next();
					if (sis.getSisIndInt().equals(D7Sistemas.EXTERNO)) {
						// Hay un sistema externo, lo meto en el menu con la URL dada
						DatosMenu dm = new DatosMenu();
						dm.setID(PREFIJO_EXTERNO+sis.getSisCod());
						dm.setName(sis.getSisDescrip());
						dm.setURL(sis.getSisUrl());
						menuData.addItem(dm);
					} else {
						if (tama>1 && sis.getSisCod().equals(CODIGO_DEFAULT)) continue;
						if (contador>0) {
							sistema = sistema.clonar();
							menuData.addItem(sistema);
						}
						sistema.setID(sis.getSisCod());
						sistema.setName(sis.getSisDescrip());
						contador++;
					}
				}
				// Ajusto las opciones para el usuario conectado !!
				Enumeration en = menuData.getItems().elements();
				while (en.hasMoreElements()) {
					sistema = (DatosMenu)en.nextElement();
					sistema.removeNotPermited(this,new String[]{sistema.getID()});;
				}
				menuData.removeNotNamed();
				menuData.traduceItems(null,this,null);
			}
		} catch (ManagerException ex) {
			logger.fatal(ex.getError());
		}
	}

}
