/*
 * Created on 16-sep-2005
 *
 */
package dumbo7.utils;

import java.util.*;
import java.sql.*;

import atica.frameweb.utils.*;
import org.hibernate.Session;

import dumbo7.GestorGenerico;
import dumbo7.utils.ldap.LDAPUser;
import dumbo7.utils.ldap.LDAPUtil;

/**
 * @author Universidad de Murcia - ATICA
 *
 */
public class D7Conversor extends GeneralManager {
    
	// Create cache
    static final int MAX_ENTRIES = 100;
    static final boolean CACHE_ACTIVE = true;
    
    private static Map cache = new LinkedHashMap(MAX_ENTRIES+1,.75F,true) {
        // This method is called just after a new entry has been added
        public boolean removeEldestEntry(Map.Entry eldest) {
            return size() > MAX_ENTRIES;
        }
    };

    private static void addCache(String key,String value) {
    	// Add to cache
    	if (!CACHE_ACTIVE || key==null || value == null || key.trim().equals("") || value.trim().equals("")) return;
    	if (getCache(key)==null) cache.put(key, value);
    }
    
    private static String getCache(Object key) {
    	// Get object
    	Object o = cache.get(key);
    	if (o == null && !cache.containsKey(key)) {
    		// Object not in cache. If null is not a possible value in the cache,
    		// the call to cache.contains(key) is not needed
    		return null;
    	} else return (String)o;
    }
    
    static {
    	// If the cache is to be used by multiple threads,
    	// the cache must be wrapped with code to synchronize the methods
    	cache = (Map)Collections.synchronizedMap(cache);
    }

	public static String convierteMail(Session c,String mail) throws Exception {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		D7Conversor dc = new D7Conversor();
		try{
		
		if (mail.indexOf("@")>0) {
			if (!GestorGenerico.isDominioInterno(mail)) {
				// Ojo es un Usuario Externo !!
				throw new Exception("El Correo Introducido "+mail+" no es correcto.");
			//} else { // Lo busco en LDAP
				//if (!GestorGenerico.existsMailUser(mail)) throw new Exception("El Correo Introducido "+mail+" no se encuentra en el LDAP.");
			}
		} else {
			//LDAPUser usr = LDAPUtil.findUserByDni(mail);
			List usuarios = null;
			String myQuery = dc.getQuery("BuscaUsuariosPorDNIConRelacion");
			usuarios = new ArrayList();
			pstmt = c.connection().prepareStatement(myQuery);
			pstmt.setString(1,mail);
			rs = pstmt.executeQuery();

			while (rs.next()) {
				D7DatosPersonales dp = new D7DatosPersonales();
				dp.setDatIdentificador(rs.getString(1));
				dp.setDatNombre(rs.getString(2));
				dp.setDatApellidos(rs.getString(3));
				dp.setDatEmail(rs.getString(4));
				
				usuarios.add(dp); 
			}
			
			if (usuarios.size()==0) throw new Exception("El Identificador Introducido "+mail+" no tiene relacin actual con la universidad o no dispone de correo.");
			return mail;
		}
		D7Conversor conv = new D7Conversor();
		String id = conv.getIdentificadorFrom(c,mail);
		// No se usa la cache dado un mail la asociacin no tiene porque ser del todo correcta
		//addCache(id,mail);
		return id;
		} catch (Exception ex) {
			throw new Exception("D7Conversor[convierteMail]: "+ex);
		} finally {
			try {
				if (rs!=null) rs.close();
				if (pstmt!=null) pstmt.close();
			} catch (Exception exx) {}
		}
	}

	public static String convierteMailExterno(Session c,String mail,String externo) throws Exception {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		D7Conversor dc = new D7Conversor();
		try{
		
		if (mail.indexOf("@")>0) {
			if (!GestorGenerico.isDominioInterno(mail)) {
				// Ojo es un Usuario Externo !!
				if (!GestorGenerico.isMailString(mail)) throw new Exception("El Correo Introducido, "+mail+" no es correcto.");
				return externo;
			//} else { // Lo busco en LDAP
				//if (!GestorGenerico.existsMailUser(mail)) throw new Exception("El Correo Introducido "+mail+" no se encuentra en el LDAP.");
			}
		} else {
			//LDAPUser usr = LDAPUtil.findUserByDni(mail);
			List usuarios = null;
			String myQuery = dc.getQuery("BuscaUsuariosPorDNIConRelacion");
			usuarios = new ArrayList();
			pstmt = c.connection().prepareStatement(myQuery);
			pstmt.setString(1,mail);
			rs = pstmt.executeQuery();

			while (rs.next()) {
				D7DatosPersonales dp = new D7DatosPersonales();
				dp.setDatIdentificador(rs.getString(1));
				dp.setDatNombre(rs.getString(2));
				dp.setDatApellidos(rs.getString(3));
				dp.setDatEmail(rs.getString(4));
				
				usuarios.add(dp); 
			}
			
			if (usuarios.size()==0) throw new Exception("El Identificador Introducido "+mail+" no tiene relacin actual con la universidad o no dispone de correo.");
			return mail;
		}
		D7Conversor conv = new D7Conversor();
		String id = conv.getIdentificadorFrom(c,mail);
		
		return id;
		} catch (Exception ex) {
			throw new Exception("D7Conversor[convierteMailExterno]: "+ex);
		} finally {
			try {
				if (rs!=null) rs.close();
				if (pstmt!=null) pstmt.close();
			} catch (Exception exx) {}
		}
	}

	
	public static String convierteIdentificador(Session c,String id) {
		String s = getCache(id);
		if (s!=null) return s;
		D7Conversor conv = new D7Conversor();
		String mail = conv.getMailFrom(c,id);
		addCache(id,mail);
		return mail;
	}

	public static String convierteIdentificador(Connection c,String id) {
		String s = getCache(id);
		if (s!=null) return s;
		D7Conversor conv = new D7Conversor();
		String mail = conv.getMailFrom(c,id);
		addCache(id,mail);
		return mail;
	}
	
	public String getIdentificadorFrom(Session conn,String mail) {
		String id = "";
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {

			pstmt = conn.connection().prepareStatement(getQuery("GetIdentificadorFromMail"));
			pstmt.setString(1,mail);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				id = rs.getString(1);
			}
			
		} catch (Exception ex) {
			System.out.println("D7Conversor[getIdentificadorFromMail]: "+ex);
		} finally {
			try {
				if (rs!=null) rs.close();
				if (pstmt!=null) pstmt.close();
			} catch (Exception exx) {}
		}
		return id;
	}
	

	public String getMailFrom(Session conn,String id) {
		return getMailFrom(conn.connection(),id);
	}

	public String getMailFrom(Connection conn,String id) {
		String mail = "";
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
		
			pstmt = conn.prepareStatement(getQuery("GetMailFromIdentificador"));
			pstmt.setString(1,id);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				mail = rs.getString(1);
			}
			
		} catch (Exception ex) {
			System.out.println("D7Conversor[getMailFrom]: "+ex);
		} finally {
			try {
				if (rs!=null) rs.close();
				if (pstmt!=null) pstmt.close();
			} catch (Exception exx) {}
		}
		return mail;
	}
	
	public static String date2Order(java.util.Date fecha) {
		int mes = fecha.getMonth()+1;
		int anyo = fecha.getYear()+1900;
		int dia = fecha.getDate();
		int sec = fecha.getSeconds();
		int min = fecha.getMinutes();
		int hou = fecha.getHours();
		return ""+anyo+((mes<10)?"0"+mes:""+mes)+((dia<10)?"0"+dia:""+dia)+((hou<10)?"0"+hou:""+hou)+((min<10)?"0"+min:""+min)+((sec<10)?"0"+sec:""+sec);
	}
	
	public static Timestamp calendar2Timestamp(Calendar fecha) {
		Timestamp t = 
			new Timestamp(fecha.get(Calendar.YEAR)-1900,
					fecha.get(Calendar.MONTH),
					fecha.get(Calendar.DAY_OF_MONTH),
					fecha.get(Calendar.HOUR_OF_DAY),
					fecha.get(Calendar.MINUTE),
					fecha.get(Calendar.SECOND),0);
		return t;
	}
	
}
