package dumbo7.utils;

import java.util.Calendar;

import dumbo7.utils.base.BaseD7Tareas;

/**
 * This is the object class that relates to the DUMBO.D7_TAREAS table.
 * Any customizations belong here.
 */
public class D7Tareas extends BaseD7Tareas {

	public static final String FINALIZADA = "R";
	//public static final String FINALIZADA_CONSUBTAREAS = "F";
	public static final String PENDIENTE = "P";
	public static final String ASIGNADA = "A";
	public static final String ANULADA = "N";
	public static final String EN_ESPERA = "W";
	public static final String RECHAZADA = "X";
	public static final String URGENTE = "U";
	public static final String NO_URGENTE = "N";
	
/*[CONSTRUCTOR MARKER BEGIN]*/
	public D7Tareas () {
		super();
	}

	/**
	 * Constructor for primary key
	 */
	public D7Tareas (
		java.lang.String _tarSisCod,
		java.lang.Integer _tarCod) {

		super (
			_tarSisCod,
			_tarCod);
	}

	/**
	 * Constructor for required fields
	 */
	public D7Tareas (
		java.lang.String _tarSisCod,
		java.lang.Integer _tarCod,
		dumbo7.utils.D7Sistemas _sistema,
		dumbo7.utils.D7TiposTareas _tipoTarea,
		java.lang.String _tarUsuCod,
		java.lang.String _tarAsunto) {

		super (
			_tarSisCod,
			_tarCod,
			_sistema,
			_tipoTarea,
			_tarUsuCod,
			_tarAsunto);
	}
/*[CONSTRUCTOR MARKER END]*/
	
	public String getTarInfEmail() {
		if (getDatosInform()==null) return "";
		return getDatosInform().getDatEmail(); 
	}
	public String getTarInfNombre() {
		if (getDatosInform()==null) return "Desconocido";
		return getDatosInform().getDatApellidos()+", "+getDatosInform().getDatNombre(); 
	}

	public String getTarUsuEmail() { 
		if (getDatosUser()==null) {
			String copi = getTarEmailCopia();
			if ((copi!=null) && copi.indexOf(",")>0) copi = copi.substring(0,copi.indexOf(","));
			return copi;
		}
		return getDatosUser().getDatEmail(); 
	}
	public String getTarUsuNombre() { 
		if (getDatosUser()==null) {
			return "Usuario Externo";
		}
		return getDatosUser().getDatApellidos()+", "+getDatosUser().getDatNombre(); 
	}

	public String getAplicacionDesc() {
		if (getAplicacion()==null) return "";
		return getAplicacion().getAplDescrip();
	}

	public String getAplicacionTipoAsig() {
		if (getAplicacion()==null) return "";
		return getAplicacion().getAplTipoAsig();
	}

	public String getTarPriDesc() {
		if (getPrioridad()==null) return "";
		return getPrioridad().getPriDescrip();
	}
	
	public String getTarCatDesc() {
		if (getCategoria()==null) return "";
		return getCategoria().getCatDescrip();
	}
	
	public String caducada = "N";
	public String getTarCaducada() { return caducada; }
	public void setTarCaducada(String s) { caducada = s; }

	public String rechazar = "N";
	public String getTarRechazar() { return rechazar; }
	public void setTarRechazar(String s) { rechazar = s; }

	public String totales = "";
	public String getTarTotales() { return totales; }
	public void setTarTotales(String s) { totales = s; }

	public boolean subtareasNoFin = false;
	public String getTarSubtareasNoFin() { return subtareasNoFin?"S":"N"; }
	public boolean tieneSubtareasNoFinalizadas() {
		java.util.Set s = getSubtareas();
		if ((s!=null) && !s.isEmpty()) {
			// Miro las subtareas
			java.util.Iterator it = s.iterator();
			while (it.hasNext()) {
				D7Tareas d = (D7Tareas)it.next();
				// Si la tarea est en un estado distinto de R o N 
				if (!d.getTarEstado().equals(D7Tareas.FINALIZADA) && !d.getTarEstado().equals(D7Tareas.ANULADA)) { 
					subtareasNoFin = true; 
					return true; 
				} else {
					// Miro las Subtareas de la tarea
					if (d.tieneSubtareasNoFinalizadas()) {
						subtareasNoFin = true; 
						return true; 
					}
				}
			}
		}
		return false;
	}
	
	public java.util.Date fechaAlerta = null;
	public java.util.Date getFechaAlerta() { return fechaAlerta; }
	public void setFechaAlerta(java.util.Date d) { fechaAlerta = d; }
	
	public String solucion = null;
	public String getSolucion() { return solucion; }
	public void setSolucion(String s) { solucion = s; }
	
	public D7Tareas cloneTarea() {
		D7Tareas nt = new D7Tareas();
		nt.setTarSisCod(getTarSisCod());
		nt.setTarCod(getTarCod());
		nt.setTarTipCod(getTarTipCod());
		nt.setTarAplCod(getTarAplCod());
		nt.setTarPadreTarCod(getTarPadreTarCod());
		nt.setTarInfCod(getTarInfCod());
		nt.setTarPriCod(getTarPriCod());
		nt.setTarCatCod(getTarCatCod());
		nt.setTarFecmaxResol(getTarFecmaxResol());
		nt.setTarUbicacion(getTarUbicacion());
		nt.setTarDescrip(getTarDescrip());
		nt.setTarCosteMat(getTarCosteMat());
		nt.setTarDescripSegui(getTarDescripSegui());
		nt.setTarFeccrea(getTarFeccrea());
		nt.setTarEmailCopia(getTarEmailCopia());
		nt.setTarHorasRes(getTarHorasRes());
		nt.setTarPlanif(getTarPlanif());
		nt.setTarEstado(getTarEstado());
		nt.setTarCosteEco(getTarCosteEco());
		nt.setTarUrgencia(getTarUrgencia());
		nt.setTarFecfin(getTarFecfin());
		nt.setTarUsuCod(getTarUsuCod());
		nt.setTarAsunto(getTarAsunto());
		nt.setDatosInform(getDatosInform());
		nt.setDatosUser(getDatosUser());
		//nt.setTarUsuEmail(getTarUsuEmail());
		//nt.setTarInfEmail(getTarInfEmail());
		nt.setAplicacion(getAplicacion());
		return nt;
	}
/*
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("\nSisCod: "+getTarSisCod());
		sb.append("\nCod: "+getTarCod());
		sb.append("\nTipCod: "+getTarTipCod());
		sb.append("\nAplCod: "+getTarAplCod());
		sb.append("\nPadreCod: "+getTarPadreTarCod());
		sb.append("\nInfCod: "+getTarInfCod());
		sb.append("\nPriCod: "+getTarPriCod());
		sb.append("\nCatCod: "+getTarCatCod());
		sb.append("\nFecmaxResol: "+getTarFecmaxResol());
		sb.append("\nUbicacion: "+getTarUbicacion());
		sb.append("\nDescrip: "+getTarDescrip());
		sb.append("\nCosteMat: "+getTarCosteMat());
		sb.append("\nDescripSegui: "+getTarDescripSegui());
		sb.append("\nFeccrea: "+getTarFeccrea());
		sb.append("\nEmailCopia: "+getTarEmailCopia());
		sb.append("\nHorasRes: "+getTarHorasRes());
		sb.append("\nPlanif: "+getTarPlanif());
		sb.append("\nEstado: "+getTarEstado());
		sb.append("\nCosteEco: "+getTarCosteEco());
		sb.append("\nUrgencia: "+getTarUrgencia());
		sb.append("\nFecfin: "+getTarFecfin());
		sb.append("\nUsuCod: "+getTarUsuCod());
		sb.append("\nAsunto: "+getTarAsunto());
		return sb.toString();
	}
*/	
}
