package dumbo7.utils.ldap.ssl;

import java.awt.Color;
import java.io.PrintStream;
import java.util.*;

public class LDAPIResource {


    public LDAPIResource(String name)
        throws MissingResourceException
    {
        bundle = ResourceBundle.getBundle("lbe.util.resources." + name, Locale.getDefault());
    }

    public LDAPIResource(ResourceBundle bundle)
    {
        this.bundle = bundle;
    }

    public String get(String nm)
    {
        return get(nm, null);
    }

    public String get(String nm, String defValue)
    {
        try
        {
            return bundle.getString(nm);
        }
        catch(MissingResourceException _ex)
        {
            return defValue;
        }
    }

    public ResourceBundle getBundle()
    {
        return bundle;
    }

    public Color getColor(String key, Color defColor)
    {
        try
        {
            String code = bundle.getString(key);
            return Color.decode(code);
        }
        catch(MissingResourceException _ex)
        {
            return defColor;
        }
        catch(NumberFormatException _ex)
        {
            return defColor;
        }
    }

    public static LDAPIResource getDefault()
    {
        return defaultResource;
    }

    public int getInt(String nm, int defValue)
    {
        try
        {
            String str = bundle.getString(nm);
            return Integer.parseInt(str);
        }
        catch(MissingResourceException _ex)
        {
            return defValue;
        }
        catch(NumberFormatException _ex)
        {
            return defValue;
        }
    }

    public static LDAPIResource getResource(String name)
    {
        if(bundles == null)
            bundles = new HashMap();
        LDAPIResource rs = (LDAPIResource)bundles.get(name);
        if(rs != null)
            return rs;
        try
        {
            rs = new LDAPIResource(name);
        }
        catch(MissingResourceException e)
        {
            System.err.println("IResource: Failed to load rb: " + name + " (" + e.getMessage() + ")");
            return defaultResource;
        }
        bundles.put(name, rs);
        return rs;
    }

    private static LDAPIResource defaultResource;
    private static HashMap bundles;
    private ResourceBundle bundle;

    static 
    {
        try
        {
            defaultResource = new LDAPIResource("browser");
        }
        catch(MissingResourceException e)
        {
            System.err.println("Error: " + e.getMessage());
            System.exit(-2);
        }
    }
}
