package dumbo7.utils.ldap.ssl;

import com.sun.net.ssl.*;
import com.sun.net.ssl.internal.ssl.Provider;
import java.io.*;
import java.net.*;
import java.security.KeyStore;
import java.security.Security;
import java.util.Hashtable;
import javax.net.SocketFactory;
import javax.net.ssl.SSLSocketFactory;

public class LDAPSocketFactory extends SSLSocketFactory {

    protected LDAPSocketFactory()
    {
        factory = null;
        init(null, null);
    }

    protected LDAPSocketFactory(InputStream in, String keyStore, String password)
        throws Exception
    {
        factory = null;
        KeyStore ks = null;
        if(keyStore.endsWith(".p12"))
            ks = KeyStore.getInstance("PKCS12");
        else
            ks = KeyStore.getInstance("JKS");
        char pwd[] = password.toCharArray();
        ks.load(in, pwd);
        init(ks, pwd);
    }

    protected LDAPSocketFactory(String keyStore, String passphrase)
    {
        factory = null;
        init(null, null);
    }

    private static void closeStream(InputStream in)
    {
        if(in == null)
            return;
        try
        {
            in.close();
        }
        catch(Exception _ex) { }
    }

    public Socket createSocket(String host, int port)
        throws IOException, UnknownHostException
    {
        return factory.createSocket(host, port);
    }

    public Socket createSocket(String host, int port, InetAddress client_host, int client_port)
        throws IOException, UnknownHostException
    {
        return factory.createSocket(host, port, client_host, client_port);
    }

    public Socket createSocket(InetAddress host, int port)
        throws IOException, UnknownHostException
    {
        return factory.createSocket(host, port);
    }

    public Socket createSocket(InetAddress host, int port, InetAddress client_host, int client_port)
        throws IOException, UnknownHostException
    {
        return factory.createSocket(host, port, client_host, client_port);
    }

    public Socket createSocket(Socket socket, String host, int port, boolean autoclose)
        throws IOException, UnknownHostException
    {
        return factory.createSocket(socket, host, port, autoclose);
    }

    public static synchronized SocketFactory getDefault()
    {
        String keyStore = LDAPConfig.getSessionKeyStore();
        if(keyStore == null)
            return getDefaultFactory();
        if(factories == null)
            factories = new Hashtable();
        SocketFactory fac = (SocketFactory)factories.get(keyStore);
        if(fac == null)
        {
            fac = load(keyStore, LDAPConfig.getSessionKeyStorePwd());
            if(fac != null)
                factories.put(keyStore, fac);
        }
        return fac;
    }

    public String[] getDefaultCipherSuites()
    {
        return factory.getDefaultCipherSuites();
    }

    private static SocketFactory getDefaultFactory()
    {
        if(default_factory == null)
            default_factory = new LDAPSocketFactory();
        return default_factory;
    }

    private TrustManager[] getDefaultTrustManager()
    {
        if(trustManagers == null)
            trustManagers = (new LDAPTruster[] {new LDAPTruster()});
        return trustManagers;
    }

    private static String getPassword(String keyStore)
    {
    	return "changeit";
    }

    public String[] getSupportedCipherSuites()
    {
        return factory.getSupportedCipherSuites();
    }

    private void init(KeyStore ks, char password[])
    {
        SSLContext ctx = null;
        com.sun.net.ssl.KeyManager keyManagers[] = null;
        TrustManager trustManagers[] = null;
        try
        {
            if(ks != null)
            {
                KeyManagerFactory kmf = null;
                kmf = KeyManagerFactory.getInstance("SunX509");
                kmf.init(ks, password);
                keyManagers = kmf.getKeyManagers();
            }
            ctx = SSLContext.getInstance("TLS");
            trustManagers = getDefaultTrustManager();
            ctx.init(keyManagers, trustManagers, null);
            factory = ctx.getSocketFactory();
        }
        catch(Exception e)
        {
            System.err.println("ASF Error: failed to initialize : " + e.getMessage());
        }
    }

    private static SocketFactory load(String keyStore, String pwd)
    {
        InputStream in = null;
        try
        {
            in = new FileInputStream(keyStore);
        }
        catch(IOException e)
        {
            return null;
        }
        if(pwd == null)
        {
            pwd = getPassword(keyStore);
            if(pwd == null)
            {
                closeStream(in);
                return null;
            }
        }
        try
        {
            SocketFactory socketfactory;
            try
            {
                LDAPSocketFactory advancedsocketfactory = new LDAPSocketFactory(in, keyStore, pwd);
                return advancedsocketfactory;
            }
            catch(Exception e)
            {
                socketfactory = null;
            }
            return socketfactory;
        }
        finally
        {
            closeStream(in);
        }
    }

    private SSLSocketFactory factory;
    private static TrustManager trustManagers[] = null;
    private static LDAPSocketFactory default_factory = null;
    private static Hashtable factories = null;

    static 
    {
        Security.addProvider(new Provider());
    }
}
