package dumbo7.utils.ldap.ssl;

import com.sun.net.ssl.TrustManagerFactory;
import com.sun.net.ssl.X509TrustManager;
import java.io.*;
import java.net.*;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

public class LDAPTruster implements X509TrustManager
{

 public LDAPTruster()
 {
     certStore = null;
     certStorePwd = null;
     trustManager = null;
     ks = null;
     certStore = LDAPConfig.getCACertFile();
     certStorePwd = LDAPConfig.getCACertPwd().toCharArray();
     init();
 }

 private boolean deleteCert(String id)
 {
     try
     {
         ks.deleteEntry(id);
     }
     catch(KeyStoreException _ex)
     {
         return false;
     }
     return true;
 }

 public X509Certificate[] getAcceptedIssuers()
 {
     if(trustManager == null)
         return null;
     else
         return trustManager.getAcceptedIssuers();
 }

 private X509Certificate getCACert(X509Certificate chain[])
 {
     X509Certificate ca = chain[chain.length - 1];
     if(ca.getSubjectDN().equals(ca.getIssuerDN()))
         return ca;
     else
         return null;
 }

 private void init()
 {
     try
     {
         if(certStore.endsWith(".p12"))
             ks = KeyStore.getInstance("PKCS12");
         else
             ks = KeyStore.getInstance("JKS");
     }
     catch(KeyStoreException e)
     {
         System.err.println("ASF Truster: Failed to create cert store : " + e.getMessage());
         return;
     }
     InputStream in = null;
     if(certStore.indexOf("://") == -1)
         try
         {
             in = new FileInputStream(certStore);
         }
         catch(FileNotFoundException _ex) { }
     else
         try
         {
             URL url = new URL(certStore);
             URLConnection con = url.openConnection();
             in = con.getInputStream();
         }
         catch(MalformedURLException e)
         {
             System.err.println("ASF Truster: The location of the cert store file is invalid: " + e.getMessage());
         }
         catch(IOException _ex) { }
     try
     {
         ks.load(in, certStorePwd);
     }
     catch(Exception e)
     {
         System.err.println("ASF Truster: Failed to load the cert store : " + e.getMessage());
         return;
     }
     finally
     {
         if(in != null)
             try
             {
                 in.close();
             }
             catch(Exception _ex) { }
     }
     try
     {
         trustManager = initTrustManager(ks);
     }
     catch(Exception e)
     {
         System.err.println("ASF Truster: Failed to create initial trust manager : " + e.getMessage());
         return;
     }
 }

 private X509TrustManager initTrustManager(KeyStore ks)
     throws NoSuchAlgorithmException, KeyStoreException
 {
     TrustManagerFactory trustManagerFactory = null;
     trustManagerFactory = TrustManagerFactory.getInstance("SunX509");
     trustManagerFactory.init(ks);
     com.sun.net.ssl.TrustManager trusts[] = trustManagerFactory.getTrustManagers();
     return (X509TrustManager)trusts[0];
 }

 private boolean isAccepted(X509Certificate caCert)
 {
     X509Certificate certs[] = getAcceptedIssuers();
     if(certs == null)
         return false;
     for(int i = 0; i < certs.length; i++)
         if(caCert.equals(certs[i]))
             return true;

     return false;
 }

 public boolean isClientTrusted(X509Certificate chain[])
 {
     if(trustManager == null)
         return false;
     else
         return trustManager.isClientTrusted(chain);
 }

 public boolean isServerTrusted(X509Certificate chain[])
 {
     if(trustManager != null)
     {
         boolean rs = trustManager.isServerTrusted(chain);
         if(rs)
             return rs;
     }
     X509Certificate ca = getCACert(chain);
     if(ca != null)
     {
         if(isAccepted(ca))
         {
             System.out.println("Fallo2");
             return false;
         }
         String id = String.valueOf(System.currentTimeMillis());
         X509TrustManager tmpTrustManager = null;
         try
         {
             ks.setCertificateEntry(id, ca);
             tmpTrustManager = initTrustManager(ks);
         }
         catch(Exception e)
         {
             System.err.println("ASF Truster: Failed to create tmp trust store : " + e.getMessage());
             return false;
         }
         if(tmpTrustManager.isServerTrusted(chain))
         {
         	// Acepta el certificado en esta Sesin 
         	 deleteCert(id);
             return true;
         } else
         {
         	System.out.println("Fallo3");
             return false;
         }
     } else
     {
     	System.out.println("Fallo1");
         return false;
     }
 }

 private boolean saveStore()
 {
     OutputStream out = null;
     try
     {
         try
         {
             if(certStore.indexOf("://") == -1)
             {
                 out = new FileOutputStream(certStore);
             } else
             {
                 URL url = new URL(certStore);
                 URLConnection con = url.openConnection();
                 con.setDoOutput(true);
                 out = con.getOutputStream();
             }
             ks.store(out, certStorePwd);
             boolean flag = true;
             return flag;
         }
         catch(Exception e)
         {
             System.err.println("ASF Truster: Failed to save trust store : " + e.getMessage());
         }
         boolean flag1 = false;
         return flag1;
     }
     finally
     {
         if(out != null)
             try
             {
                 out.close();
             }
             catch(Exception _ex) { }
     }
 }

 private String certStore;
 private char certStorePwd[];
 private X509TrustManager trustManager;
 private KeyStore ks;
}
