package atica.database.util.db;

/**
*
*	Interface que define los mtodos necesarios para gestionar una comunicacin con la
*	BBDD, nos permitir disponer de diferentes formas de obtener esta conexin sin
*	necesidad de modificar las clases que hagan uso de ellas.
*
*	@author Juan Jos Meroo Snchez
*	@version 1.0
*
*	@see DBConnectionImpl
*	@see DBConnectionImplWebSphere
*
*/

//import net.sf.hibernate.*;
import org.hibernate.*;

public interface DBConnection {

	/**
	*
	*	Permite obtener una conexin a BBDD.
	*
	*	@return Conexin establecida con la BBDD definida en la Fuente de Datos.
	*
	*/
  public java.sql.Connection getConnection();
  
  /**
  *
  *	Permite liberar la conexin obtenida mediante getConnection.
  *
  *	@param conn La conexin a liberar.
  *	@see #getConnection
  *
  */
  public void freeConnection(java.sql.Connection conn);

	/**
	*
	*	Permite obtener una session Hibernate a BBDD.
	*
	*	@return Sesin Hibernate establecida con la BBDD definida en la Fuente de Datos.
	*
	*/
public Session getSession();

/**
*
*	Permite liberar la sesin hibernate obtenida mediante getSession.
*
*	@param session La session a liberar.
*	@see #getSession
*
*/
public void freeSession(Session session);
  
  /**
  *
  *	Retorna un identificador de la Fuente de Datos que est siendo utilizada para
  * obtener las conexiones a BBDD.
  *
  *	@return Una cadena que identifica al DataSource que est siendo utilizado.
  *
  */
  public String idOfDataSource();
  
  /**
  *
  *	Devuelve en nmero de conexiones que han sido entregadas mediante el mtodo getConnection
  * por este gestor de conexiones. Se utiliza desde el SnoopServlet para mostrar el informe de conexiones.
  *
  *	@return Nmero de conexiones entregadas mediante el mtodo getConnection.
  *	@see #getConnection
  * @see atica.samples.SnoopServlet
  *
  */
  public long getConnectionCount();
  
  /**
  *
  *	Devuelve en nmero de conexiones que han sido liberadas mediante el mtodo freeConnection
  * por este gestor de conexiones. Se utiliza desde el SnoopServlet para mostrar el informe de conexiones.
  *
  *	@return Nmero de conexiones liberadas mediante el mtodo freeConnection.
  *	@see #freeConnection
  * @see atica.samples.SnoopServlet
  *
  */
  public long freeConnectionCount();
  
  /**
  *
  *	Devuelve en nmero de conexiones activas, es decir, la diferencia entre el nmero de conexiones
  * entregadas y las liberadas por este gestor de conexiones. Se utiliza desde el SnoopServlet para mostrar el informe de conexiones.
  *
  *	@return Diferencia entre nmero de conexiones entregadas y liberadas.
  *	@see #getConnection
  *	@see #freeConnection
  * @see atica.samples.SnoopServlet
  *
  */
  public long activeConnectionCount();
  
}
