package atica.database.util.db;

/**
*
*	Factora de Gestores de Conexiones a BBDD. Mediante esta clase definimos el tipo de
* gestor que se utilizar en las diferentes aplicaciones. Un cambio en el gestor a utilizar
*	no provoca cambios en a las aplicaciones que lo utilicen.
*
*	@author Juan Jos Meroo Snchez
*	@version 1.0
*
*/

public class DBConnectionFactory {
	static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(DBConnectionFactory.class);
	/**
	*
	*	Mtodo esttico que retorna un gestor de conexin a partir de un nombre de fuente de datos.
	* Las clases que crearn para utilizar este mtodo utilizarn esta clase para obtener el
	*	gestor de conexiones.
	*
	*	@param name Nombre de la Fuente de Datos que utilizar el gestor devuelto.
	*	@return Retorna un gestor de conexiones mediante la interface DBConnection.
	*	@see atica.util.obj.ObjectBD
	*	@see atica.util.obj.HttpServletBD
	*
	*/
  public static DBConnection createDBConnection(String name) {
  	try {
  		try {
  			// Comprueba si est en WebSphere !!
  			Class.forName("com.ibm.ejs.ns.jndi.CNInitialContextFactory");
  			return new DBConnectionImplWebSphere(name);
  		} catch (Exception ex) {
  			// Comprueba si est en Tomcat !!
            String ch = System.getProperty("catalina.home");
            if ((ch==null) || ch.equals("")) return new DBConnectionImpl(name);
            else return new DBConnectionImplTomcat(name);
  		}
  	} catch (Exception ex) {
  		logger.fatal("DBConnectionFactory["+name+"]: "+ex);
  	}
  	return null;
  }

}
