package atica.database.util.db;

/**
*
*	Esta clase es una implementacin para WebSphere de un gestor de conexiones usando Fuente de Datos.
*	Esta implementacin slo es vlida en un servidor de aplicaciones WebSphere ya que hace uso
*	expreso de la clase de IBM para conextos jndi.
*
*	@author Juan Jos Meroo Snchez
*	@version 1.0
*
*/
//import net.sf.hibernate.*;
import org.hibernate.*;

public class DBConnectionImplWebSphere implements DBConnection {

	static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(DBConnectionImplWebSphere.class);
	/**
	*
	*	Almacena la fuente de datos que utiliza este objeto.
	*
	*/
	protected javax.sql.DataSource _ds = null;
	protected SessionFactory _hs = null;
	/**
	*
	*	Almacena el nombre de la fuente de datos que utiliza este objeto.
	*
	*/
	protected String nameDS = "";
	/**
	*
	*	Contadores de nmero de getConnection y freeConnection realizados.
	*
	*/
	protected long numGets = 0, numFrees = 0;
	
	/**
	*
	*	Constructor del Gestor de Conexiones. Lo construye en base al nombre de fuente de datos
	*	enviado como parmetro.
	*
	*	@param name Nombre de la fuente de datos que se desea que utilice.
	*
	*/
  public DBConnectionImplWebSphere(String name) throws java.io.IOException {
  	
    	try
    	{
    		nameDS = name;
    		java.util.Hashtable params = new java.util.Hashtable();
            params.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, "com.ibm.ejs.ns.jndi.CNInitialContextFactory");
            javax.naming.Context ctx= new javax.naming.InitialContext(params);
      	_ds = (javax.sql.DataSource)ctx.lookup (nameDS);
    	}
    	catch (javax.naming.NamingException exp)
    	{
      	exp.printStackTrace() ;
    	}
    	/*
    	try {
    		javax.naming.Context initCtx = (javax.naming.Context) new javax.naming.InitialContext() ;
    		javax.naming.Context envCtx = (javax.naming.Context) initCtx.lookup("java:comp/env");
    		_hs = (SessionFactory)envCtx.lookup (nameDS.replaceFirst("j","h"));
    	} catch (Exception ex) {
    		// Puedeo no interesar hibernate
    	}
    	*/
  }

  /**
  *
  *	Obtiene una conexin a raz de la fuente de datos que utiliza el gestor.
  *
  *	@return Conexin a la BBDD determinada por la fuente de datos utilizada.
  *
  */
  public java.sql.Connection getConnection() {
		try {
  		java.sql.Connection conn = _ds.getConnection();
  		if (conn!=null) numGets++;
  		return conn;
		} catch (Exception ex) {
			logger.fatal("GetConnection["+nameDS+"]: "+ex);
		}
		return null;
  }
  

  /**
  *
  *	Libera la conexin a BBDD entregada previamente mediante getConnection.
  *
  *	@param conn Conexin a BBDD obtenida previamente.
  *	@see #getConnection
  *
  */
  public void freeConnection(java.sql.Connection conn) {
		try {
  		if ((conn!=null)&&(!conn.isClosed())) {
  			conn.close();
  			numFrees++;
  		}
		} catch (Exception ex) {
			logger.fatal("FreeConnection["+nameDS+"]: "+ex);
		}
  }
  
	/**
	*
	*	Permite obtener una session Hibernate a BBDD.
	*
	*	@return Sesin Hibernate establecida con la BBDD definida en la Fuente de Datos.
	*
	*/
public Session getSession() {
	try {
		Session conn = _hs.openSession();
		if (conn!=null) numGets++;
		return conn;
		} catch (Exception ex) {
			logger.fatal("GetSession["+nameDS+"]: "+ex);
		}
		return null;
}

/**
*
*	Permite liberar la sesin hibernate obtenida mediante getSession.
*
*	@param session La session a liberar.
*	@see #getSession
*
*/
public void freeSession(Session session) {
	try {
		if (session!=null) {
			session.close();
			numFrees++;
		}
		} catch (Exception ex) {
			logger.fatal("FreeSession["+nameDS+"]: "+ex);
		}
}
  
  /**
  *
  *	Retorna una cadena que identifica la fuente de datos utilizada por el gestor, y por tanto al gestor.
  *
  *	@return Cadena que identifica al gestor de conexiones.
  *	@see #freeConnection
  *
  */
  public String idOfDataSource() { return "WebSphere["+nameDS+"]"; }

  /**
  *
  *	Retorna el nmero de conexiones a BBDD obtenidas mediante el mtodo getConnection.
  *
  *	@return Nmero de conexiones entregadas con getConnection.
  *	@see #getConnection
  *
  */
  public long getConnectionCount() { return numGets; }
  /**
  *
  *	Retorna el nmero de conexiones liberadas mediante el mtoro freeConnection.
  *
  *	@return Nmero de conexiones a BBDD liberadas.
  *	@see #freeConnection
  *
  */
  public long freeConnectionCount() { return numFrees; }
  /**
  *
  *	Retorna la diferencia existente entre el nmero de conexiones entregadas por getConnection
  * y liberadas por freeConnection, que dan una idea de las conexiones en uso o perdidas.
  *
  *	@return Diferencia entre conexiones entregadas y liberadas.
  *	@see #getConnection
  *	@see #freeConnection
  *
  */
  public long activeConnectionCount() { return numGets-numFrees; }
}
