package atica.database.util.obj;

/**
*
*	Esta clase incluye la utilizacin de los gestores de conexiones definidos 
*	incorporando su uso a los objetos de una aplicacin. Cualquier objeto de
* nuestra aplicacin requiera conexin a BBDD debera heredar de esta clase 
* incorporando de forma sencilla el uso de Fuente de Datos.
*
*	@author Juan Jos Meroo Snchez
*	@version 1.0
*
*/

//import net.sf.hibernate.*;
import org.hibernate.*;

public class ObjectBD {
	static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(ObjectBD.class);
	/**
	*
	*	Tabla esttica que mantiene el conjunto de Gestores utilizados por la aplicacin.
	*
	*/
  protected static java.util.Hashtable brokers = new java.util.Hashtable(50);   
  /**
  *
  *	Gestor utilizado en concreto por el objeto en cuestin.
  *
  */
  protected atica.database.util.db.DBConnection broker = null;

  /**
  *
  *	Constructor sin parmetros. Obtiene el nombre de la fuente de datos a partir de un fichero
  *	denominado db.conf situado junto al archivo .class. El formato de este archivo es de texto,
  *	e incluye en una nica lnea el nombre de la fuente de datos.
  *
  */
  public ObjectBD() {
  	if (broker==null) {
  		try {
  			java.io.BufferedReader in = new java.io.BufferedReader(new java.io.InputStreamReader(getClass().getResource("db.conf").openStream()));
  			String dsName = in.readLine();
  			in.close();
  			broker = (atica.database.util.db.DBConnection)brokers.get(dsName);
  			if (broker==null) {
  				broker = atica.database.util.db.DBConnectionFactory.createDBConnection(dsName);
  				brokers.put(dsName,broker);
  			}
  		} catch (Exception ex) {
  			logger.fatal("ObjectBD["+getClass().getName()+"]: "+ex);
  		}
  	}
  }
  
  /**
  *
  *	Constructor parametrizado, recibe el nombre de la fuente de datos a utilizar.
  *
  *	@param dsName Nombre de la fuente de datos a utilizar.
  *
  */
  public ObjectBD(String dsName) {
  	broker = (atica.database.util.db.DBConnection)brokers.get(dsName);
  	if (broker==null) {
  		broker = atica.database.util.db.DBConnectionFactory.createDBConnection(dsName);
  		brokers.put(dsName,broker);
  	}
  }
  
  /**
  *
  *	Mtodo para obtener una conexin a BBDD. Nos independiza de la interface de gestor utilizada.
  *
  *	@return Conexin a BBDD obtenida a partir de la fuente de datos establecida para el objeto.
  *	@see atica.util.db.DBConnection#getConnection()
  *
  */
	public java.sql.Connection getConnection() { return broker.getConnection(); }
	/**
	*
	*	Mtodo para liberar una conexin a BBDD. Nos independiza de la interface de gestor utilizada.
	*
	*	@param c Conexin a liberar.
	*	@see atica.util.db.DBConnection#freeConnection(java.sql.Connection)
	*
	*/
	public void freeConnection(java.sql.Connection c) { broker.freeConnection(c); }
	  /**
	  *
	  *	Mtodo para obtener una sesin hibernate a BBDD. Nos independiza de la interface de gestor utilizada.
	  *
	  *	@return Conexin a BBDD obtenida a partir de la fuente de datos establecida para el objeto.
	  *	@see atica.util.db.DBConnection#getSession()
	  *
	  */
		public Session getSession() { return broker.getSession(); }
		/**
		*
		*	Mtodo para liberar una sesin hibernate a BBDD. Nos independiza de la interface de gestor utilizada.
		*
		*	@param c Conexin a liberar.
		*	@see atica.util.db.DBConnection#freeSession(net.sf.hibernate.Session)
		*
		*/
		public void freeSession(Session c) { broker.freeSession(c); }
	
	/**
	*
	*	Retorna una cadena que identifica al gestor de conexin utilizado por este objeto.
	*
	*	@return Cadena identificativa del gestor utilizado.
	* @see atica.util.db.DBConnection#idOfDataSource()
	*
	*/
	public String idOfDataSource() { return broker.idOfDataSource(); }
	/**
	*
	*	Retorna el nmero de conexiones entregadas por el gestor de conexiones utilizado.
	*
	*	@return Nmero de conexiones entregadas.
	*
	*/
	public long getConnectionCount() { return broker.getConnectionCount(); }
	/**
	*
	*	Retorna el nmero de conexiones liberadas por el gestor de conexiones utilizado.
	*
	*	@return Nmero de conexiones liberadas.
	*
	*/
	public long freeConnectionCount() { return broker.freeConnectionCount(); }
	/**
	*
	*	Retorna la diferencia entre conexiones entregadas y liberadas por el gestor de conexiones utilizado.
	*
	*	@return Diferencia entre conexiones entregadas y liberadas.
	*
	*/
	public long activeConnectionCount() { return broker.activeConnectionCount(); }
	
}
