package atica.frameweb.crypto;

import java.io.*;
import java.security.*;
import javax.crypto.*;

public class UtilCrypto {

	public static String KEY_FILE_NAME = "frmweb.ky";
	
	public static void main(String[] args) {
		try {
			if (args.length!=2) {
				System.out.println("Use: atica.frameweb.crypto.UtilCrypto [-e cadena] [-d cadena] [-g file]");
			} else {
				if (args[0].equals("-e")) System.out.println(encryptString(args[1]));
				if (args[0].equals("-d")) System.out.println(decryptString(args[1]));
				if (args[0].equals("-g")) generateKey(args[1]);
			}
		} catch (Exception ex) {
			System.out.println("ERROR: "+ex.getMessage());
		}
	}

	public static void encryptFile(File in,File out) {
		encryptFile(null,in,out);
	}
	public static void encryptFile(String file,File in,File out) {
		try {
		    Key key = getKey(file);
			Cipher encrypt = Cipher.getInstance("Rijndael");	
			encrypt.init(Cipher.ENCRYPT_MODE,key);
			
			CipherOutputStream cout = new CipherOutputStream(new FileOutputStream(out),encrypt);
			FileInputStream cin = new FileInputStream(in);
			byte [] bytes = new byte[4096];
			int n = cin.read(bytes);
			while (n==4096) {
				cout.write(bytes,0,n);
				n = cin.read(bytes);
			}
			cout.write(bytes,0,n);
			cout.flush(); cout.close();
			
		} catch (Exception ex) {
			System.out.println("EncryptFile: "+ex);
		}
	}

	public static void decryptFile(File in,File out) {
		decryptFile(null,in,out);
	}
	public static void decryptFile(String file,File in,File out) {
		try {
		    Key key = getKey(file);
			Cipher decrypt = Cipher.getInstance("Rijndael");	
			decrypt.init(Cipher.DECRYPT_MODE,key);
			
			CipherInputStream cin = new CipherInputStream(new FileInputStream(in),decrypt);
			FileOutputStream cout = new FileOutputStream(out);
			byte [] bytes = new byte[4096];
			int n = cin.read(bytes);
			while (n==4096) {
				cout.write(bytes,0,n);
				n = cin.read(bytes);
			}
			cout.write(bytes,0,n);
			cout.flush(); cout.close();
			
		} catch (Exception ex) {
			System.out.println("DecriptFile: "+ex);
		}
	}
	
	public static String encryptString(String st) {
		return encryptString(null,st);
	}
	
	public static String encryptString(String file,String st) {
		try {
		    Key key = getKey(file);
			Cipher encrypt = Cipher.getInstance("Rijndael");	
			encrypt.init(Cipher.ENCRYPT_MODE,key);
			
			ByteArrayOutputStream out = new ByteArrayOutputStream(); 
			PrintStream cout = new PrintStream(new CipherOutputStream(out,encrypt));
			cout.println(st);
			cout.flush(); cout.close();
			
			return new String(out.toByteArray());
			
		} catch (Exception ex) {
			System.out.println("EncryptString: "+ex);
		}
		return null;
	}
	
	public static String decryptString(String st) {
		return decryptString(null,st);
	}
	
	public static String decryptString(String file,String st) {
		try {
		    Key key = getKey(file);
			Cipher decrypt = Cipher.getInstance("Rijndael");	
			decrypt.init(Cipher.DECRYPT_MODE,key);
			
			ByteArrayInputStream in = new ByteArrayInputStream(st.getBytes());
			BufferedReader cin = new BufferedReader(new InputStreamReader(new CipherInputStream(in,decrypt)));
			String total = "";
			String linea = cin.readLine();
			while (linea!=null) {
				total += linea;
				linea = cin.readLine();
			}
			cin.close();
			return total;
		} catch (Exception ex) {
			System.out.println("DecryptString: "+ex);
		}
		return null;
	}

	public static Key generateKey() {
		try {
		    KeyGenerator gen = KeyGenerator.getInstance("Rijndael");
		    gen.init(new SecureRandom());
		    Key key = gen.generateKey();
		    return key;
		} catch (Exception ex) {
			System.out.println("GenerateKey: "+ex);
		}
		return null;
	}

	public static void generateKey(String file) {
		try {
		    Key k = generateKey();
		    ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(file));
		    out.writeObject(k);
		    out.flush(); out.close();
		} catch (Exception ex) {
			System.out.println("GenerateKeyFile: "+ex);
		}
	}
	
	public static Key getKey(String file) {
		try {
			UtilCrypto uc = new UtilCrypto();
		    return uc.getCipherKey(file);
		} catch (Exception ex) {
			System.out.println("getKey: "+ex);
		}
		return null;
	}

	public Key getCipherKey(String file) {
		try {
			ObjectInputStream in = null;
			if (file!=null) in = new ObjectInputStream(new FileInputStream(getClass().getResource(file).getFile()));
		    else in = new ObjectInputStream(new FileInputStream(getClass().getResource(KEY_FILE_NAME).getFile()));
		    Key key = (Key)in.readObject();
		    in.close();
		    return key;
		} catch (Exception ex) {
			System.out.println("getCipherKey: "+ex);
		}
		return null;
	}
	
}
