/*
 * Created on 12-feb-2005
 *
 */
package atica.frameweb.manager;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.*;
import atica.frameweb.broker.*;
import atica.frameweb.estadisticas.*;
import atica.frameweb.documentacion.*;

/**
 * @author Juan Jos Meroo Snchez
 *
 *	Servlet nico que interpreta las acciones 
 *	a realizar dependiendo de los parmetros recibidos. 
 */

public class HttpServletManager extends HttpServlet {

	protected static final long MAX_UPLOAD=1024*1024*512; // 512MB
	protected static final String DB_CONFIG = "config.properties";
	protected static final String LOG_LEVEL = "LOG_LEVEL";
	protected static final String UPLOAD_DIR = "UPLOAD_DIR";
	protected static final String UPLOAD_LENGTH = "UPLOAD_LENGTH";
	protected static final String URL_ERROR = "URL_ERROR";
	protected static final String ADMIN_LOGIN = "ADMIN_LOGIN";
	protected static final String ADMIN_PASSWD = "ADMIN_PASSWD";
	protected static final String FORCE_FWD = "FORCE_FWD";
	protected static final String SESSION_LOST = "SESSION_LOST";
	protected static final String RECOVER_PARAM = "recover";
	
	public static Logger logger = Logger.getLogger(HttpServletManager.class);
	protected static Properties prop = new Properties();
	static {
		prop.setProperty("log4j.appender.frameweb","org.apache.log4j.ConsoleAppender");
		prop.setProperty("log4j.appender.frameweb.layout","org.apache.log4j.PatternLayout");
		//prop.setProperty("log4j.appender.frameweb.layout.ConversionPattern","[%d{dd/MM/yyyy hh:mm:ss}][%p] (%C.%M:%L) %m%n");
		prop.setProperty("log4j.appender.frameweb.layout.ConversionPattern","[%d{dd/MM/yyyy hh:mm:ss}][%p] %c{1}: %m%n");
	}
	
	public static void setConfigureProp(String mod,String level) {
		if (level!=null) {
			prop.setProperty("log4j.logger."+mod,level);
			PropertyConfigurator.configure(prop);
		}
	}
	
	protected Properties confGeneral = new Properties();
	protected Hashtable confModules = new Hashtable();
	protected BrokerTable brkTable = new BrokerTable(this,this.getClass().getName());
	protected TablaEstadistica staTable = new TablaEstadistica(this.getClass().getName());
	protected GestorDocumentacion document = new GestorDocumentacion(this.getClass().getClassLoader(),this.getClass().getName());
	protected boolean forceForward = false;
	
	public void initHB(String id) {
		// Carga las Sessiones de Hibernate !!
		logger.info("Cargando Sesiones Hibernate "+id+"...");
		IBroker ib = brkTable.getBroker(id);
		org.hibernate.Session s = ib.getSession();
		ib.freeSession(s);
	}
	
	public void init(ServletConfig conf) throws ServletException {
		super.init(conf);
		// Propiedades del Log4j.
		prop.setProperty("log4j.rootLogger","ERROR,frameweb");
		PropertyConfigurator.configure(prop);
		// Se redirige todo al log4j.
		//ManagerLogger outLog = new ManagerLogger(System.out);
		//System.setOut(outLog); System.setErr(outLog);
		// Configuracin General
		try {
			// Se define en el:
			//	- Mximo Upload General
			// 	- Directorio de Upload General
			//	- Nombres de Conexiones a BBDD
			// Utilizo un properties por cuestiones rendimiento.
			confGeneral.load(getClass().getResource(DB_CONFIG).openStream());
			brkTable.setProperties(confGeneral);
			// Establece el Destino de Logger y su Nivel default.
			prop.setProperty("log4j.rootLogger",confGeneral.getProperty(LOG_LEVEL,"FATAL")+",frameweb");
			PropertyConfigurator.configure(prop);
			String forceFwd = confGeneral.getProperty(FORCE_FWD,"false").toLowerCase();
			forceForward = forceFwd.toLowerCase().equals("true");
		} catch (Exception ex) {
			// Error en la carga del fichero general de conf.
			logger.fatal(ex.getMessage());
		}
	}
	
	// Dispone de un nico mtodo service para realizar la tarea.
	public void service(HttpServletRequest request,HttpServletResponse response) throws IOException,ServletException {
		String modulo = "", action = "", next = "";
		long init = System.currentTimeMillis();
		try {
			// Recojo los parametros.
			String uploadDir = confGeneral.getProperty(UPLOAD_DIR,".");
			if (!uploadDir.endsWith(File.separator)) uploadDir += File.separator;
			File ufile = new File(uploadDir);
			if (!ufile.exists() || !ufile.isDirectory()) throw new ManagerException("HttpServletManager","El Directorio de Upload ["+uploadDir+"] no existe o no es un directorio.");
			String maxUploadLength = confGeneral.getProperty(UPLOAD_LENGTH,""+MAX_UPLOAD);
			ManagerUtilities.getAllParameters(request,uploadDir,Long.parseLong(maxUploadLength));
			// Captura el Modulo y la accion ubicandolos en su lugar
			ManagerUtilities.setModuleOperation(request,getClass().getName().substring(0,getClass().getName().lastIndexOf(".")));
			modulo = ManagerUtilities.getModuleParameter(request);
			action = ManagerUtilities.getActionParameter(request);
			next = ManagerUtilities.getNextParameter(request);
			// Compruebo si ForceForward
			if (forceForward) ManagerUtilities.isValidForward(getClass().getName().substring(0,getClass().getName().lastIndexOf(".")),getClass().getClassLoader(),request,modulo,action);
			// Incorpora a Sesin si no estn los datos de BBDD y Estadisticos !!
			ManagerUtilities.setMetaObject(this.getClass().getName(),request,brkTable);
			ManagerUtilities.setMetaObject(this.getClass().getName(),request,staTable);
			ManagerUtilities.setMetaObject(this.getClass().getName(),request,document);
			// Leo el fichero de configuracin adecuado.
			ManagerModule fwm = new ManagerModule(this,request,response,brkTable,staTable);
			// Ejecuto la tarea descrita.
			fwm.doActions();
			// Me guardo la accin ejecutada !!
			if (forceForward) {
				String newModule = ManagerUtilities.getModuleParameter(request);
				String newAction = ManagerUtilities.getActionParameter(request);
				ManagerUtilities.saveNextForward(request,newModule,newAction);
			}
			// Envio la respuesta al cliente.
			long initOut = System.currentTimeMillis();
			// Si hay una sesin perdida y se pide recuperar
			Hashtable pars = (Hashtable) request.getSession(true).getAttribute(SESSION_LOST);
			String recover = request.getParameter(RECOVER_PARAM);
			if (recover==null) recover = "";
			if (pars!=null) {
				if (!recover.equals("")) fwm.sendFWDResponse(pars,getServletContext());
				else { 
					request.getSession(true).removeAttribute(HttpServletManager.SESSION_LOST);
					fwm.sendResponse(getServletContext());
				}
			} else fwm.sendResponse(getServletContext());
			staTable.pararCronoResponse(System.currentTimeMillis()-initOut,modulo,action,next);
		} catch (ManagerException mex) {
			String sessionPage = confGeneral.getProperty(SESSION_LOST,"NULL");
			if (mex.getError().indexOf("No se puede comprobar el acceso al modulo mediante el objeto de sesin [")>0 && !sessionPage.equals("NULL")) {
				HttpSession miSession = request.getSession(true);
				Hashtable params = new Hashtable();
				Enumeration en = request.getAttributeNames();
				while (en.hasMoreElements()) {
					String name = (String) en.nextElement();
					params.put(name,request.getAttribute(name));
				}
				miSession.setAttribute(SESSION_LOST,params);
				response.sendRedirect(sessionPage);
			} else {
				logger.fatal(mex.getError());
				mex.sendErrorPage(getClass().getClassLoader(),getServletContext(),request,response,confGeneral.getProperty(URL_ERROR,""));
			}
		} finally {
			ManagerUtilities.deleteTempFiles(request);
			staTable.pararCrono(System.currentTimeMillis()-init,modulo,action,next);
		}
		
	}
	
	public boolean isAdministrador(String u,String p) {
		String usr = confGeneral.getProperty(ADMIN_LOGIN);
		if (usr == null) usr = "admin";
		String pwd = confGeneral.getProperty(ADMIN_PASSWD);
		if (pwd == null) pwd = "admin";
		return (u.equals(usr)&&p.equals(pwd));
	}
	
}
