/*
 * Created on 14-feb-2005
 *
 */
package atica.frameweb.manager;


import org.w3c.dom.*;

import javax.servlet.*;
import javax.servlet.http.*;
import atica.frameweb.xml.*;
import atica.frameweb.output.IOutput;
import atica.frameweb.user.*;
import atica.frameweb.user.menu.*;
import atica.frameweb.utils.*;
import atica.frameweb.broker.*;
import atica.frameweb.estadisticas.*;
import java.lang.reflect.*;
import java.sql.*;
import java.util.*;
import org.apache.log4j.*;
//import net.sf.hibernate.*;
import org.hibernate.*;
/**
 * @author Juan Jos Meroo Snchez
 *
 * Hace todas las tareas referentes al modulo.
 * 
 */
public class ManagerModule {
	
	static Logger logger = HttpServletManager.logger;
	
	protected HttpServletRequest request = null;
	protected HttpServletResponse response = null;
	protected Node modulo = null;
	protected Node operation = null;
	protected String action = "",module="",logLevel="";
	protected Vector objectsEjecuted = new Vector();
	protected Vector modulesEjecuted = new Vector();
	protected Boolean notFirstPage = null;
	protected BrokerTable brkTable = null;
	protected TablaEstadistica staTable = null;
	protected Hashtable ciclos = null;
	protected Hashtable parameters = new Hashtable();
	protected boolean orgCall = true;
	protected ClassLoader classLoader = null;
	protected HttpServlet servlet = null;
	
	public ManagerModule(HttpServlet srv,HttpServletRequest req,HttpServletResponse res,BrokerTable brkt,TablaEstadistica stab) throws ManagerException {
		request = req; 
		response = res; 
		brkTable = brkt; staTable = stab;
		servlet = srv;
		classLoader = srv.getClass().getClassLoader();
		
	}
	
	private void loadModule() throws ManagerException {
		action = ManagerUtilities.getActionParameter(request);
		module = ManagerUtilities.getModuleParameter(request);
		modulo = ManagerReader.readConfigModule(classLoader,request);
		operation = XMLUtil.selectSingleNode(modulo,ManagerTags.ACTION,ManagerTags.ACTION_NAME,action);
		if (operation==null) throw new ManagerException("ManagerModule["+module+"."+action+"]","La Accin no est definida en el mdulo.");
		// Comprobacin de Seguridad !!
		String accessMod = XMLUtil.getAttributeValue(modulo,ManagerTags.MODULE_ACC);
		String profileMod = XMLUtil.getAttributeValue(modulo,ManagerTags.MODULE_PRF);
		String accessAct = XMLUtil.getAttributeValue(operation,ManagerTags.ACTION_ACC);
		String profileAct = XMLUtil.getAttributeValue(operation,ManagerTags.ACTION_PRF);
		// OJO: Prevalece lo que se diga en Action sobre Module !!!
		if ((accessMod!=null) || (accessAct!=null)) {
			// Es un modulo/accin protegido
			if (accessAct!=null) {
				if (profileAct!=null) accessProfileProtection(request,accessAct,profileAct);
				else accessProtection(request,accessAct);
			} else {
				if (profileMod!=null) accessProfileProtection(request,accessMod,profileMod);
				else accessProtection(request,accessMod);
			}
		} else logger.warn("ManagerModule["+module+"."+action+"] Modulo sin proteccin de Acceso.");
		logLevel = XMLUtil.getAttributeValue(modulo,ManagerTags.MODULE_LOG);
		HttpServletManager.setConfigureProp(module.replace('/','.'),logLevel);
	}
	
	public void setObjectsEjecuted(Vector v) {
		objectsEjecuted = v;
	}
	
	public void setModulesEjecuted(Vector v) {
		modulesEjecuted = v;
	}
	
	public void addOutParameter(String name,String value) {
		parameters.put(name,value);
	}
	
	private void accessProfileProtection(HttpServletRequest req,String acc,String prf) throws ManagerException {
		try {
			IManagerAccessControl mac = (IManagerAccessControl)req.getSession(true).getAttribute(acc);
			if (mac==null) throw new ManagerException("ManagerModule["+module+"."+action+"]","No se puede comprobar el acceso al modulo mediante el objeto de sesin ["+acc+"] ya que no existe.");
			String [] perfiles = ManagerUtilities.getParameterValues(req,prf);
			if (perfiles==null) {
				try {
					perfiles = (String[])req.getSession(true).getAttribute(prf);
				} catch (ClassCastException ex) {
					perfiles = new String[1];
					try {
						perfiles[0] = (String)req.getSession(true).getAttribute(prf);
					} catch (ClassCastException exx) {
						throw new ManagerException("ManagerModule["+module+"."+action+"]","No se puede comprobar el acceso al modulo mediante el perfil en sesin ["+prf+"] ya que no es un String o Array de String.");
					}
				}
			}
			if (perfiles==null) throw new ManagerException("ManagerModule["+module+"."+action+"]","No se puede comprobar el acceso al modulo mediante el perfil en request o sesin ["+prf+"] ya que no existe.");
			if (!mac.permitidoPerfiles(perfiles,module,action)) throw new ManagerException("ManagerModule["+module+"."+action+"]","Usted no tiene permiso para ejecutar la accin solicitada.");
		} catch (ClassCastException ex) {
			throw new ManagerException("ManagerModule["+module+"."+action+"]","No se puede comprobar el acceso al modulo mediante el objeto de sesin ["+acc+"] ya que no hereda del Usuario de FrameWeb.",ex);
		} catch (ManagerException mex) {
			throw mex;
		} catch (Exception ex) {
			throw new ManagerException("ManagerModule["+module+"."+action+"]","No se puede comprobar el acceso al modulo debido a un error.",ex);
		}
	}
	
	private void accessProtection(HttpServletRequest req,String acc) throws ManagerException {
		try {
			IManagerAccessControl mac = (IManagerAccessControl)req.getSession(true).getAttribute(acc);
			if (mac==null) throw new ManagerException("ManagerModule["+module+"."+action+"]","No se puede comprobar el acceso al modulo mediante el objeto de sesin ["+acc+"] ya que no existe.");
			if (!mac.permitido(module,action)) throw new ManagerException("ManagerModule["+module+"."+action+"]","Usted no tiene permiso para ejecutar la accin solicitada.");
		} catch (ClassCastException ex) {
			throw new ManagerException("ManagerModule["+module+"."+action+"]","No se puede comprobar el acceso al modulo mediante el objeto de sesin ["+acc+"] ya que no hereda del Usuario de FrameWeb.",ex);
		} catch (ManagerException mex) {
			throw mex;
		} catch (Exception ex) {
			throw new ManagerException("ManagerModule["+module+"."+action+"]","No se puede comprobar el acceso al modulo debido a un error.",ex);
		}
	}
	
	public void setOriginalCall() {
		if (orgCall) {
			// Incluye en el request la llamada original
			orgCall = false;
			ManagerUtilities.moveModuleToOrg(request);
		}
	}
	
	public boolean sinCiclos(String mod,String act,String nxt) {
		if (ciclos==null) {
			// Introduzco la llamada original
			ciclos = new Hashtable();
			Vector v1 = new Vector();
			v1.addElement(action+"."+nxt);
			ciclos.put(module,v1);
		}
		Vector v = (Vector)ciclos.get(mod);
		if (v==null) v = new Vector();
		if (v.contains(act)) return false;
		else {
			v.addElement(act);
			ciclos.put(mod,v);
		}
		return true;
	}
	
	public void doForward() throws ManagerException {
		String next = ManagerUtilities.getNextParameter(request);
		if ((next==null) || next.equals("")) return;
		Node fwd = XMLUtil.selectSingleNode(operation,ManagerTags.FORWARD);
		if (fwd==null) throw new ManagerException("ManagerModule","No Existe una redireccin definida para next ["+next+"] en action ["+action+"].");
		NodeList lst = XMLUtil.selectNodes(fwd,ManagerTags.NEXT);
		for (int k=0; k<lst.getLength(); k++) {
			String name = XMLUtil.getAttributeValue(lst.item(k),ManagerTags.NEXT_NAME);
			if ((name==null) || !name.equals(next)) continue;
			String newModule = XMLUtil.getAttributeValue(lst.item(k),ManagerTags.NEXT_MODULE);
			if (newModule==null) newModule = ManagerUtilities.getModuleParameter(request);
			String newAction = XMLUtil.getAttributeValue(lst.item(k),ManagerTags.NEXT_ACTION);
			if (newAction==null) throw new ManagerException("ManagerModule","No Existe una action definida para next ["+next+"] en action ["+action+"].");
			if (sinCiclos(newModule,newAction,next)) {
				ManagerUtilities.setModuleParameter(request,newModule);
				ManagerUtilities.setActionParameter(request,newAction);
				ManagerUtilities.setNextParameter(request,"");
				loadModule();
				return;
			} else throw new ManagerException("ManagerModule","Se ha provocado una llamada ciclo de Forward: ["+newModule+"."+newAction+"."+next+"].");
		}
		throw new ManagerException("ManagerModule","No Existe una redireccin definida para next ["+next+"] en action ["+action+"].");
	}
	
	public void doActions() throws ManagerException {
		// Hago los Posibles Forwards iniciales
		setOriginalCall();
		loadModule();
		doForward();
		setMenuOption();
		NodeList nlist = operation.getChildNodes();
		for (int k=0; k<nlist.getLength(); k++) {
			Node nodo = nlist.item(k);
			if (nodo.getNodeName().equals(ManagerTags.OUTPUT)) return;
			if (!nodo.getNodeName().equals(ManagerTags.OBJECT)) continue;
			else doObject(nodo);
		}
	}
	
	private void setMenuOption() {
		IDataMenu dm = null;
		String txt = XMLUtil.getAttributeValue(modulo,ManagerTags.MODULE_MENU);
		if (txt!=null) {
			dm = (IDataMenu)request.getSession(true).getAttribute(txt);
			String value = ManagerUtilities.getParameter(request,ManagerTags.MENU.toLowerCase());
			if ((value!=null) && (dm!=null)) {
				dm.getDatosMenu().setOption(value);
			}
		}
	}
	
	private void doObject(Node nodo) throws ManagerException {
		String clase = "";
		try {
			Object object = null;
			clase = XMLUtil.getAttributeValue(nodo,ManagerTags.OBJECT_CLASS);
			String paginable = XMLUtil.getAttributeValue(nodo,ManagerTags.OBJECT_PAG);
			String pageRange = XMLUtil.getAttributeValue(nodo,ManagerTags.OBJECT_PAGRANGE);
			String defPage = XMLUtil.getAttributeValue(nodo,ManagerTags.OBJECT_DEFPAG);
			String userSt = XMLUtil.getAttributeValue(nodo,ManagerTags.OBJECT_USER);
			boolean isLoadPage = ((paginable!=null) && (paginable.equals(ManagerTags.LOAD)));
			boolean isPaginable = (paginable!=null);
			boolean isFirstPage = false;
			String loadObj = XMLUtil.getAttributeValue(nodo,ManagerTags.OBJECT_LD);
			if ((loadObj==null) && (isLoadPage)) loadObj = ManagerTags.SESSION;
			if (loadObj!=null) {
				if (loadObj.equals(ManagerTags.REQUEST)) {
					object = ManagerUtilities.getRequestObject(request,clase);
				} else if (loadObj.equals(ManagerTags.SESSION)) {
					object = request.getSession(true).getAttribute(clase);
				} else if (loadObj.equals(ManagerTags.SESSION_REQ)) {
					object = request.getSession(true).getAttribute(clase);
					if (object==null) object = ManagerUtilities.getRequestObject(request,clase);
				} else if (loadObj.equals(ManagerTags.REQ_SESSION)) {
					object = ManagerUtilities.getRequestObject(request,clase);
					if (object==null) object = request.getSession(true).getAttribute(clase);
				}
			}

			if (object==null) {
				object = classLoader.loadClass(clase).getConstructor(new Class[]{}).newInstance(new Object[]{});
				isLoadPage = false;
			}
			if (userSt!=null) {
				try {
					// Ejecuta el mtodo setUser de forma automtica !!
					((GeneralManager)object).setUser((User)request.getSession(true).getAttribute(userSt));
				} catch (ClassCastException ex) {}
			}
			if (isPaginable) isFirstPage = doPageMethods(object,clase,defPage,pageRange);
			if (!isLoadPage || isFirstPage) {
				NodeList nlist = nodo.getChildNodes();
				for (int k=0; k<nlist.getLength(); k++) {
					Node metodo = nlist.item(k);
					if (metodo.getNodeType()!=Node.ELEMENT_NODE) continue;
					if (metodo.getNodeName().equals(ManagerTags.DB_METHOD)) {
						doObjectDBMethod(Connection.class,object,clase,metodo);		
					} else if (metodo.getNodeName().equals(ManagerTags.DBH_METHOD)) {
						doObjectDBMethod(Session.class,object,clase,metodo);
					} else if (metodo.getNodeName().equals(ManagerTags.OUT_METHOD)) {
						doObjectOUTMethod(object,clase,metodo);
					} else {
						doObjectMethod(object,clase,metodo);
					}
				}
			}
			if (!objectsEjecuted.contains(object)) {
				objectsEjecuted.addElement(object);
				modulesEjecuted.addElement(module);
			}
			String save = XMLUtil.getAttributeValue(nodo,ManagerTags.OBJECT_SV);
			if ((save==null) && (isLoadPage)) save = ManagerTags.SESSION;
			if (save!=null) {
				if (save.equals(ManagerTags.SESSION)) {
					setSessionAttribute(clase,"",object);
				} else if (save.equals(ManagerTags.REQUEST)) {
					setRequestAttribute(clase,"",object);
				} else if (save.startsWith(ManagerTags.SESSION)) {
					setSessionAttribute(clase+save.substring(ManagerTags.SESSION.length()),"",object);
				} else if (save.startsWith(ManagerTags.REQUEST)) {
					setRequestAttribute(clase+save.substring(ManagerTags.REQUEST.length()),"",object);
				}
			} 
		} catch (ManagerException mex) {
			throw mex;
		} catch (Exception ex) {
			if (ex.getCause() instanceof ManagerException) throw (ManagerException)ex.getCause();
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error ejecutando Clase["+clase+"].",ex);
		}

	}
	
	private boolean doPageMethods(Object object,String clase,String pag,String def) throws ManagerException {
		String methodName = GeneralPaginator.SET_PAGE_PARAMETERS; 
		try {
			String p = ManagerUtilities.getParameter(request,GeneralPaginator.PAGE_PARAMETER);
			if ((p==null) || p.equals("")) p = (String) request.getSession(true).getAttribute(GeneralPaginator.PAGE_PARAMETER);
			if ((p==null) || p.equals("")) p = pag;
			if ((p==null) || p.equals("")) throw new ManagerException("doPageMethods","No se ha indicado un valor de Pagina (page).");
			String s = ManagerUtilities.getParameter(request,GeneralPaginator.PAGE_RANGE_PARAMETER);
			if ((s==null) || s.equals("")) s = (String) request.getSession(true).getAttribute(GeneralPaginator.PAGE_RANGE_PARAMETER);
			if ((s==null) || s.equals("")) s = def;
			if ((s==null) || s.equals("")) throw new ManagerException("doPageMethods","No se ha indicado un valor de Tamao de Pgina (pagesize).");
			IPaginable ipag = (IPaginable) object;
			return ipag.setPageParameters(p,s);
		} catch (ManagerException ex) {
			throw ex;
		} catch (Exception ex) {
			if (ex.getCause() instanceof ManagerException) throw (ManagerException)ex.getCause();
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error ejecutando Clase["+clase+"] Metodo["+methodName+"].",ex);
		}
	}
	
	private void doObjectDBMethod(Class typeConn,Object object,String clase,Node metodo) throws ManagerException {
		String methodName = "";
		Object conn = null;
		IBroker brk = null;
		long initDB = System.currentTimeMillis();
		try {
			
			String dsName = XMLUtil.getAttributeValue(modulo,ManagerTags.MODULE_DS);
			if (dsName==null) throw new Exception("No se ha especificado DataSource para Conectar a BBDD.");
			brk = brkTable.getBroker(dsName);
			methodName = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_NAME);
			String methodType = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_TYPE);
			Class [] methodTypes = (methodType==null)?new Class[]{typeConn}:new Class[]{typeConn,classLoader.loadClass(methodType)};
			Object [] arguments = null; Object beanObject = null;
			
			if (typeConn.getName().equals(Connection.class.getName()))
				conn = brk.getConnection();
			else conn = brk.getSession();
			
			if (methodType!=null) {
				String fromParam = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_FROM);
				String paramName = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_PARAM);
				if ((paramName!=null) && (fromParam!=null)) {
					if (fromParam.equals(ManagerTags.SESSION)) {
						Object p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{conn,p};
					} else if (fromParam.equals(ManagerTags.REQUEST)) {
						Object p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{conn,p};
					} else if (fromParam.equals(ManagerTags.REQ_SESSION)) {
						Object p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{conn,p};
					} else if (fromParam.equals(ManagerTags.SESSION_REQ)) {
						Object p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{conn,p};
					} else arguments = new Object[]{conn,XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE)};
				} else if ((fromParam!=null) && fromParam.equals(ManagerTags.REQUEST)) {
					// Recupero un Bean del Request !!
					beanObject = getMethod(object,methodName,classLoader.loadClass(methodType));
					if (beanObject==null) {
						beanObject = classLoader.loadClass(methodType).getConstructor(new Class[]{}).newInstance(new Object[]{});
						setBeanProperties(null,beanObject);
						arguments = new Object[]{conn,beanObject};
					} else {
						arguments = null;
						setBeanProperties(null,beanObject);
					}
				} else arguments = new Object[]{conn,XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE)};
			} else arguments = new Object[]{conn};
			Object objRdo = (arguments==null)?beanObject:doMethod(methodName,object,methodTypes,arguments);
			String save = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_SAVE);
			if (save!=null) {
				if (save.equals(ManagerTags.SESSION)) {
					if (objRdo!=null) setSessionAttribute(clase,methodName,objRdo);
					else delSessionAttribute(clase,methodName);
				} else if (save.equals(ManagerTags.REQUEST)) {
					if (objRdo!=null) setRequestAttribute(clase,methodName,objRdo);
					else delRequestAttribute(clase,methodName);
				} else if (save.startsWith(ManagerTags.SESSION)) {
					if (objRdo!=null) setSessionAttribute(clase,methodName+save.substring(ManagerTags.SESSION.length()),objRdo);
					else delSessionAttribute(clase,methodName+save.substring(ManagerTags.SESSION.length()));
				} else if (save.startsWith(ManagerTags.REQUEST)) {
					if (objRdo!=null) setRequestAttribute(clase,methodName+save.substring(ManagerTags.REQUEST.length()),objRdo);
					else delRequestAttribute(clase,methodName+save.substring(ManagerTags.REQUEST.length()));
				}
			} 
		} catch (ManagerException ex) {
			throw ex;
		} catch (Exception ex) {
			if (ex.getCause() instanceof ManagerException) throw (ManagerException)ex.getCause();
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error ejecutando Clase["+clase+"] DB-Metodo["+methodName+"].",ex);
		} finally {
			if (conn!=null) {
				if (typeConn.getName().equals(Connection.class.getName()))
					brk.freeConnection((Connection)conn);
				else brk.freeSession((Session)conn);
			}
			staTable.pararCronoDataBase(System.currentTimeMillis()-initDB,module,action,methodName);
		}
	}
	
	private void doObjectOUTMethod(Object object,String clase,Node metodo) throws ManagerException {
		String methodName = "";
		try {
			methodName = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_NAME);
			String methodType = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_TYPE);
			Class [] methodTypes = (methodType==null)?new Class[]{HttpServlet.class,HttpServletRequest.class,HttpServletResponse.class}:new Class[]{HttpServlet.class,HttpServletRequest.class,HttpServletResponse.class,classLoader.loadClass(methodType)};
			Object [] arguments = null; Object beanObject = null;
			if (methodType!=null) {
				String fromParam = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_FROM);
				String paramName = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_PARAM);
				if ((paramName!=null) && (fromParam!=null)) {
					if (fromParam.equals(ManagerTags.SESSION)) {
						Object p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{servlet,request,response,p};
					} else if (fromParam.equals(ManagerTags.REQUEST)) {
						Object p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{servlet,request,response,p};
					} else if (fromParam.equals(ManagerTags.REQ_SESSION)) {
						Object p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{servlet,request,response,p};
					} else if (fromParam.equals(ManagerTags.SESSION_REQ)) {
						Object p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{servlet,request,response,p};
					} else arguments = new Object[]{servlet,request,response,XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE)};
				} else if ((fromParam!=null) && fromParam.equals(ManagerTags.REQUEST)) {
					// Recupero un Bean del Request !!
					beanObject = getMethod(object,methodName,classLoader.loadClass(methodType));
					if (beanObject==null) {
						beanObject = classLoader.loadClass(methodType).getConstructor(new Class[]{}).newInstance(new Object[]{});
						setBeanProperties(null,beanObject);
						arguments = new Object[]{servlet,request,response,beanObject};
					} else {
						arguments = null;
						setBeanProperties(null,beanObject);
					}
				} else arguments = new Object[]{servlet,request,response,XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE)};
			} else arguments = new Object[]{servlet,request,response};
			Object objRdo = (arguments==null)?beanObject:doMethod(methodName,object,methodTypes,arguments);
			String save = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_SAVE);
			if (save!=null) {
				if (save.equals(ManagerTags.SESSION)) {
					if (objRdo!=null) setSessionAttribute(clase,methodName,objRdo);
					else delSessionAttribute(clase,methodName);
				} else if (save.equals(ManagerTags.REQUEST)) {
					if (objRdo!=null) setRequestAttribute(clase,methodName,objRdo);
					else delRequestAttribute(clase,methodName);
				} else if (save.startsWith(ManagerTags.SESSION)) {
					if (objRdo!=null) setSessionAttribute(clase,methodName+save.substring(ManagerTags.SESSION.length()),objRdo);
					else delSessionAttribute(clase,methodName+save.substring(ManagerTags.SESSION.length()));
				} else if (save.startsWith(ManagerTags.REQUEST)) {
					if (objRdo!=null) setRequestAttribute(clase,methodName+save.substring(ManagerTags.REQUEST.length()),objRdo);
					else delRequestAttribute(clase,methodName+save.substring(ManagerTags.REQUEST.length()));
				}
			} 
		} catch (ManagerException ex) {
			throw ex;
		} catch (Exception ex) {
			if (ex.getCause() instanceof ManagerException) throw (ManagerException)ex.getCause();
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error ejecutando Clase["+clase+"] OUT-Metodo["+methodName+"].",ex);
		}
	}
	
	private void doObjectMethod(Object object,String clase,Node metodo) throws ManagerException {
		String methodName = "";
		try {
			methodName = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_NAME);
			String methodType = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_TYPE);
			Class [] methodTypes = (methodType==null)?new Class[]{}:new Class[]{classLoader.loadClass(methodType)};
			Object [] arguments = null; Object beanObject = null;
			if (methodType!=null) {
				String fromParam = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_FROM);
				String paramName = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_PARAM);
				if ((paramName!=null) && (fromParam!=null)) {
					if (fromParam.equals(ManagerTags.SESSION)) {
						Object p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{p};
					} else if (fromParam.equals(ManagerTags.REQUEST)) {
						Object p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{p};
					} else if (fromParam.equals(ManagerTags.REQ_SESSION)) {
						Object p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{p};
					} else if (fromParam.equals(ManagerTags.SESSION_REQ)) {
						Object p = request.getSession(true).getAttribute(paramName);
						if (p==null) p = ManagerUtilities.getRequestObject(request,paramName,classLoader.loadClass(methodType));
						if (p==null) p = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE);
						arguments = new Object[]{p};
					} else arguments = new Object[]{XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE)};
				} else if ((fromParam!=null) && fromParam.equals(ManagerTags.REQUEST)) {
					// Recupero un Bean del Request !!
					beanObject = getMethod(object,methodName,classLoader.loadClass(methodType));
					if (beanObject==null) {
						beanObject = classLoader.loadClass(methodType).getConstructor(new Class[]{}).newInstance(new Object[]{});
						setBeanProperties(null,beanObject);
						arguments = new Object[]{beanObject};
					} else {
						arguments = null;
						setBeanProperties(null,beanObject);
					}
				} else arguments = new Object[]{XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_VALUE)};
			} else arguments = new Object[]{};
			Object objRdo = (arguments==null)?beanObject:doMethod(methodName,object,methodTypes,arguments);
			String save = XMLUtil.getAttributeValue(metodo,ManagerTags.METHOD_SAVE);
			if (save!=null) {
				if (save.equals(ManagerTags.SESSION)) {
					if (objRdo!=null) setSessionAttribute(clase,methodName,objRdo);
					else delSessionAttribute(clase,methodName);
				} else if (save.equals(ManagerTags.REQUEST)) {
					if (objRdo!=null) setRequestAttribute(clase,methodName,objRdo);
					else delRequestAttribute(clase,methodName);
				} else if (save.startsWith(ManagerTags.SESSION)) {
					if (objRdo!=null) setSessionAttribute(clase,methodName+save.substring(ManagerTags.SESSION.length()),objRdo);
					else delSessionAttribute(clase,methodName+save.substring(ManagerTags.SESSION.length()));
				} else if (save.startsWith(ManagerTags.REQUEST)) {
					if (objRdo!=null) setRequestAttribute(clase,methodName+save.substring(ManagerTags.REQUEST.length()),objRdo);
					else delRequestAttribute(clase,methodName+save.substring(ManagerTags.REQUEST.length()));
				}
			} 
		} catch (ManagerException ex) {
			throw ex;
		} catch (Exception ex) {
			if (ex.getCause() instanceof ManagerException) throw (ManagerException)ex.getCause();
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error ejecutando Clase["+clase+"] Metodo["+methodName+"].",ex);
		}
	}

	private String singleName(String name,String subname) {
		String last = name.substring(name.lastIndexOf(".")+1);
		if ((subname!=null) && !subname.equals("")) last = subname;
		return last; 
	}

	private String resumeName(String name,String subname) {
		String last = name.substring(name.lastIndexOf(".")+1);
		if ((subname!=null) && !subname.equals(""))
			last += "."+subname;
		return last; 
	}

	private String completeName(String name,String subname) {
		if ((subname==null) || subname.equals("")) return name;
		return name+"."+subname; 
	}
	
	private void setRequestAttribute(String name,String subname,Object value) {
		request.setAttribute(completeName(name,subname),value);
		request.setAttribute(completeName(name,subname).replace('.','_'),value);
		request.setAttribute(resumeName(name,subname),value);
		request.setAttribute(resumeName(name,subname).replace('.','_'),value);
		request.setAttribute(singleName(name,subname),value);
	}
	
	private void delRequestAttribute(String name,String subname) {
		request.removeAttribute(completeName(name,subname));
		request.removeAttribute(completeName(name,subname).replace('.','_'));
		request.removeAttribute(resumeName(name,subname));
		request.removeAttribute(resumeName(name,subname).replace('.','_'));
		request.removeAttribute(singleName(name,subname));
	}
	
	private void setSessionAttribute(String name,String subname,Object value) {
		request.getSession(true).setAttribute(completeName(name,subname),value);
		request.getSession(true).setAttribute(completeName(name,subname).replace('.','_'),value);
		request.getSession(true).setAttribute(resumeName(name,subname),value);
		request.getSession(true).setAttribute(resumeName(name,subname).replace('.','_'),value);
		request.getSession(true).setAttribute(singleName(name,subname),value);
	}
	
	private void delSessionAttribute(String name,String subname) {
		request.getSession(true).removeAttribute(completeName(name,subname));
		request.getSession(true).removeAttribute(completeName(name,subname).replace('.','_'));
		request.getSession(true).removeAttribute(resumeName(name,subname));
		request.getSession(true).removeAttribute(resumeName(name,subname).replace('.','_'));
		request.getSession(true).removeAttribute(singleName(name,subname));
	}
	
	public void setBeanProperties(String name,Object obj) throws Exception {
		Method [] mts = obj.getClass().getMethods();
		for (int k=0; k<mts.length; k++) {
			if (mts[k].getName().startsWith("set")) {
				// Parece un Setter !!
				String prop = mts[k].getName().substring(3,4).toLowerCase()+mts[k].getName().substring(4);
				if (name!=null) prop = name.substring(3,4).toLowerCase()+name.substring(4)+"_"+prop;
				Class [] pt = mts[k].getParameterTypes();
				if ((pt.length==1) && (ManagerUtilities.existsRequestElement(request,prop) || ManagerUtilities.existsRequestElement(request,prop+"_"))) {
					String metName = (name!=null)?name+"_"+mts[k].getName().substring(3,4).toLowerCase()+mts[k].getName().substring(4):mts[k].getName();
					Object objReq = ManagerUtilities.getRequestObjectBean(metName,request,prop,pt[0],classLoader,this);
					if (objReq!=null) {
						mts[k].invoke(obj,new Object[]{objReq});
					}
				}
			}
		}
		
	}
	
	private Object doMethod(String methodName,Object object,Class[] types,Object[] arguments) throws Exception {
		Method m = object.getClass().getMethod(methodName,types);
		return m.invoke(object,arguments);
	}
	
	private Object getMethod(Object object,String methodName,Class type) throws Exception {
		Method [] mts = object.getClass().getMethods();
		for (int k=0; k<mts.length; k++) {
			if (mts[k].getName().equals(methodName)) {
				Class [] tips = mts[k].getParameterTypes();
				if ((tips.length==0) && // No tiene Argumentos y retorna el tipo deseado
					(mts[k].getReturnType().getName().equals(type.getName()))) {
					Object ob = mts[k].invoke(object,new Object[]{});
					if (ob==null) {
						// El mtodo existe pero ha devuelto Null !!!
						ob = type.getConstructor(new Class[]{}).newInstance(new Object[]{});
					}
					return ob;
				}
			}
		}
		return null;
	}
	
	public void sendFWDResponse(Hashtable params,ServletContext srvCtx) throws ManagerException {
		String txt = "";
		try {

			String mod = ((String[])params.get(ManagerUtilities.MODULE_PARAMETER))[0];
			String act = ((String[])params.get(ManagerUtilities.ACTION_PARAMETER))[0];
			Node file = ManagerSendResponse.createFileNode(mod,act);
			ManagerSendResponse.translateParam(request,params);
			request.getSession(true).removeAttribute(HttpServletManager.SESSION_LOST);
			
			Class outClass = Class.forName("atica.frameweb.output.FWDOutput");
			IOutput out = (IOutput)outClass.getConstructor(new Class[]{}).newInstance(new Object[]{});
			
			out.setFileNode(file);
			out.setEjecutedObjects(objectsEjecuted);
			out.setEjecutedModules(modulesEjecuted);
			out.setManagerModule(this);
			out.sendResponse(classLoader,logLevel,srvCtx,request,response);
				
		} catch (ClassCastException ex) {
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error Cargando Diccionario a partir del objeto de sesin ["+txt+"].",ex);
		} catch (ManagerException ex) {
			throw ex;
		} catch (Exception ex) {
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error Inesperado Enviando Respuesta.",ex);
		}
	}
	
	public void sendResponse(ServletContext srvCtx) throws ManagerException {
		String txt = "";
		try {

			String fileId = ManagerSendResponse.obtenerFileId(modulo,request);
			Node file = ManagerSendResponse.obtenerFileNode(operation,fileId);
			if (file==null) {
				logger.warn("No existe salida para "+fileId+" debe haberse emitido con OUT-METHOD.");
				return;
			}
			
			IDictionary dic = null;
			txt = XMLUtil.getAttributeValue(modulo,ManagerTags.MODULE_DIC);
			if (txt!=null) dic = (IDictionary)request.getSession(true).getAttribute(txt);

			IDataMenu dm = null;
			txt = XMLUtil.getAttributeValue(modulo,ManagerTags.MODULE_MENU);
			if (txt!=null) dm = (IDataMenu)request.getSession(true).getAttribute(txt);
			
			String contentType = XMLUtil.getAttributeValue(file,ManagerTags.FILE_CTYPE);
			if (contentType==null) contentType="text/html";
			
			String type = XMLUtil.getAttributeValue(file,ManagerTags.FILE_TYPE);
			if (type==null) throw new Exception("No se ha indicado el tipo de Salida para el cliente ["+fileId+"].");
		
			Class outClass = Class.forName("atica.frameweb.output."+type+"Output");
			IOutput out = (IOutput)outClass.getConstructor(new Class[]{}).newInstance(new Object[]{});
			
			out.setContentType(contentType);
			out.setFileNode(file);
			out.setDictionary(dic);
			out.setDataMenu(dm);
			out.setEjecutedObjects(objectsEjecuted);
			out.setEjecutedModules(modulesEjecuted);
			out.setManagerModule(this);
			out.sendResponse(classLoader,logLevel,srvCtx,request,response);
				
		} catch (ClassCastException ex) {
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error Cargando Diccionario a partir del objeto de sesin ["+txt+"].",ex);
		} catch (ManagerException ex) {
			throw ex;
		} catch (Exception ex) {
			throw new ManagerException("ManagerModule["+module+"."+action+"]","Error Inesperado Enviando Respuesta.",ex);
		}
	}

	public void setNewLocation(String nm,String na) {
		String newUrl = ManagerUtilities.getUrlParameters(request,nm,na);
		response.setHeader("Location",newUrl);
		response.setHeader("Content-Location",newUrl);
	}
	
}
