/*
 * Created on 12-feb-2005
 *
 */
package atica.frameweb.manager;

import java.net.*;
import org.w3c.dom.*; 
import javax.servlet.http.*;
import atica.frameweb.xml.*;
import atica.frameweb.user.User;

/**
 * @author Juan Jos Meroo Snchez
 *
 * Utilidad para la lectura del fichero de conf. de un mdulo.
 * 
 */
public class ManagerReader {
	
	protected static final String CONFIG_MODULE = "config.xml";
	protected static final String ACCESS_MODULE = "access.xml";
	protected static final String ROLE_MODULE = "roles.xml";
	protected static final String PROFILE_ROLE_MODULE = "profileRoles.xml";
	protected static final String MAPPING_MODULE = "mapping.xml";
	protected static final String DICTIONARY_MODULE = "dictionary";
	protected static final String QUERY_MODULE = "query.xml";
	protected static final String DATA_MENU = "menu.xml";
	
	public static URL getMappingResource(ClassLoader cl,String mod) {
		return cl.getResource(mod+ManagerUtilities.FILE_SEPARATOR+MAPPING_MODULE);
	}	
	public static Node readConfigModule(ClassLoader cl,HttpServletRequest req) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Node modXML = null;
		String modName = "";
		try {
			
			modName = ManagerUtilities.getModuleParameter(req);
			URL urlConfig = cl.getResource(modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE);
			Document miXML = XMLUtil.parse(urlConfig.openStream());
			modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.MODULE,ManagerTags.MODULE_PCK,modName);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE+"] no contiene la descripcin del mdulo ["+modName+"].");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Configuracin del Mdulo.",ex);
		}
		
		return modXML;
		
	}

	public static Document readFileModules(ClassLoader cl,String modName) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Document miXML = null;
		try {
			
			URL urlConfig = cl.getResource(modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE);
			if (urlConfig==null) throw new ManagerException("ManagerReader","El Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE+"] no existe.");
			miXML = XMLUtil.parse(urlConfig.openStream());
			Node modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.MODULES);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE+"] no contiene la descripcin de los mdulos.");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Configuracin del Mdulo.",ex);
		}
		
		return miXML;
		
	}

	public static Document readFileModule(ClassLoader cl,String modName) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Document miXML = null;
		try {
			
			URL urlConfig = cl.getResource(modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE);
			if (urlConfig==null) throw new ManagerException("ManagerReader","El Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE+"] no existe.");
			miXML = XMLUtil.parse(urlConfig.openStream());
			Node modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.MODULE,ManagerTags.MODULE_PCK,modName);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE+"] no contiene la descripcin de los mdulos.");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+CONFIG_MODULE+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Configuracin del Mdulo.",ex);
		}
		
		return miXML;
		
	}

	public static Node readAccessModule(User user,String modName) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Node modXML = null;
		try {
			
			URL urlConfig = user.getClass().getClassLoader().getResource(modName+ManagerUtilities.FILE_SEPARATOR+ACCESS_MODULE);
			Document miXML = XMLUtil.parse(urlConfig.openStream());
			modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.MODULE,ManagerTags.MODULE_PCK,modName);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Accesos ["+modName+ManagerUtilities.FILE_SEPARATOR+ACCESS_MODULE+"] no contiene la descripcin del mdulo ["+modName+"].");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Fichero de Accesos ["+modName+ManagerUtilities.FILE_SEPARATOR+ACCESS_MODULE+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Accesos del Mdulo.",ex);
		}
		
		return modXML;
		
	}

	public static Node readRoleModule(User user) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Node modXML = null;
		try {
			
			URL urlConfig = user.getClass().getResource(ROLE_MODULE);
			Document miXML = XMLUtil.parse(urlConfig.openStream());
			modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.ROLES);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Roles ["+ACCESS_MODULE+"] no contiene la descripcin del roles.");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Fichero de Roles ["+ROLE_MODULE+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Roles del Mdulo.",ex);
		}
		
		return modXML;
		
	}

	public static Node readProfileRoleModule(User user) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Node modXML = null;
		try {
			
			URL urlConfig = user.getClass().getResource(PROFILE_ROLE_MODULE);
			Document miXML = XMLUtil.parse(urlConfig.openStream());
			modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.PROFILE_ROLES);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Roles Perfiles ["+PROFILE_ROLE_MODULE+"] no contiene la descripcin del roles perfiles.");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Fichero de Roles Perfiles ["+PROFILE_ROLE_MODULE+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Roles Perfiles del Mdulo.",ex);
		}
		
		return modXML;
		
	}

	public static Document readDictionaryModule(ClassLoader cl,HttpServletRequest req,String language) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Document miXML = null;
		String modName = "";
		try {
			
			modName = ManagerUtilities.getModuleParameter(req);
			modName = modName+ManagerUtilities.FILE_SEPARATOR+DICTIONARY_MODULE+(language.equals("")?"":"_")+language+".xml";
			URL urlConfig = cl.getResource(modName);
			miXML = XMLUtil.parse(urlConfig.openStream());
			Node modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.XMLROOT);
			if (modXML==null) throw new ManagerException("ManagerReader","El Diccionario ["+modName+"] no tiene como nodo raiz ["+ManagerTags.XMLROOT+"].");

		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Diccionario ["+modName+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Diccionario del Mdulo.",ex);
		}
		
		return miXML;
		
	}

	public static Document readDictionaryModule(ClassLoader cl,String modName,String language) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Document miXML = null;
		try {
			
			URL urlConfig = cl.getResource(modName+ManagerUtilities.FILE_SEPARATOR+DICTIONARY_MODULE+(language.equals("")?"":"_")+language+".xml");
			miXML = XMLUtil.parse(urlConfig.openStream());
			Node modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.XMLROOT);
			if (modXML==null) throw new ManagerException("ManagerReader","El Diccionario ["+modName+"] no tiene como nodo raiz ["+ManagerTags.XMLROOT+"].");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Diccionario ["+modName+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Diccionario del Mdulo.",ex);
		}
		
		return miXML;
		
	}

	public static Document readQueryModule(ClassLoader cl,String modName) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Document miXML = null;
		try {
			
			URL urlConfig = cl.getResource(modName+ManagerUtilities.FILE_SEPARATOR+QUERY_MODULE);
			if (urlConfig==null) throw new ManagerException("ManagerReader","El Fichero de Querys ["+modName+ManagerUtilities.FILE_SEPARATOR+QUERY_MODULE+"] no existe.");
			miXML = XMLUtil.parse(urlConfig.openStream());
			Node modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.QUERYS,ManagerTags.QUERYS_PCK,modName);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Query ["+modName+ManagerUtilities.FILE_SEPARATOR+QUERY_MODULE+"] no contiene las querys del mdulo.");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+QUERY_MODULE+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Querys del Mdulo.",ex);
		}
		
		return miXML;
		
	}

	public static Document readAccessModule(ClassLoader cl,String modName) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Document miXML = null;
		try {
			
			URL urlConfig = cl.getResource(modName+ManagerUtilities.FILE_SEPARATOR+ACCESS_MODULE);
			if (urlConfig==null) throw new ManagerException("ManagerReader","El Fichero de Accesos ["+modName+ManagerUtilities.FILE_SEPARATOR+ACCESS_MODULE+"] no existe.");
			miXML = XMLUtil.parse(urlConfig.openStream());
			Node modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.MODULE,ManagerTags.MODULE_PCK,modName);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Accesos ["+modName+ManagerUtilities.FILE_SEPARATOR+ACCESS_MODULE+"] no contiene los accesos del mdulo.");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			throw new ManagerException("ManagerReader","No se encuentra el Fichero de Configuracin ["+modName+ManagerUtilities.FILE_SEPARATOR+ACCESS_MODULE+"].");
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Accesos del Mdulo.",ex);
		}
		
		return miXML;
		
	}

	public static Node readDataMenu(User user) throws ManagerException {
		// Leo el XML General en Busca del Mdulo
		Node modXML = null;
		try {
			
			URL urlConfig = user.getClass().getResource(DATA_MENU);
			Document miXML = XMLUtil.parse(urlConfig.openStream());
			modXML = XMLUtil.selectSingleNode(miXML,ManagerTags.MENU);
			if (modXML==null) throw new ManagerException("ManagerReader","El Fichero de Datos de Menu ["+DATA_MENU+"] no contiene la descripcin de los datos del Menu.");
			
		} catch (ManagerException mex) {
			throw mex;
		} catch (NullPointerException nex) {
			// No se requiere menu
		} catch (Exception ex) {
			throw new ManagerException("ManagerReader","Error en el Fichero de Datos de Menu.",ex);
		}
		return modXML;
		
	}

}
