/*
 * Created on 12-feb-2005
 *
 */
package atica.frameweb.manager;

import java.io.*;
import java.net.URLEncoder;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import atica.frameweb.utils.*;
import atica.frameweb.xml.*;
import org.w3c.dom.*;

/**
 * @author Juan Jos Meroo Snchez
 *
 * Diferentes Utilidades para obtener informacin de la peticin.
 */
public class ManagerUtilities {
	
	public static final String FILE_SEPARATOR = "/";
	public static final String DATE_FORMAT = "dd/MM/yyyy";
	public static final String TIME_FORMAT = "HH:mm:ss";
	public static final String DATE_TIME_FORMAT = "dd/MM/yyyy HH:mm:ss";
	public static final String DATE_SEPARATOR = "/";
	public static final String TIME_SEPARATOR = ":";
	
	public static final String LAST_REQUEST = "lastRequest";
	public static final String PACKAGE_BASE_PARAMETER = "package";
	public static final String ORG_MODULE_PARAMETER = "org-module";
	public static final String ORG_ACTION_PARAMETER = "org-action";
	public static final String MODULE_PARAMETER = "module";
	public static final String ACTION_PARAMETER = "action";
	public static final String NEXT_PARAMETER = "next";
	public static final String SRV_CONTROL_PARAMETER = "servlet";
	public static final String OUTPUT_PARAMETER = "output";
	public static final String FORCE_FWD = "forcefwd";

	/**
	*  The meat of the servlet.  This method parses the input, and
 	*  returns a hashtable of either String[] values (for parameters)
 	*  or Hashtable values (for files uploaded).  The values of the entries
 	*  in the hashtable are name, filename, Content-Type, and Contents.
 	*  Note that uploads should be capped in size by the calling method, since
 	*  otherwise a denial of service attack on server memory becomes trivial.
 	*/
 	
	private static Hashtable parseMulti(String boundary,long contLength, ServletInputStream in,String dirTemp,long TAM_MAXIMO) throws Exception {
	
	   String contentLength = ""+contLength;
	   int buffSize = 1024*8; // 8K
	   Hashtable hash = new Hashtable();
	   int result;
	   String line;
	   String lowerline;
	   String boundaryStr = "--"+boundary;
	   OutputStream content = null;
	   String filename = null;
	   String contentType;
	   String name;
	   String value;
	   boolean lecturaPermitida = true;
	   String diskFileName = "";
	
	   byte[] b = new byte[buffSize];
	   int fileNumber = 0;
	
	   try {
		   result = in.readLine(b,0,b.length);
		   // failure.
		   if (result == -1) throw new IllegalArgumentException("InputStream truncated.");
		   line = new String(b,0,0,result);
		   // failure.
		   if (!line.startsWith(boundaryStr)) throw new IllegalArgumentException("MIME boundary missing: "+line);
		   
		   while (true) {
		   		// Some initialization
		 		filename = null;
		 		contentType = null;
		 		name = null;
		
		       // get next line (should be content disposition)
		       result = in.readLine(b,0,b.length);
		       if (result == -1) return hash;
		       line = new String(b,0,0,result-2);
		       lowerline = line.toLowerCase();
		       if (!lowerline.startsWith("content-disposition"))
		           // don't know what to do, so we'll keep looking...
		           continue;
		       // determine what the disposition is
		       int ind = lowerline.indexOf("content-disposition: ");
		       int ind2 = lowerline.indexOf(";");
		       if (ind == -1 || ind2 == -1) throw new IllegalArgumentException("Content Disposition line misformatted: "+line);
		       String disposition = lowerline.substring(ind+21,ind2);
		       if (!disposition.equals("form-data")) throw new IllegalArgumentException("Content Disposition of "+disposition+" is not supported");
		       // determine what the name is
		       int ind3 = lowerline.indexOf("name=\"",ind2);
		       int ind4 = lowerline.indexOf("\"",ind3+7);
		       if (ind3 == -1 || ind4 == -1) throw new IllegalArgumentException("Content Disposition line misformatted: "+line);
		       name = line.substring(ind3+6,ind4);
		       // determine filename, if any
		       int ind5 = lowerline.indexOf("filename=\"",ind4+2);
		       int ind6 = lowerline.indexOf("\"",ind5+10);
		       if (ind5 != -1 && ind6 != -1) filename = line.substring(ind5+10,ind6);
		
		       lecturaPermitida = ((filename==null) || (TAM_MAXIMO<0) || (contLength<=TAM_MAXIMO));
		 			
		       // Whew!  We now move onto the next line, which
		       // will either be blank, or Content-Type, followed by blank.
		       result = in.readLine(b,0,b.length);
		       if (result == -1) return hash;
		       line = new String(b,0,0,result-2); // -2 to remove \r\n
		       lowerline = line.toLowerCase();
		       if (lowerline.startsWith("content-type")) {
		     		int ind7 = lowerline.indexOf(" ");
		     		if (ind7 == -1) throw new IllegalArgumentException("Content-Type line misformatted: "+line);
		     		contentType = lowerline.substring(ind7+1);
		     		//  read blank header line
		     		result = in.readLine(b,0,b.length);
		     		if (result == -1) return hash;
		     		line = new String(b,0,0,result-2); // -2 to remove \r\n
		     		if (line.length() != 0) throw new IllegalArgumentException("Unexpected line in MIMEpart header: "+line);
		       } else if (line.length() != 0) throw new IllegalArgumentException("Misformatted line following disposition: "+line);
		
		       //read content, implement readahead by one line
		       boolean readingContent = true;
		       boolean firstLine = true;
		       byte[] buffbytes = new byte[buffSize];
		       int buffnum = 0;
		
		       result = in.readLine(b,0,b.length);
		       if (result == -1) return hash;
		       line = new String(b,0,0,result);
		       if (!line.startsWith(boundaryStr)) {
		     		System.arraycopy(b,0,buffbytes,0,result);
		     		buffnum = result;
		     		result = in.readLine(b,0,b.length);
		     		if (result == -1) return hash;
		     		line = new String(b,0,0,result);
		     		firstLine = false;
		     		if (line.startsWith(boundaryStr)) readingContent = false;
		       } else readingContent = false;
					
		       if (filename==null) content = new ByteArrayOutputStream();
		       else {
		       		diskFileName = dirTemp+"upFile"+fileNumber+"-"+System.currentTimeMillis();
		       		content = new FileOutputStream(diskFileName);
		       }
		 			
		       while (readingContent) {
		     		if (lecturaPermitida) content.write(buffbytes,0,buffnum);
		     		System.arraycopy(b,0,buffbytes,0,result);
		     		buffnum = result;
		     		result = in.readLine(b,0,b.length);
		     		if (result == -1) return hash;
		     		line = new String(b,0,0,result);
		     		if (line.startsWith(boundaryStr)) readingContent = false;
		       }
		      
		       if (!firstLine) {
		     		// -2 to trim \r\n
		     		if ((buffnum>2) && (lecturaPermitida)) content.write(buffbytes,0,buffnum-2);
		       }
	     
		       //now set appropriate variable, populate hashtable
		       if (filename == null) {
		    		if (hash.get(name) == null) {
		  				String[] values = new String[1];
		  				values[0] = content.toString();
		        		hash.put(name,values);
		    		} else {
		  				Object prevobj = hash.get(name);
		  				if (prevobj instanceof String[]) {
		      				String[] prev = (String[])prevobj;
		      				String[] newStr = new String[prev.length+1];
		      				System.arraycopy(prev,0,newStr,0,prev.length);
		      				newStr[prev.length] = content.toString();
		      				hash.put(name,newStr);
		  				} else {
			    				//now what? I think this breaks the standard.
		   							throw new IllegalArgumentException("failure in parseMulti hashtable building code");
		  				}
		    		}
		       } else {
		    			// Yes, we don't return Hashtable[] for multiple files of same name.  AFAIK, that's not allowed.
		    			Hashtable filehash = new Hashtable(4);
		    			filehash.put("name",name);
		    			filehash.put("filename",filename);
		    			if (contentType == null) contentType = "application/octet-stream";
		    			filehash.put("content-type",contentType);
		    			filehash.put("content-length",contentLength);
		    			if (lecturaPermitida) {
		    				filehash.put("content",diskFileName);
		    				content.close();
		    			}	else {
		    				filehash.put("content","");
		    				content.close();
		    				File f = new File(diskFileName);
		    				f.delete();
		    			}
		    			hash.put(name,filehash);
		    			fileNumber++;
		       }
		   }
	   } catch (Exception ex) {
	   		if (diskFileName != null) {
   				content.close();
   				File f = new File(diskFileName);
   				f.delete();
	   		}
	   		throw ex;
	   }
	}

	//---------------------------------------------------------------
	// Parametros de una Llamada Normal
	//---------------------------------------------------------------

	private static Hashtable getParameters(HttpServletRequest req) {
		Hashtable table = new Hashtable(5);
		Enumeration nombres = req.getParameterNames();
		while (nombres.hasMoreElements()) {
			String nombre = (String) nombres.nextElement();
			table.put(nombre,req.getParameterValues(nombre));
		}
		return table;
	}

	//---------------------------------------------------------------
	// Parametros de una Llamada de UpLoad
	//---------------------------------------------------------------
	
	private static Hashtable getUploadParameters(HttpServletRequest req,String dirTemp,long TAM_MAXIMO) {
		Hashtable table = null;
		String contType = req.getContentType();
		long fileSize = req.getContentLength();
		
		if (contType==null) {
			// No Es UpLoad
			table = getParameters(req);
			return table;
		}
		if (!contType.toLowerCase().startsWith("multipart/form-data")) {
			// No Es UpLoad
			table = getParameters(req);
			return table;
		}
		int ind = contType.indexOf("boundary=");
		if (ind == -1) {
    	// No Es UpLoad
			table = getParameters(req);
			return table;
		}
    
		String boundary = contType.substring(ind+9);

		if (boundary == null) {
			// No Es UpLoad
			table = getParameters(req);
			return table;
		}

		try {
			// Es UpLoad.
			table = parseMulti(boundary, fileSize, req.getInputStream(),dirTemp,TAM_MAXIMO);
		} catch (Exception ex) {
			table = getParameters(req);
 			return table;
		}
		
		return table;
	}

	//----------------------------------------------------------
	// Recoge todos los parmetros de la peticin
	//----------------------------------------------------------
	public static void getAllParameters(HttpServletRequest req,String dirTemp,long TAM_MAXIMO) {
		Hashtable h = getUploadParameters(req,dirTemp,TAM_MAXIMO);
		if (h!=null) {
			Enumeration en = h.keys();
			while (en.hasMoreElements()) {
				String key = (String) en.nextElement();
				req.setAttribute(key,h.get(key));
			}
		}
	}
	
	//---------------------------------------------------------------
	// GetParameter Simple
	//---------------------------------------------------------------
	public static String getParameter(HttpServletRequest req,String n) {
		String [] st = (String []) req.getAttribute(n);
		if (st==null) return null;
		return st[0];
	}
	//---------------------------------------------------------------
	// SetParameter Simple
	//---------------------------------------------------------------
	public static void setParameter(HttpServletRequest req,String n,String v) {
		req.setAttribute(n,new String[]{v});
	}

	//---------------------------------------------------------------
	// GetParameterValues
	//---------------------------------------------------------------
	public static String [] getParameterValues(HttpServletRequest req,String n) {
		return (String []) req.getAttribute(n);
	}

	//---------------------------------------------------------------
	// GetFileParameter
	//---------------------------------------------------------------
	public static Hashtable getFileParameter(HttpServletRequest req,String n) {
		return (Hashtable) req.getAttribute(n);
	}

	//----------------------------------------------------------
	// Elimina los archivos que ocupan ms espacio del deseado
	//----------------------------------------------------------
	public static void setFileLength(HttpServletRequest req,long TAM_MAXIMO) {
		Enumeration en = req.getAttributeNames();
		while (en.hasMoreElements()) {
			// Recorre todos los parmetros.
			try {
				String key = (String) en.nextElement();
				Hashtable file = (Hashtable)req.getAttribute(key);
				long l = Long.parseLong((String)file.get("content-length"));
				if (l>TAM_MAXIMO) {
					// Elimino el archivo.
					String fileName = (String)file.get("content");
					File f = new File(fileName);
					f.delete();
					file.put("content","");
				}
			} catch (Exception ex) {} // Si falla no es un file
		}
	}
	
	//------------------------------------------------------------------
	// Establece los parmetros de mdulo y operacin de forma fija
	//------------------------------------------------------------------
	
	public static void setModuleOperation(HttpServletRequest req,String paquete) throws ManagerException {
		String modulo = getParameter(req,MODULE_PARAMETER);
		String accion = getParameter(req,ACTION_PARAMETER);
		// En el next slo es posible especificar el next !!
		String next = getParameter(req,NEXT_PARAMETER);
		if (modulo==null) {
			if (accion==null) throw new ManagerException("ManagerUtilities","No se ha indicado la accin a realizar, indique el parametro module y/o action.");
			int indOfPto = accion.indexOf(".");
			if (indOfPto>0) {
				modulo = accion.substring(0,indOfPto);
				accion = accion.substring(indOfPto+1);
				indOfPto = accion.indexOf(".");
				if (indOfPto>0) {
					if (next!=null) throw new ManagerException("ManagerUtilities","Next Duplicado: Se ha indicado next en action y next.");
					next = accion.substring(indOfPto+1);
					accion = accion.substring(0,indOfPto);
				}
			}
			if (modulo==null) throw new ManagerException("ManagerUtilities","No se ha indicado la accin a realizar, indique el parametro module y/o action.");
		} else {
			int indOfPto = modulo.indexOf(".");
			if (indOfPto>0) {
				if (accion!=null) throw new ManagerException("ManagerUtilities","Action Duplicada: Se ha indicado la accin en el module y action.");
				accion = modulo.substring(indOfPto+1);
				modulo = modulo.substring(0,indOfPto);
				indOfPto = accion.indexOf(".");
				if (indOfPto>0) {
					if (next!=null) throw new ManagerException("ManagerUtilities","Next Duplicado: Se ha indicado next en module y next.");
					next = accion.substring(indOfPto+1);
					accion = accion.substring(0,indOfPto);
				}
			}
			if (accion==null) throw new ManagerException("ManagerUtilities","No se ha indicado la accin a realizar, indique el parametro module y/o action.");
			if ((accion.indexOf(".")>0) && (indOfPto<=0)) throw new ManagerException("ManagerUtilities","Module Duplicado: Se ha indicado el mdulo en el module y action.");
		}
		req.setAttribute(MODULE_PARAMETER,new String[]{modulo});
		req.setAttribute(ACTION_PARAMETER,new String[]{accion});
		if (next!=null) req.setAttribute(NEXT_PARAMETER,new String[]{next});
		if (paquete!=null) req.setAttribute(PACKAGE_BASE_PARAMETER,new String[]{paquete});
	}
	
	public static String getModuleParameter(HttpServletRequest req) {
		return getParameter(req,MODULE_PARAMETER);
	}
	
	public static String getActionParameter(HttpServletRequest req) {
		return getParameter(req,ACTION_PARAMETER);
	}
	
	public static String getOrgModuleParameter(HttpServletRequest req) {
		return getParameter(req,ORG_MODULE_PARAMETER);
	}
	
	public static String getOrgActionParameter(HttpServletRequest req) {
		return getParameter(req,ORG_ACTION_PARAMETER);
	}
	
	public static String getNextParameter(HttpServletRequest req) {
		return getParameter(req,NEXT_PARAMETER);
	}
	
	public static void setModuleParameter(HttpServletRequest req,String s) {
		setParameter(req,MODULE_PARAMETER,s);
	}
	
	public static void setActionParameter(HttpServletRequest req,String s) {
		setParameter(req,ACTION_PARAMETER,s);
	}
	
	public static void setOrgModuleParameter(HttpServletRequest req,String s) {
		setParameter(req,ORG_MODULE_PARAMETER,s);
	}
	
	public static void setOrgActionParameter(HttpServletRequest req,String s) {
		setParameter(req,ORG_ACTION_PARAMETER,s);
	}
	
	public static void setNextParameter(HttpServletRequest req,String s) {
		setParameter(req,NEXT_PARAMETER,s);
	}
	
	public static Object getRequestObject(HttpServletRequest req,String name) {
		return req.getAttribute(name);
	}
	
	public static Object getRequestBasicObject(HttpServletRequest req,String name,Class cl,boolean b) throws Exception {
		if (cl.getName().equals(String.class.getName()) || cl.getName().equals(String[].class.getName())) {
			// Retorna un String o Array de String
			if (cl.getName().equals(String.class.getName())) return getParameter(req,name);
			else return getParameterValues(req,name);
		} else if (cl.getName().equals(Integer.class.getName()) || cl.getName().equals(int.class.getName()) || cl.getName().equals(Integer[].class.getName()) || cl.getName().equals(int[].class.getName())) {
			// Retorna un Integer
			try {
				if (cl.getName().equals(Integer.class.getName()) || cl.getName().equals(int.class.getName())) {
					String obj = getParameter(req,name);
					if ((obj==null) || obj.equals("")) return null;
					return new Integer(obj);
				} else if (cl.getName().equals(int[].class.getName())) {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					int [] enteros = new int[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) enteros[k] = Integer.parseInt(st[k]);
					return enteros;
				} else {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					Integer [] enteros = new Integer[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) enteros[k] = new Integer(st[k]);
					return enteros;
				}
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a Integer/int: "+ex);
			}
		} else if (cl.getName().equals(Float.class.getName()) || cl.getName().equals(float.class.getName()) || cl.getName().equals(Float[].class.getName()) || cl.getName().equals(float[].class.getName())) {
			// Retorna un Float
			try {
				if (cl.getName().equals(Float.class.getName()) || cl.getName().equals(float.class.getName())) {
					String obj = getParameter(req,name);
					if ((obj==null) || obj.equals("")) return null;
					return new Float(obj);
				} else if (cl.getName().equals(float[].class.getName())) {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					float [] floats = new float[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) floats[k] = Float.parseFloat(st[k]);
					return floats;
				} else {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					Float [] floats = new Float[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) floats[k] = new Float(st[k]);
					return floats;
				}
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a Float/float: "+ex);
			}
		} else if (cl.getName().equals(Double.class.getName()) || cl.getName().equals(double.class.getName())) {
			// Retorna un Double
			try {
				if (cl.getName().equals(Double.class.getName()) || cl.getName().equals(double.class.getName())) {
					String obj = getParameter(req,name);
					if ((obj==null) || obj.equals("")) return null;
					return new Double(obj);
				} else if (cl.getName().equals(double[].class.getName())) {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					double [] doubles = new double[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) doubles[k] = Double.parseDouble(st[k]);
					return doubles;
				} else {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					Double [] doubles = new Double[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) doubles[k] = new Double(st[k]);
					return doubles;
				}
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a Double/double: "+ex);
			}
		} else if (cl.getName().equals(Long.class.getName()) || cl.getName().equals(long.class.getName())) {
			// Retorna un Long
			try {
				if (cl.getName().equals(Long.class.getName()) || cl.getName().equals(long.class.getName())) {
					String obj = getParameter(req,name);
					if ((obj==null) || obj.equals("")) return null;
					return new Long(obj);
				} else if (cl.getName().equals(long[].class.getName())) {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					long [] longs = new long[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) longs[k] = Long.parseLong(st[k]);
					return longs;
				} else {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					Long [] longs = new Long[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) longs[k] = new Long(st[k]);
					return longs;
				}
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a Long/long: "+ex);
			}
		} else if (cl.getName().equals(Boolean.class.getName()) || cl.getName().equals(boolean.class.getName())) {
			// Retorna un Boolean
			try {
				if (cl.getName().equals(Boolean.class.getName()) || cl.getName().equals(boolean.class.getName())) {
					String obj = getParameter(req,name);
					if ((obj==null) || obj.equals("")) return null;
					return new Boolean(obj);
				} else if (cl.getName().equals(boolean[].class.getName())) {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					boolean [] booleans = new boolean[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) booleans[k] = Boolean.valueOf(st[k]).booleanValue();
					return booleans;
				} else {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					Boolean [] booleans = new Boolean[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) booleans[k] = new Boolean(st[k]);
					return booleans;
				}
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a Boolean/boolean: "+ex);
			}
		} else if (cl.getName().equals(Byte.class.getName()) || cl.getName().equals(byte.class.getName())) {
			// Retorna un Byte
			try {
				if (cl.getName().equals(Byte.class.getName()) || cl.getName().equals(byte.class.getName())) {
					String obj = getParameter(req,name);
					if ((obj==null) || obj.equals("")) return null;
					return new Byte(obj);
				} else if (cl.getName().equals(byte[].class.getName())) {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					byte [] bytes = new byte[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) bytes[k] = Byte.parseByte(st[k]);
					return bytes;
				} else {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					Byte [] bytes = new Byte[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) bytes[k] = new Byte(st[k]);
					return bytes;
				}
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a Byte/byte: "+ex);
			}
		} else if (cl.getName().equals(Short.class.getName()) || cl.getName().equals(short.class.getName())) {
			// Retorna un Short
			try {
				if (cl.getName().equals(Short.class.getName()) || cl.getName().equals(short.class.getName())) {
					String obj = getParameter(req,name);
					if ((obj==null) || obj.equals("")) return null;
					return new Short(obj);
				} else if (cl.getName().equals(short[].class.getName())) {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					short [] shorts = new short[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) shorts[k] = Short.parseShort(st[k]);
					return shorts;
				} else {
					String [] st = getParameterValues(req,name);
					if (st==null) return st;
					Short [] shorts = new Short[st.length];
					for (int k=0; k<st.length; k++) if ((st[k]!=null) && !st[k].equals("")) shorts[k] = new Short(st[k]);
					return shorts;
				}
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a Short/short: "+ex);
			}
		} else if (cl.getName().equals(java.util.Date.class.getName())) {
			// Retorna un Date
			try {
				String obj = getParameter(req,name);
				if ((obj==null) || obj.equals("")) return null;
				// Miro el posible Formato de Fecha
				if (obj.indexOf(DATE_SEPARATOR)>0) {
					if (obj.indexOf(TIME_SEPARATOR)>0) return (new SimpleDateFormat(DATE_TIME_FORMAT)).parse(obj);
					return (new SimpleDateFormat(DATE_FORMAT)).parse(obj);
				} else if (obj.indexOf(TIME_SEPARATOR)>0) return (new SimpleDateFormat(TIME_FORMAT)).parse(obj);
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a Date/date, use "+DATE_FORMAT+" y "+TIME_FORMAT+": "+ex);
			}
			throw new Exception("Formato de Fecha Incorrecto ["+name+"], use "+DATE_FORMAT+" y "+TIME_FORMAT);
		} else if (cl.getName().equals(java.math.BigDecimal.class.getName())) {
			// Retorna un BigDecimal
			try {
				String obj = getParameter(req,name);
				if ((obj==null) || obj.equals("")) return null;
				return new java.math.BigDecimal(obj);
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a BigDecimal: "+ex);
			}
		} else if (cl.getName().equals(java.math.BigInteger.class.getName())) {
			// Retorna un BigInteger
			try {
				String obj = getParameter(req,name);
				if ((obj==null) || obj.equals("")) return null;
				return new java.math.BigInteger(obj);
			} catch (Exception ex) {
				throw new Exception("Error Convirtiendo ["+name+"] a BigInteger: "+ex);
			}
		} else if (cl.getName().equals(Hashtable.class.getName())) {
			// Retorna un Hashtable
			return getFileParameter(req,name);
		} else if (b) {
			return req.getAttribute(name);
		} else throw new Exception("");
		
	}
	
	public static Object getRequestObject(HttpServletRequest req,String name,Class cl) throws Exception {
		// Busca Objetos de tipos Bsicos !!
		return getRequestBasicObject(req,name,cl,true);
	}

	public static Object getRequestObjectBean(String metName,HttpServletRequest req,String name,Class cl,ClassLoader classLoader,ManagerModule mm) throws Exception {
		// Busca Objetos de Tipos Bsicos o Complejos !!!
		Object obj = null;
		try {
			obj = getRequestBasicObject(req,name,cl,false);
			return obj;
		} catch (Exception ex) {
			if (!ex.getMessage().equals("")) throw ex;
			// Es una clase no bsica
			// Con un constructor vaco !! 
			try {
				Object beanObject = classLoader.loadClass(cl.getName()).getConstructor(new Class[]{}).newInstance(new Object[]{});
				mm.setBeanProperties(metName,beanObject);
				return beanObject;
			} catch (Exception exx) { 
				// Hay algn Problema 
				// Suele ser un set de algo no nuestro !!
				return null;
			}
		}
	}
	
	public static void moveModuleToOrg(HttpServletRequest req) {
		setOrgModuleParameter(req,getModuleParameter(req));
		setOrgActionParameter(req,getActionParameter(req));
	}
	
	public static void setMetaObject(String className,HttpServletRequest req,Object obj) {
		IMetaObject ob = (IMetaObject) req.getSession(true).getAttribute(obj.getClass().getName());
		if ((ob==null) || (!ob.getClassName().equals(className))) req.getSession(true).setAttribute(obj.getClass().getName(),obj);
	}
	
	public static void deleteTempFiles(HttpServletRequest req) {
		Enumeration en = req.getAttributeNames();
		while (en.hasMoreElements()) {
			String name = (String) en.nextElement();
			try {
				Hashtable h = (Hashtable) req.getAttribute(name);
				String filename = (String) h.get("content");
				File f = new File(filename);
				f.delete();
			} catch (Exception ex) {
				// No es un fichero !!
			}
		}
	}
	
	public static boolean existsRequestElement(HttpServletRequest req,String name) {
		Enumeration en = req.getAttributeNames();
		while (en.hasMoreElements()) {
			String s = (String)en.nextElement();
			if (s.startsWith(name)) return true;
		}
		return false;
	}
	
	public static String getUrlParameters(HttpServletRequest req,String nm,String na) {
		String query = "?module="+nm+"."+na;
		// Metodo Get o Post, Paso a Get
		Hashtable h = ManagerUtilities.getParameters(req);
		Enumeration en = h.keys();
		while (en.hasMoreElements()) {
			String name = (String) en.nextElement();
			if (name.equals(MODULE_PARAMETER) || name.equals(ACTION_PARAMETER) || name.equals(NEXT_PARAMETER)) continue;
			String [] s = (String[])h.get(name);
			for (int k=0; k<s.length; k++) {
				query += "&"+name+"="+URLEncoder.encode(s[k]);
			}
		}
		String thisUrl = req.getRequestURL().append(query).toString();
		return thisUrl;
	}
	
	// COMPROBACIN FORCE-FWD
	
	public static Vector noFwdActions = null;
	
	public static void readNoFwdActions(ClassLoader cl,String paq) throws ManagerException {
		// Leo el config general y busco las acciones NO FWD
		noFwdActions = new Vector();
		Document xml = ManagerReader.readFileModules(cl,paq);
		NodeList nl = XMLUtil.selectNodes(xml.getFirstChild(),ManagerTags.MODULE);
		for (int k=0; k<nl.getLength(); k++) {
			Node n = nl.item(k);
			String mod = XMLUtil.getAttributeValue(n,ManagerTags.MODULE_PCK);
			NodeList actions = XMLUtil.selectNodes(n,ManagerTags.NOFWDACTION);
			for (int j=0; j<actions.getLength(); j++) {
				Node ac = actions.item(j);
				String act = XMLUtil.getAttributeValue(ac,ManagerTags.ACTION_NAME);
				noFwdActions.addElement(mod+"."+act);
			}
		}
	}
	
	public static void isValidForward(String paq,ClassLoader cl,HttpServletRequest req,String mod,String act) throws ManagerException {
		// Comprueba si se trata de un forward correcto !!
		// Un forward es correcto si llama a la misma accin y mdulo que la peticin anterior.
		String newMdac = mod+"."+act; 
		String mdac = (String)req.getSession().getAttribute(FORCE_FWD);
		if (mdac!=null) {
			if (!newMdac.startsWith(mdac)) {
				if (noFwdActions==null) readNoFwdActions(cl,paq);
				if (!noFwdActions.contains(newMdac)) throw new ManagerException("Llamada Fuera de FORWARD","No se permite realizar una llamada directa a ["+newMdac+"] desde ["+mdac+"]. Utiliza un FORWARD.");
			}
		}
	}
	
	public static void saveNextForward(HttpServletRequest req,String mod,String act) {
		// Guardo el mdulo y accin final, para comprobar el prximo forward
		req.getSession().setAttribute(FORCE_FWD,mod+"."+act);
	}
	
}
