/*
 * Created on 19-feb-2005
 *
 */
package atica.frameweb.user;

import atica.frameweb.query.FrameWebCallable;
import atica.frameweb.query.FrameWebQuery;
import atica.frameweb.user.menu.*;
import atica.frameweb.manager.*;
import atica.frameweb.utils.*;
import atica.frameweb.xml.*;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.log4j.*;
import java.sql.*;
//import net.sf.hibernate.*;
import org.hibernate.*;

/**
 * @author Juan Jos Meroo Snchez
 *
 * Clase Usuario Genrica para cualquier aplicacin.
 * 
 */
public abstract class User implements IUser,Serializable {
	
	static Logger logger = HttpServletManager.logger;
	
	protected Vector staticRoles = new Vector(); 
	protected Vector staticProfiles = new Vector();
	protected static Node roleTree = null, profileRoleTree = null;
	protected String language = "";
	protected String login = "", password = "";
	protected DatosMenu menuData = null;

	/*
	 * COLECCION DE CONSTRUCTURES DEL OBJETO USUARIO
	 */
	//-----------------------------------------------------
	
	public User() {
		setLanguage("es");
		if (roleTree==null) loadTreeRoleAndProfile();
		crearUsuario();	
		loadProfileRoles(null,null);
		crearDatosMenu();
	}
	public User(String lang) {
		setLanguage(lang);
		if (roleTree==null) loadTreeRoleAndProfile();
		crearUsuario();	
		loadProfileRoles(null,null);
		crearDatosMenu();
	}

	//-----------------------------------------------------
	
	public User(String log,String pwd) {
		setLanguage("es");
		if (roleTree==null) loadTreeRoleAndProfile();
		login = log; password = pwd;
		crearUsuario();	
		loadProfileRoles(null,null);
		crearDatosMenu();
	}
	public User(String log,String pwd,String lang) {
		setLanguage(lang);
		if (roleTree==null) loadTreeRoleAndProfile();
		login = log; password = pwd;
		crearUsuario();	
		loadProfileRoles(null,null);
		crearDatosMenu();
	}

	//-----------------------------------------------------
	
	public User(Connection conn) {
		setLanguage("es");
		if (roleTree==null) loadTreeRoleAndProfile();
		crearUsuario(conn);	
		loadProfileRoles(conn,null);
		crearDatosMenu(conn);
	}
	public User(Connection conn,String lang) {
		setLanguage(lang);
		if (roleTree==null) loadTreeRoleAndProfile();
		crearUsuario(conn);	
		loadProfileRoles(conn,null);
		crearDatosMenu(conn);
	}
	
	//-----------------------------------------------------
	
	public User(Connection conn,String log,String pwd) {
		setLanguage("es");
		if (roleTree==null) loadTreeRoleAndProfile();
		login = log; password = pwd;
		crearUsuario(conn);	
		loadProfileRoles(conn,null);
		crearDatosMenu(conn);
	}
	public User(Connection conn,String log,String pwd,String lang) {
		setLanguage(lang);
		if (roleTree==null) loadTreeRoleAndProfile();
		login = log; password = pwd;
		crearUsuario(conn);	
		loadProfileRoles(conn,null);
		crearDatosMenu(conn);
	}

	//-----------------------------------------------------

	public User(Session conn) {
		setLanguage("es");
		if (roleTree==null) loadTreeRoleAndProfile();
		crearUsuario(conn);	
		loadProfileRoles(null,conn);
		crearDatosMenu(conn);
	}
	public User(Session conn,String lang) {
		setLanguage(lang);
		if (roleTree==null) loadTreeRoleAndProfile();
		crearUsuario(conn);	
		loadProfileRoles(null,conn);
		crearDatosMenu(conn);
	}
	
	//-----------------------------------------------------
	
	public User(Session conn,String log,String pwd) {
		setLanguage("es");
		if (roleTree==null) loadTreeRoleAndProfile();
		login = log; password = pwd;
		crearUsuario(conn);	
		loadProfileRoles(null,conn);
		crearDatosMenu(conn);
	}
	
	public User(Session conn,String log,String pwd,String lang) {
		setLanguage(lang);
		if (roleTree==null) loadTreeRoleAndProfile();
		login = log; password = pwd;
		crearUsuario(conn);	
		loadProfileRoles(null,conn);
		crearDatosMenu(conn);
	}
	
	//-----------------------------------------------------
	
	private void writeObject(java.io.ObjectOutputStream out) throws IOException { overwriteObject(out); }
	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException { overreadObject(in); }
	
 	protected void overwriteObject(java.io.ObjectOutputStream out) throws IOException {
 		out.writeObject(staticRoles);
 		out.writeObject(staticProfiles);
 		out.writeObject(roleTree);
 		out.writeObject(profileRoleTree);
 		out.writeObject(language);
 		out.writeObject(login);
 		out.writeObject(password);
 		out.writeObject(menuData);
 	}
	
 	protected void overreadObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
 		staticRoles = (Vector)in.readObject();
 		staticProfiles = (Vector)in.readObject();
 		roleTree = (Node)in.readObject();
 		profileRoleTree = (Node)in.readObject();
 		language = (String)in.readObject();
 		login = (String)in.readObject();
 		password = (String)in.readObject();
 		menuData = (DatosMenu)in.readObject();
 	}
	
	protected void loadTreeRoleAndProfile() {
		try {
			roleTree = ManagerReader.readRoleModule(this);
			profileRoleTree = ManagerReader.readProfileRoleModule(this);
		} catch (ManagerException mex) {
			logger.error(mex.getError());
		} catch (Exception ex) {
			logger.fatal(ex);
		}
	}
	
	protected void loadProfileRoles(Connection conn,Session sesion) {
		NodeList pr = XMLUtil.selectNodes(profileRoleTree,ManagerTags.ROLE);
		if (pr!=null) {
			for (int k=0; k<pr.getLength(); k++) {
				Node n = pr.item(k);
				String rolName = XMLUtil.getAttributeValue(n,ManagerTags.ROL_NAME);
				if (rolName!=null) {
					// Miro si el usuario tiene el rol
					int ind = staticRoles.indexOf(new Role(null,rolName,null));
					if (ind>=0) {
						// Si lo tiene -> Cargo sus profiles y los de sus Hijos !!!
						Role rolBase = (Role)staticRoles.get(ind);
						if (rolBase.isNodoHoja()) {
							if (conn!=null) cargarPerfilesRol(rolBase,conn);
							else if (sesion!=null) cargarPerfilesRol(rolBase,sesion);
							else cargarPerfilesRol(rolBase);
						}
						else rolBase.cargaPerfilesRolHijos(this);
					}
				}
			}
		}
	}
	
	protected Vector rolesPermitidos(String mod, String act) throws ManagerException {
		Vector roles = new Vector();
		Node access = ManagerReader.readAccessModule(this,mod);
		// Hay dos posibilidades action dentro de rol o rol dentro de action.
		// Busco los roles con action adecuada y luego miro si el usuario los tiene
		NodeList roleList = XMLUtil.selectNodes(access,ManagerTags.ROLE);
		if (roleList!=null) {
			for (int k=0; k<roleList.getLength(); k++) {
				Node n = roleList.item(k);
				Node actionRole = XMLUtil.selectSingleNode(n,ManagerTags.ACTION,ManagerTags.ACTION_NAME,act);
				if (actionRole!=null) roles.addElement(XMLUtil.getAttributeValue(n,ManagerTags.ROL_NAME));
			}
		}
		// Busco el action adecuado y miro si el usuario tiene alguno de los roles
		Node action = XMLUtil.selectSingleNode(access,ManagerTags.ACTION,ManagerTags.ACTION_NAME,act);
		if (action!=null) {
			NodeList nl = XMLUtil.selectNodes(action,ManagerTags.ROLE);
			if (nl!=null) {
				for (int k=0; k<nl.getLength(); k++) {
					Node n = nl.item(k);
					roles.addElement(XMLUtil.getAttributeValue(n,ManagerTags.ROL_NAME));
				}
			}
		}
		return roles;
	}
	
	public boolean permitido(String mod, String act) throws ManagerException {
		Vector roles = rolesPermitidos(mod,act);
		if (roles.size()>0) return hasRoles(roles);
		return false;
	}
	public boolean permitidoPerfiles(String prf, String mod, String act) throws ManagerException {
		Vector roles = rolesPermitidos(mod,act);
		if (roles.size()>0) return hasProfileRoles(new String[]{prf},roles);
		return false;
	}
	public boolean permitidoPerfiles(String[] prf, String mod, String act) throws ManagerException {
		Vector roles = rolesPermitidos(mod,act);
		if (roles.size()>0) return hasProfileRoles(prf,roles);
		return false;
	}
	
	public String getLanguage() { return language; }
	public void setLanguage(String lan) { if (lan!=null) language = lan; }
	public String getLogin() { return login; }
	public void setLogin(String log) { if (log!=null) login = log; }
	public String getIdentificador() { return login; }
	public String getPasswd() { return password; }
	public void setPasswd(String pwd) { if (pwd!=null) password = pwd; }
	
	protected Node findRoleInTree(Node tree,Role r) {
		// Hago la busqueda en funcin del codigo.
		Node n = XMLUtil.selectSingleNode(tree,ManagerTags.ROLE,ManagerTags.ROL_CODE,r.getCode());
		if (n==null) {
			// Est debajo de la jerarqua.
			NodeList nl = XMLUtil.selectNodes(tree,ManagerTags.ROL_GROUP);
			for (int k=0; k<nl.getLength(); k++) {
				Node ng = nl.item(k);
				Node nr = findRoleInTree(ng,r);
				if (nr!=null) return nr;
			}
		}
		return n;
	}
	
	protected Role getRoleFrom(Node nr) {
		Role r = new Role (
		XMLUtil.getAttributeValue(nr,ManagerTags.ROL_CODE),
		XMLUtil.getAttributeValue(nr,ManagerTags.ROL_NAME),
		XMLUtil.getAttributeValue(nr,ManagerTags.ROL_DESC));
		return r;
	}
	
	protected void addNodeAndParentNodes(Node real,Role hijo) {
		Role r = getRoleFrom(real);
		int ind = staticRoles.indexOf(r);
		if (ind<0) staticRoles.addElement(r);
		else r = (Role)staticRoles.get(ind);
		if (hijo!=null) r.addRoleHijo(hijo);
		Node n = real.getParentNode();
		if ((n!=null) && !n.getNodeName().equals(ManagerTags.ROLES)) 
			addNodeAndParentNodes(n,r);
	}
	
	protected void addCompleteTreeRole(Role r) {
		// Debo aadir el rol tal y como est en el fichero de roles
		// Los roles con cdigo Son Nodos Hoja de roles.xml
		Node real = findRoleInTree(roleTree,r);
		if (real==null) return;
		// Adems debo aadir a todos los padres del rol
		addNodeAndParentNodes(real,null);
	}
	
	protected void addStaticRole(Role r) {
		if (!staticRoles.contains(r)) addCompleteTreeRole(r);  
	}
	
	public void addStaticRole(String cod) {
		Role r = new Role(cod);
		if (!staticRoles.contains(r)) addCompleteTreeRole(r); 
	}
	
	public void addProfileToRol(Role r,Profile p) { 
		if (!staticProfiles.contains(p)) staticProfiles.addElement(p);
		r.addProfile(p); 
	}
	
	protected void addProfileToRol(Role r,String pc,String pd) {
		Profile p = new Profile(pc,pd);
		if (!staticProfiles.contains(p)) staticProfiles.addElement(p);
		r.addProfile(p); 
	}
	
	protected void addProfileToRolCode(String c,Profile p) {
		int ind = staticRoles.indexOf(new Role(c));
		if (ind>=0) {
			Role r = (Role)staticRoles.get(ind);
			if (!staticProfiles.contains(p)) staticProfiles.addElement(p);
			r.addProfile(p);
		}
	}

	protected void addProfileToRolCode(String c,String pc,String pd) {
		int ind = staticRoles.indexOf(new Role(c));
		if (ind>=0) {
			Role r = (Role)staticRoles.get(ind);
			Profile p = new Profile(pc,pd);
			if (!staticProfiles.contains(p)) staticProfiles.addElement(p);
			r.addProfile(p);
		}
	}

	private boolean hasRoles(Vector v) {
		Enumeration en = v.elements();
		while (en.hasMoreElements()) {
			if (staticRoles.contains(new Role(null,(String)en.nextElement(),null))) return true;
		}
		return false;
	}
	
	private boolean hasProfileRoles(String [] prf,Vector v) {
		Enumeration en = v.elements();
		while (en.hasMoreElements()) {
			int ind = staticRoles.indexOf(new Role(null,(String)en.nextElement(),null));
			if (ind>=0) {
				Role r = (Role)staticRoles.get(ind);
				if (r.hasProfiles(prf)) return true;
			}
		}
		return false;
	}
	
	public String getRolName(String code) {
		int ind = staticRoles.indexOf(new Role(code));
		if (ind>=0) return ((Role)staticRoles.get(ind)).getName();
		return null;
	}
	
	public String getProfileName(String code) {
		int ind = staticProfiles.indexOf(new Role(code));
		if (ind>=0) return ((Profile)staticProfiles.get(ind)).getName();
		return null;
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		
		sb.append("\n"+super.toString()+"\n");
		Enumeration en = staticRoles.elements();
		while (en.hasMoreElements()) {
			Role r = (Role)en.nextElement();
			sb.append(r.toString());
		}
		return sb.toString();
	}
	
	public String getMensajeTraducido(String modulo,String mensaje) throws Exception {
		return GeneralTraductor.getMensaje(getClass().getClassLoader(),modulo,getLanguage(),mensaje);
	}
	
	public DatosMenu getDatosMenu() {
		return menuData; 
	}
	
	/********
	 * 
	 * Implementacin de Mtodos Abstractos Por defecto !!!
	 * 
	 ********/
	
	// Creacin de Roles Estticos
	
	public void crearUsuario(Connection conn) { crearUsuario(); }
	public void crearUsuario(Session conn) { crearUsuario(); }
	public void crearUsuario() {}

	// Creacin de Roles Dinmicos
	
	public void cargarPerfilesRol(Role role,Connection conn) { cargarPerfilesRol(role); }
	public void cargarPerfilesRol(Role role,Session conn) { cargarPerfilesRol(role); }
	public void cargarPerfilesRol(Role role) {}
	
	// Creacin de Datos de Menu
	public void readDataMenu() throws ManagerException {
		Node nodo = ManagerReader.readDataMenu(this);
		if (nodo!=null) {
			String paquete = getClass().getName().substring(0,getClass().getName().lastIndexOf("."));
			menuData = (DatosMenu)XMLUtil.getObjectFromXML(getClass().getClassLoader(),new ManagerResolver(getClass().getClassLoader()),paquete.replace('.','/'),nodo);
		}
	}
	public void crearDatosMenu(Connection conn) { crearDatosMenu(); }
	public void crearDatosMenu(Session conn) { crearDatosMenu(); }
	public void crearDatosMenu() {
		try {
			readDataMenu();
			if (menuData!=null) {
				menuData.removeNotPermited(this);
				menuData.traduceItems(null,this,null);
			}
		} catch (ManagerException ex) {
			logger.fatal(ex.getError());
		}
	}

	public Document readDictionary(String module) throws ManagerException {
		String mod = module;
		if (mod==null) mod = getClass().getName().substring(0,getClass().getName().lastIndexOf(".")).replace('.','/');
		return ManagerReader.readDictionaryModule(getClass().getClassLoader(),mod,getLanguage());
	}
	
	public String traduce(Document doc,String id,String dialecto) {
		// Busca dialectos dentro de un idioma !!
		Node lan = null;
		if (dialecto!=null) lan = XMLUtil.selectSingleNode(doc,"DICTIONARY","LANGUAGE",getLanguage()+"_"+dialecto);
		if (lan==null) lan = XMLUtil.selectSingleNode(doc,"DICTIONARY","LANGUAGE",getLanguage());
		Node n = XMLUtil.selectSingleNode(lan,"KEY","NAME",id);
		if (n!=null) return XMLUtil.getAttributeValue(n,"VALUE");
		return id;
	}

	public String traduce(String module,String id,String dialecto) {
		// Busca dialectos dentro de un idioma !!
		try { return traduce(readDictionary(module),id,dialecto); } catch (Exception ex) { return id; }
	}
	
	
	public final FrameWebQuery getQuery(Connection conn,String id,int [] cnt) throws ManagerException,SQLException {
		String name = getClass().getName();
		int ind = name.lastIndexOf(".");
		name = name.substring(0,ind).replace('.','/');
		Document qry = ManagerReader.readQueryModule(getClass().getClassLoader(),name);
		Node n = XMLUtil.selectSingleNode(qry,ManagerTags.QUERY,ManagerTags.QUERY_NAME,id);
		if (n==null) throw new ManagerException("GeneralManager","La Consulta de BBDD ["+id+"] no existe en el fichero de querys.");
		Node nsql = XMLUtil.selectSingleNode(n,ManagerTags.SQL);
		return new FrameWebQuery(conn,nsql.getFirstChild().getNodeValue(),cnt);
	}

	public final FrameWebQuery getQuery(Connection conn,String id) throws ManagerException,SQLException {
		String name = getClass().getName();
		int ind = name.lastIndexOf(".");
		name = name.substring(0,ind).replace('.','/');
		Document qry = ManagerReader.readQueryModule(getClass().getClassLoader(),name);
		Node n = XMLUtil.selectSingleNode(qry,ManagerTags.QUERY,ManagerTags.QUERY_NAME,id);
		if (n==null) throw new ManagerException("GeneralManager","La Consulta de BBDD ["+id+"] no existe en el fichero de querys.");
		Node nsql = XMLUtil.selectSingleNode(n,ManagerTags.SQL);
		return new FrameWebQuery(conn,nsql.getFirstChild().getNodeValue(),null);
	}

	public final FrameWebCallable getCallable(Connection conn,String id,int [] cnt) throws ManagerException,SQLException {
		String name = getClass().getName();
		int ind = name.lastIndexOf(".");
		name = name.substring(0,ind).replace('.','/');
		Document qry = ManagerReader.readQueryModule(getClass().getClassLoader(),name);
		Node n = XMLUtil.selectSingleNode(qry,ManagerTags.QUERY,ManagerTags.QUERY_NAME,id);
		if (n==null) throw new ManagerException("GeneralManager","La Consulta de BBDD ["+id+"] no existe en el fichero de querys.");
		Node nsql = XMLUtil.selectSingleNode(n,ManagerTags.SQL);
		return new FrameWebCallable(conn,nsql.getFirstChild().getNodeValue(),cnt);
	}

	public final FrameWebCallable getCallable(Connection conn,String id) throws ManagerException,SQLException {
		String name = getClass().getName();
		int ind = name.lastIndexOf(".");
		name = name.substring(0,ind).replace('.','/');
		Document qry = ManagerReader.readQueryModule(getClass().getClassLoader(),name);
		Node n = XMLUtil.selectSingleNode(qry,ManagerTags.QUERY,ManagerTags.QUERY_NAME,id);
		if (n==null) throw new ManagerException("GeneralManager","La Consulta de BBDD ["+id+"] no existe en el fichero de querys.");
		Node nsql = XMLUtil.selectSingleNode(n,ManagerTags.SQL);
		return new FrameWebCallable(conn,nsql.getFirstChild().getNodeValue(),null);
	}

	public final String getQuery(String id) throws ManagerException {
		String name = getClass().getName();
		int ind = name.lastIndexOf(".");
		name = name.substring(0,ind).replace('.','/');
		Document qry = ManagerReader.readQueryModule(getClass().getClassLoader(),name);
		Node n = XMLUtil.selectSingleNode(qry,ManagerTags.QUERY,ManagerTags.QUERY_NAME,id);
		if (n==null) throw new ManagerException("GeneralManager","La Consulta de BBDD ["+id+"] no existe en el fichero de querys.");
		Node nsql = XMLUtil.selectSingleNode(n,ManagerTags.SQL);
		return nsql.getFirstChild().getNodeValue();
	}
	

}
