/*
 * Created on 14-feb-2005
 *
 */
package atica.frameweb.xml;

import java.io.*;
import java.util.*;

import javax.xml.parsers.*;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.stream.StreamResult;

import org.apache.avalon.framework.logger.ConsoleLogger;
import org.apache.fop.apps.Driver;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.exolab.castor.mapping.Mapping;
import org.exolab.castor.xml.*;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.EntityResolver;

import atica.frameweb.manager.ManagerException;
import atica.frameweb.manager.ManagerReader;
import atica.frameweb.manager.ManagerResolver;
import javax.servlet.http.HttpServletResponse;

/**
 * @author Juan Jos Meroo Snchez
 *
 * Clase para Parseo de Archivos XML
 * 
 */
public class XMLUtil {
	
	public static final String ENCODING = "ISO-8859-1";
	
	public static Document createDocument() throws Exception {
		DocumentBuilderFactory df = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = df.newDocumentBuilder();
		return db.newDocument();
	}
	
	public static Document parse(InputStream in) throws Exception {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		return db.parse(in);
	}

	public static Document parse(InputStream in,EntityResolver rsv) throws Exception {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		db.setEntityResolver(rsv);
		return db.parse(in);
	}

	public static Node selectSingleNode(Document doc,String name,String att,String id) {
		NodeList nl = doc.getElementsByTagName(name);
		if (nl==null) return null;
		for (int k=0; k<nl.getLength(); k++) {
			Node n = nl.item(k);
			if (getAttributeValue(n,att).equals(id)) return n;
		}
		return null;
	}
	
	public static Node selectSingleNode(Document doc,String name) {
		NodeList nl = doc.getElementsByTagName(name);
		if (nl==null) return null;
		return nl.item(0);
	}
	
	public static Node selectSingleNode(Node node,String name) {
		NodeList nl = ((Element)node).getElementsByTagName(name);
		if (nl==null) return null;
		return nl.item(0);
	}
	
	public static Node selectSingleNode(Node node,String name,String att,String id) {
		NodeList nl = ((Element)node).getElementsByTagName(name);
		if (nl==null) return null;
		for (int k=0; k<nl.getLength(); k++) {
			Node n = nl.item(k);
			if (getAttributeValue(n,att).equals(id)) return n;
		}
		return null;
	}

	public static NodeList selectNodes(Node node,String name) {
		return ((Element)node).getElementsByTagName(name);
	}

	public static String getAttributeValue(Node n,String att) {
		try {
			return n.getAttributes().getNamedItem(att).getNodeValue();
		} catch (Exception ex) {}
		return null;
	}
	
	public static String max(NodeList nl) {
		double max = 0;
		if (nl==null) return ""+max;
		for (int k=0; k<nl.getLength(); k++) {
			Node n = nl.item(k);
			double s = Double.parseDouble(n.getNodeValue());
			if (s>max) max = s;
		}
		return ""+max;
	}
	
	public static void mezclaDoc(ClassLoader cl,Document xml,String ficheroXsl,Hashtable h,OutputStream out) throws ManagerException {
		
		try {
			TransformerFactory tfactory = SAXTransformerFactory.newInstance();
			tfactory.setURIResolver(new ManagerResolver(cl));
			
			SAXSource xslSource = new SAXSource(new InputSource(cl.getResource(ficheroXsl).openStream()));
			Transformer transformer = tfactory.newTransformer(xslSource);
			transformer.setOutputProperty(OutputKeys.ENCODING,ENCODING);
			if (h!=null) {
				Enumeration en = h.keys();
				while (en.hasMoreElements()) {
					String key = (String) en.nextElement();
					transformer.setParameter(key,h.get(key));
				}
			}
			DOMSource xmlSource = new DOMSource(xml);
			StreamResult result = new StreamResult(out);
			transformer.transform(xmlSource,result);
			
		} catch (Exception ex) {
			throw new ManagerException("XMLUtil","El fichero xsl ["+ficheroXsl+"] no existe, o contiene errores.",ex);
		}			

	}

	public static void mezclaFop(ClassLoader cl,Document xml,String ficheroXsl,Hashtable h,HttpServletResponse response,String contentType,String impFileName) throws ManagerException {
		
		try {
			TransformerFactory tfactory = SAXTransformerFactory.newInstance();
			tfactory.setURIResolver(new ManagerResolver(cl));
			
			SAXSource xslSource = new SAXSource(new InputSource(cl.getResource(ficheroXsl).openStream()));
			Transformer transformer = tfactory.newTransformer(xslSource);
			transformer.setOutputProperty(OutputKeys.ENCODING,ENCODING);
			if (h!=null) {
				Enumeration en = h.keys();
				while (en.hasMoreElements()) {
					String key = (String) en.nextElement();
					transformer.setParameter(key,h.get(key));
				}
			}
			DOMSource xmlSource = new DOMSource(xml);
			DOMResult result = new DOMResult();
			transformer.transform(xmlSource,result);
			
			renderFop((Document)result.getNode(),response,contentType,impFileName);
			
		} catch (Exception ex) {
			System.out.println(""+ex);
			throw new ManagerException("XMLUtil","El fichero xsl ["+ficheroXsl+"] no existe, o contiene errores.",ex);
		}			

	}

	public static void renderFop(Document doc,HttpServletResponse response,String contentType,String impFileName) throws ManagerException {
		
		try {
			
			int render = Driver.RENDER_PDF;
			String contentDisposition = null;
			if (contentType.equals("application/pdf")) {
				render = Driver.RENDER_PDF;
				contentDisposition = "attachment; filename="+impFileName+".pdf";
			} else if (contentType.equals("application/postscript")) {
				render = Driver.RENDER_PS;
				contentDisposition = "attachment; filename="+impFileName+".ps";
			} else if (contentType.equals("image/svg")) {
				render = Driver.RENDER_SVG;
				contentDisposition = "attachment; filename="+impFileName+".svg";
			} else if (contentType.equals("text/plain")) {
				render = Driver.RENDER_TXT;
				contentDisposition = "attachment; filename="+impFileName+".txt";
			}
			
			if (contentDisposition!=null) response.setHeader("Content-Disposition", contentDisposition);
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			printDocument(doc,out);
			//printDocument(doc,System.out);
			Driver driver = new Driver();
			driver.setLogger(new ConsoleLogger(ConsoleLogger.LEVEL_DISABLED));
			driver.setRenderer(render);
			driver.setInputSource(new InputSource(new ByteArrayInputStream(out.toByteArray())));
			driver.setOutputStream(response.getOutputStream());
			//driver.run();
			driver.render(doc);
			
		} catch (Exception ex) {
			throw new ManagerException("XMLUtil","Error en renderizado.",ex);
		}			

	}

	// Obtener Mapping
	public static Mapping getMapping(ClassLoader cl,ManagerResolver mngRes,String module) throws Exception {
		try {
			Mapping m = new Mapping(cl);
			m.setEntityResolver(mngRes);
			m.loadMapping(ManagerReader.getMappingResource(cl,module));
			return m;
		} catch (Exception ex) { 
			throw ex; 
		}
	}

	// Obtener Objeto de un XML
	
	public static Object getObjectFromXML(ClassLoader cl,ManagerResolver mngRes,String module,Node nod) throws ManagerException {
		// Convierte a Objeto un XML
		Object obj = null;
		
		try {
			
    		Unmarshaller unmarshaller = new Unmarshaller();
    		unmarshaller.setMapping(getMapping(cl,mngRes,module));
    		unmarshaller.setClassLoader(cl);
    		obj = unmarshaller.unmarshal(nod);
    		
		} catch (Exception e) {
			throw new ManagerException("XMLUtil","Error en la Obtencin de Datos del Menu: "+e,e);
		}
    		
		return obj;
	}
	
	public static void printDocument(Document doc,OutputStream out) throws Exception {
		TransformerFactory xformerFactory = TransformerFactory.newInstance();
		Transformer xformer = xformerFactory.newTransformer();
		xformer.setOutputProperty(OutputKeys.INDENT,"yes");
		xformer.setOutputProperty(OutputKeys.ENCODING,ENCODING);
		xformer.setOutputProperty(OutputKeys.METHOD,"xml");
		StreamResult output = new StreamResult(out);
		DOMSource source = new DOMSource(doc);
		xformer.transform(source,output);
	}

	public static void saveDocument(Document doc,OutputStream out) throws Exception {
		OutputFormat format = new OutputFormat(doc);
	    format.setLineWidth(65);
	    format.setIndenting(true);
	    format.setIndent(2);
	    format.setEncoding(ENCODING);
	    format.setMethod("xml");
	    StringWriter sw = new StringWriter();
	    XMLSerializer serializer = new XMLSerializer(sw,format);
	    serializer.serialize(doc);
	    String xmlString = sw.toString();
	    int xmli = xmlString.indexOf("<?xml");
	    if (xmli>=0) {
	    	// Tiene una cabecera
	    	int xmld = xmlString.indexOf("<?xml",xmli+1);
	    	if (xmld>0) {
	    		// Tiene dos cabeceras !!!
	    		// Puto OAS !!
	    		// Quito una cabecera, la segunda !!
	    		int fin = xmlString.indexOf("?>",xmld);
	    		xmlString = xmlString.substring(0,xmld)+xmlString.substring(fin+2);
	    	}
	    }
		PrintStream o = new PrintStream(out);
		o.print(xmlString);
		o.flush(); o.close();
	}
	
}


