package framewebsample.jar;

import java.io.*;
import java.util.*;
import java.util.jar.*;
/**
 * @author Universidad de Murcia
 *
 * Crea un esqueleto para una nueva aplicacin.
 * 
 */
public class Main {

	public static String WEB_INF = "WEB-INF";
	public static String CLASSES = "WEB-INF/classes";
	public static String LIB = "WEB-INF/lib";
	public static String SRC = "WEB-INF/src";
	public static String USER_PKG = "user";
	public static String SAMPLE_PKG = "sample";
	public static String SAMPLE_XSL = "html";
	
	public static void main(String[] args) {
		try {
			if (args.length!=1) {
				System.out.println("Use: framewebsample.jar.Main <Project Rute>");
			} else {
				Main m = new Main();
				m.creaJarFile(args[0]);
			}
		} catch (Exception ex) {
			System.out.println("ERROR: "+ex.getMessage());
		}
	}
	
	public void creaJarFile(String ruta) throws Exception {
		
		System.out.println("Creando Jar File ["+ruta+"] ...");
		
		// Ruta de lectura
		if (ruta.endsWith(File.separator)) ruta = ruta.substring(0,ruta.length()-1);
		// Nombre del paquete
		String paqueteBase = ruta.substring(ruta.lastIndexOf(File.separator)+1).toLowerCase();
		String jarName = ruta.substring(0,ruta.lastIndexOf(File.separator)+1)+"frameWeb2.0.jar";
		int release = 0;
		File fj = new File(jarName);
		if (fj.exists()) {
			JarFile fjar = new JarFile(fj);
			Manifest m = fjar.getManifest();
			String ss = m.getMainAttributes().getValue("Release");
			release = Integer.parseInt(ss)+1;
		}
		// Abro el directorio a compactar
		File base = new File(ruta);
		// Abro el jar destino.
		String manifest = "Manifest-Version: 1.0\n";
		manifest += "Created-By: FrameWebSample\n";
		manifest += "Date: "+(new java.util.Date())+"\n";
		manifest += "FrameWeb-Version: 2.0\n";
		manifest += "Release: "+release+"\n";
		JarOutputStream out = new JarOutputStream(new BufferedOutputStream(new FileOutputStream(jarName)),new Manifest(new StringBufferInputStream(manifest)));
		comprime(null,base,out);
		out.close();
		
	}
	
	public void comprime(String ruta,File org,JarOutputStream out) throws IOException {
		if (org.isDirectory()) {
			if (org.getName().startsWith(".")) return;
			if (org.getName().endsWith("src")) return;
			if (org.getName().endsWith("docs")) return;
			if (ruta==null) System.out.println("Packing Directory "+org.getName()+"...");
			else System.out.println("Packing Directory "+ruta+org.getName()+"...");
			File [] lista = org.listFiles();
			for (int k=0; k<lista.length; k++) {
				if (ruta==null) comprime("",lista[k],out);
				else comprime(ruta+org.getName()+"/",lista[k],out);
			}
		} else {
			if (org.getName().startsWith(".")) return;
			if (org.getName().endsWith(".java")) return;
			if (org.getName().endsWith(".jar")) return;
			if (ruta==null) System.out.println("Packing File "+org.getName()+"...");
			else System.out.println("Packing File "+ruta+org.getName()+"...");
			JarEntry jarAdd = new JarEntry(ruta+org.getName());
			jarAdd.setTime(org.lastModified());
			out.putNextEntry(jarAdd);
			// Write file to archive
			FileInputStream in = new FileInputStream(org);
			byte [] bites = new byte[1024];
			int n = in.read(bites,0,1024);
			while (n==1024) {
				out.write(bites,0,n);
				n = in.read(bites,0,1024);
			}
			if (n>0) out.write(bites,0,n);
			in.close();
			out.closeEntry();
		}
		
	}
	
	
}
