package framewebsample.skeleton;

import java.io.*;
/**
 * @author Universidad de Murcia
 *
 * Crea un esqueleto para una nueva aplicacin.
 * 
 */
public class Main {

	public static String WEB_INF = "WEB-INF";
	public static String CLASSES = "WEB-INF/classes";
	public static String LIB = "WEB-INF/lib";
	public static String SRC = "WEB-INF/src";
	public static String USER_PKG = "user";
	public static String SAMPLE_PKG = "sample";
	public static String SAMPLE_XSL = "html";
	
	public static void main(String[] args) {
		try {
			if (args.length!=1) {
				System.out.println("Use: framewebsample.skeleton.Main <Project Rute>");
			} else {
				Main m = new Main();
				m.creaEsqueleto(args[0]);
			}
		} catch (Exception ex) {
			System.out.println("ERROR: "+ex.getMessage());
		}
	}
	
	public void creaEsqueleto(String ruta) throws Exception {
		
		System.out.println("Creando Esqueleto FrameWeb ["+ruta+"] ...");
		
		File base = new File(ruta);
		if (!base.exists() || !base.isDirectory()) throw new Exception("El directorio "+ruta+" no existe o no es un directorio.");
		File webinf = new File(ruta+File.separator+WEB_INF);
		if (!webinf.exists()) throw new Exception("El directorio "+ruta+" no tiene estructura J2EE. Carece de Carpeta WEB-INF.");
		File classes = new File(ruta+File.separator+CLASSES);
		if (!classes.exists()) throw new Exception("El directorio "+ruta+" no tiene estructura J2EE. Carece de Carpeta WEB-INF/classes.");
		File lib = new File(ruta+File.separator+LIB);
		if (!lib.exists()) throw new Exception("El directorio "+ruta+" no tiene estructura J2EE. Carece de Carpeta WEB-INF/lib.");
		File src = new File(ruta+File.separator+SRC);
		if (!src.exists()) throw new Exception("El directorio "+ruta+" no tiene estructura J2EE. Carece de Carpeta WEB-INF/src.");
		
		if (ruta.endsWith(File.separator)) ruta = ruta.substring(0,ruta.length()-1);
		String paqueteBase = ruta.substring(ruta.lastIndexOf(File.separator)+1).toLowerCase();
		
		// El directorio escogido parece adecuado.
		String namePrj = ruta.substring(ruta.lastIndexOf(File.separator)+1);
		String baseOrigen = getClass().getResource("../../").getFile();
		baseOrigen = baseOrigen.substring(0,baseOrigen.lastIndexOf("/"));
		baseOrigen = baseOrigen.substring(0,baseOrigen.lastIndexOf("/"))+"/";
		
		// Crea los ficheros lib
		String rutaBase = ruta+File.separator+LIB+File.separator;
		makeLibFiles(baseOrigen,rutaBase);
		
		// Crea los ficheros html
		rutaBase = ruta+File.separator;
		makeHtmlFiles(baseOrigen,namePrj,rutaBase);
		
		// Crea un paquete base y dentro el servlet de control.
		rutaBase = ruta+File.separator+SRC+File.separator+paqueteBase+File.separator;
		File fileSrc = new File(rutaBase);
		if (!fileSrc.exists()) fileSrc.mkdir();
		makeGeneralFiles(baseOrigen,namePrj,rutaBase);
		
		// Crea el paquete de usuario.
		rutaBase = ruta+File.separator+SRC+File.separator+paqueteBase+File.separator+USER_PKG+File.separator;
		fileSrc = new File(rutaBase);
		if (!fileSrc.exists()) fileSrc.mkdir(); 
		makeUserFiles(baseOrigen,namePrj,rutaBase);
		
		// Crea el paquete de inicio o ejemplo.
		rutaBase = ruta+File.separator+SRC+File.separator+paqueteBase+File.separator+SAMPLE_PKG+File.separator;
		fileSrc = new File(rutaBase);
		if (!fileSrc.exists()) fileSrc.mkdir(); 
		makeSampleFiles(baseOrigen,namePrj,rutaBase);
		
		rutaBase = ruta+File.separator+SRC+File.separator+paqueteBase+File.separator+SAMPLE_PKG+File.separator+SAMPLE_XSL+File.separator;
		fileSrc = new File(rutaBase);
		if (!fileSrc.exists()) fileSrc.mkdir(); 
		makeSampleXslFiles(baseOrigen,namePrj,rutaBase);
	}
	
	public void copyFile(File a,File b) throws Exception {
		FileInputStream in = new FileInputStream(a);
		FileOutputStream out = new FileOutputStream(b);
		byte [] bytes = new byte[1024];
		int n = in.read(bytes,0,1024);
		while (n>0) {
			out.write(bytes,0,n);
			n = in.read(bytes,0,1024);
		}
		in.close();
		out.close();
	}

	public void replicaFile(File a,File b,String name) throws Exception {
		DataInputStream in = new DataInputStream(new FileInputStream(a));
		PrintStream out = new PrintStream(new FileOutputStream(b));
		String linea = in.readLine();
		while (linea!=null) {
			linea = linea.replaceAll("FrameWebSample",name);
			linea = linea.replaceAll("framewebsample",name.toLowerCase());
			linea = linea.replaceAll("FrameWeb",name);
			linea = linea.replaceAll("../../../../../"+name+"Core/atica","../../../../../FrameWebCore/atica");
			out.println(linea);
			linea = in.readLine();
		}
		in.close();
		out.close();
	}

	public void makeLibFiles(String base,String ruta) throws Exception {
		System.out.println("Copiando libreras...");
		File origen = new File(base+LIB);
		File [] lista = origen.listFiles();
		if (lista!=null) {
			for (int k=0; k<lista.length; k++) {
				if (lista[k].getName().endsWith(".jar")) 
					copyFile(lista[k],new File(ruta+lista[k].getName()));
			}
		}
	}

	public boolean isWebFile(String name) {
		return (name.endsWith(".html") || name.endsWith(".css") || name.endsWith(".jpg"));
	}
	
	public void makeHtmlFiles(String base,String name,String ruta) throws Exception {
		System.out.println("Copiando ficheros html...");
		File origen = new File(base);
		File [] lista = origen.listFiles();
		if (lista!=null) {
			for (int k=0; k<lista.length; k++) {
				if (lista[k].isFile() && isWebFile(lista[k].getName().toLowerCase())) 
					replicaFile(lista[k],new File(ruta+lista[k].getName()),name);
			}
		}
	}

	public void makeGeneralFiles(String base,String name,String ruta) throws Exception {
		System.out.println("Copiando generales...");
		File origen = new File(base+SRC+"/framewebsample/");
		File [] lista = origen.listFiles();
		if (lista!=null) {
			for (int k=0; k<lista.length; k++) {
				if (lista[k].isFile()) {
					String nuevoName = ruta+lista[k].getName();
					nuevoName = nuevoName.replaceAll("FrameWeb",name);
					//nuevoName = nuevoName.replaceAll(".txt",".java");
					replicaFile(lista[k],new File(nuevoName),name);
				}
			}
		}
	}
	
	public void makeUserFiles(String base,String name,String ruta) throws Exception {
		System.out.println("Copiando clase usuario...");
		File origen = new File(base+SRC+"/framewebsample/"+USER_PKG);
		File [] lista = origen.listFiles();
		if (lista!=null) {
			for (int k=0; k<lista.length; k++) {
				if (lista[k].isFile()) {
					String nuevoName = ruta+lista[k].getName();
					nuevoName = nuevoName.replaceAll("FrameWeb",name);
					//nuevoName = nuevoName.replaceAll(".txt",".java");
					replicaFile(lista[k],new File(nuevoName),name);
				}
			}
		}
	}

	public void makeSampleFiles(String base,String name,String ruta) throws Exception {
		System.out.println("Copiando mdulo de ejemplo...");
		File origen = new File(base+SRC+"/framewebsample/"+SAMPLE_PKG);
		File [] lista = origen.listFiles();
		if (lista!=null) {
			for (int k=0; k<lista.length; k++) {
				if (lista[k].isFile()) {
					String nuevoName = ruta+lista[k].getName();
					nuevoName = nuevoName.replaceAll("FrameWeb",name);
					//nuevoName = nuevoName.replaceAll(".txt",".java");
					replicaFile(lista[k],new File(nuevoName),name);
				}
			}
		}
	}

	public void makeSampleXslFiles(String base,String name,String ruta) throws Exception {
		System.out.println("Copiando salida xsl...");
		File origen = new File(base+SRC+"/framewebsample/"+SAMPLE_PKG+"/"+SAMPLE_XSL);
		File [] lista = origen.listFiles();
		if (lista!=null) {
			for (int k=0; k<lista.length; k++) {
				if (lista[k].isFile()) 
					replicaFile(lista[k],new File(ruta+lista[k].getName()),name);
			}
		}
	}
}
